/******************************************************************************
*
* Utility for comparing data in Informix tables
*
* Author: Robert Sosnowski, Robert.Sosnowski@bzwbk.pl
*
* Version 1.70, 17-10-2005
*
* Copyright (c) 2001 - 2007. All Rights Reserved.
*
* Permission granted to use & distribute for free.
*
* This software can be used only at your own risk. Author is not responsible 
* for any damage caused by using this software.
* 
******************************************************************************/

#include "load2cfg.h"

EXEC SQL include sqltypes;

#ifndef HAVE_SNPRINTF
#ifdef HAVE__SNPRINTF
#define snprintf _snprintf
#else
#include <stdarg.h>
int snprintf(char *str, size_t count, ...)
{int i;
 va_list args;
 i= sprintf(str, args);
 va_end(args);
return i;
} 
#endif
#endif 

#ifndef HAVE_GETOPT
extern int      optind;
extern char    *optarg;
extern int	opterr;
int getopt(int argc, char *argv[], char *optionS);
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "load2.h"
#include "unload2.h"
#include <time.h>
#include "tempfile.h"

#define MAX_FIELDNAME TBLNAME_SIZE+1

static char *id="@(#) $Id: dbdiff.ec 2521 2007-11-21 08:55:56Z sosnor01 $";

int usage( char * );
#ifdef HAVE_ZLIB
int oninitgz(const char *fname, const char *tablename, const int batchsize,
               void **handle, const void *logfile, const int fieldsnum,
               const char *insstmt2,
               onlogerror_lpt onlogerror, int verbose,
               const char *datahandleoptions);
int onreadfieldgz(const void *fd, const int sqltype, const int4 sqlxid,
                  const char *filename,
                  char **dst, const unsigned int maxdstbuf, const char delim,
                  const char escapech, int * eol, int * sqlind,
                  const unsigned long int rownum,
                  const unsigned long int loaded, const int fieldnum,
                  const char * fieldname, const int max_fld_name,
                  char *fieldname2, const void *logfile,
                  onlogerror_lpt onlogerror, const int verbose,
                  const char *datahandleoptions);
int oncrlfgz(const void *fd, const char *filename,
             const unsigned long int rownum, const unsigned long int loaded,
             const int fields_number, const int bequiet, const char escapech,
             const void *logfile,
             onlogerror_lpt onlogerror, int verbose,
             const char *datahandleoptions);
void oncleanupgz(const char *fname, void *handle, const void *logfile,
                 onlogerror_lpt onlogerror, int verbose,
                 const char *datahandleoptions);
#endif

int dbdiff (const char * slctstmt2,
            const char *ifname, const char *ofname,
            const int batchsize, unsigned long int *unloaded_rows,
            const void *logfile, const char *collist,
            const int verbosity, const char *tablename,
            const short gzcompr);
int loader_dup(unl_handle_t *unl_handle, loader_handle_t *loader_handle,
               unl_handle_t *unl_handle2);
int compare_keys(unl_handle_t *unl_handle, loader_handle_t *loader_handle);
int same_records(unl_handle_t *unl_handle, loader_handle_t *loader_handle);
int putrec1(int flag, unl_handle_t *unl_handle, unl_handle_t *unl_handle2);
int putrec2(int flag, loader_handle_t *loader_handle, unl_handle_t *unl_handle2);
int colcmp(struct sqlvar_struct *col1, struct sqlvar_struct *col2, unl_handle_t *unl_handle);
int writerec(unl_handle_t *unl_handle);
int reverse_comparison;
int stringcmp(char *s1, char *s2);
int blob_copy(loc_t *unl_blob, loc_t *load_blob, unsigned long int rownum,
          unsigned long int loaded, const void *logfile,
          onlogerror_upt onlogerror, int verbose);
int blob_copy2(loc_t *load_blob, loc_t *unl_blob, unsigned long int rownum,
          unsigned long int loaded, const void *logfile,
          onlogerror_upt onlogerror, int verbose);
int sblob_copy(ifx_lo_t *unl_blob, ifx_lo_t *load_blob, unsigned long int rownum,
               unsigned long int loaded, const void *logfile,
               onlogerror_upt onlogerror, int verbose);

int main( int argc, char **argv )
{

  EXEC SQL BEGIN DECLARE SECTION;
  static char dbname[TBLNAME_SIZE];
  static char server[TBLNAME_SIZE];
  static char server2[TBLNAME_SIZE+1];
  static char user[USERNAME_SIZE];
  static char password[USERNAME_SIZE];
  static char tabname[TBLNAME_SIZE], tabalias[TBLNAME_SIZE];
  EXEC SQL END DECLARE SECTION;

  static char  qstr[5000];
  static char  sqlfilename[128];
  static char  logfilename[128];
  static char  infile[128];
  static char  outfile[128];
  short verbosity, use_ins_curs, use_commit, benchm, gzcompr;
  time_t t1, t2;
  int batchsize, maxerrors, argind;
  unsigned long int loaded_rows;
  int ch;
  FILE * fsqlfile, *logfile;
  int i;
  char isolation[3];
  static char  keycols[500];
  char *p;

  verbosity=3; /* show errors and progress messages */
  use_ins_curs=0;
  logfilename[0]=0;
  sqlfilename[0]=0;
  batchsize=10000;
  server[0]=0;
  user[0]=0;
  password[0]=0;
  maxerrors=0;
  logfile=NULL;
  benchm=0;
  gzcompr=0;
  isolation[0]=0;
  strcpy(tabalias, "<table>");
  keycols[0]=0;
  infile[0]=0;
  outfile[0]=0;
  reverse_comparison=0;

  setvbuf(stdout,NULL,_IONBF,0);
  if ( argc == 1 ) {
    usage( argv[0] );
    exit( -1 );
  }
    

#ifdef HAVE_ZLIB
  while ((ch = getopt( argc, argv, "Vd:s:u:p:e:t:c:n:k:i:v:l:Bga:o:r" )) != EOF) {
#else
  while ((ch = getopt( argc, argv, "Vd:s:u:p:e:t:c:n:k:i:v:l:Ba:o:r" )) != EOF) {
#endif
    switch (ch) {
      case 'V':
        fprintf( stdout,
         "\ndbdiff ver 1.70, Copyright 2001 - 2007 Robert Sosnowski\n\n" );
        exit( 0 );
        break;
      case 'd':
        /* database name */
        strcpy( dbname, optarg );
        break;
      case 's':
        /* informix server */
        strcpy( server, optarg );
        break;
      case 'u':
        /* user */
        strcpy( user, optarg );
        break;
      case 'p':
        /* password */
        strcpy( password, optarg );
        break;
      case 'e': 
        /* SQL statement */
        strcpy( qstr, optarg );
        break;
      case 't': 
        /* table name */
        strcpy( tabname, optarg );
        break;
      case 'c': 
        /* file with SQL statement */
        strcpy( sqlfilename, optarg );
        break;
      case 'n':   
        /* batchsize */
        batchsize=(int)atoi( optarg );
        break;
      case 'k':
        /* key definition */
        strcpy(keycols, optarg);
        break;
      case 'o':
        /* name of input file */
        strcpy(outfile, optarg);
        break;
      case 'i':
        /* isolation level */
        strncpy( isolation, optarg, 1 );
	isolation[1]=0;
        break;
      case 'l':
        /* log file */
        strcpy( logfilename, optarg );
        break;
      case 'v':
        /* verbosity */
        verbosity=(int)atoi( optarg );
        break;
      case 'B':
        /* Benchmark mode */
	benchm=1;
        break;
      case 'g':
        /* gzipped */
        gzcompr=1;
        break;
      case 'a':
        /* Table alias for progress messages */
        strcpy( tabalias, optarg);
        break;
      case 'r':
        /* Reverse diff */
        reverse_comparison=1;
        break;

      default:
        usage( argv[0] );
        exit(2);
    }
  }

  if (strlen( dbname ) == 0) {
    fprintf( stdout, "Must supply database (-d)!\n" );
    usage( argv[0] );
    exit(2);
  }

  if (keycols[0]==0) {
    fprintf( stdout, "List of key columns must be supplied with -k option!\n" );
    usage( argv[0] );
    exit(2);
  }
  FetBufSize=30000;
    
  EXEC SQL WHENEVER ERROR CONTINUE;

  if (strlen(server)==0 && (p = strstr(dbname, "@")))
  {
    p[0] = '\0';
    strcpy(server, p+1);
  }

  if (strlen(server)!=0 || strlen(user)!=0 || strlen(password)!=0) {
    if (strlen(server)==0) {
      fprintf( stderr, "Informix server should be specified with -s option\n");
      return 1;
    }
    strcpy(server2,"@");
    strcat(server2, server);
    if ( strlen(user)==0 && strlen(password)==0) {
      EXEC SQL CONNECT TO :server2 AS "conn1" WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting server %s:"INT4_FORMAT"\n", server,
	         sqlca.sqlcode);
	return 1;
      }
    } else {
      if ( strlen(user)==0 || strlen(password)==0) {
    	fprintf( stderr, "Both options: -u and -p should be provided\n");
        return 1;
      }
      EXEC SQL CONNECT TO :server2 AS "conn1" USER :user USING :password
               WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting server %s as user %s:"INT4_FORMAT"\n",
	         server, user, sqlca.sqlcode);
        return 1;
      }
    }
  } else {
    EXEC SQL CONNECT TO DEFAULT WITH CONCURRENT TRANSACTION;
    if (sqlca.sqlcode<0) {
      fprintf( stderr, "Error while connecting default server: "INT4_FORMAT"\n",
               sqlca.sqlcode);
      return 1;
    }
  }
  EXEC SQL DATABASE :dbname;
  if (sqlca.sqlcode < 0) {
    fprintf( stderr, "Cannot open database.  Code: "INT4_FORMAT", ISAM: "INT4_FORMAT".\n",
             sqlca.sqlcode, sqlca.sqlerrd[1] );
    return 1;
  }
  use_commit = 0;

  if (strlen( qstr ) != 0 ) {
    if (strlen(tabname)!=0) {
      fprintf(stderr, "You can't specify both -e and -t options!\n");
      return 1;
    }
    if (strlen(sqlfilename)!=0) {
      fprintf(stderr, "You can't specify both -e and -c options!\n");
      return 1;
    }
  } else {
    if (strlen(tabname) != 0) {
      if (strlen(sqlfilename)!=0) {
        fprintf(stderr, "You can't specify both -t and -c options!\n");
        return 1;
      }
      sprintf(qstr, "select * from %s order by %s", tabname, keycols);
    } else {
      if (strlen(sqlfilename)!=0) {
        fsqlfile=fopen(sqlfilename, "r");
        if (fsqlfile) {
          i=fread(qstr, 1, sizeof(qstr), fsqlfile);
          qstr[i]=0;
          fclose(fsqlfile);
        } else {
          perror("Can't open sql file");
          return 1;
        }
      } else {
        fprintf(stderr, "You have to specify one of: -e -t or -c option!\n");
        return 1;
      }
    }
  }

  if (verbosity<0 || verbosity>4) {
    fprintf(stderr, "Wrong verbosity value\n");
    return 1;
  }
  if (gzcompr<0 || gzcompr>9) {
    fprintf(stderr, "Wrong gz compression value\n");
    return 1;
  }
  if (logfilename[0]) {
    logfile=fopen(logfilename,"a");
#ifdef _IONBF
    if (logfile!=NULL) {
      setvbuf(logfile, NULL, _IONBF,0);
    }
#endif
  }

  argind=optind;
  do {
    if (argind<argc) {
      strcpy(infile, argv[argind]);
    }
    if (benchm) {
      t1=time(NULL);
      if (logfilename[0]) {
        fprintf(logfile, "Loading started at %s",ctime(&t1));
      }
      fprintf(stdout, "Loading started at %s",ctime(&t1));
    }
    if (strcmp(tabalias,"<table>")) {
      strcpy(tabname, tabalias);
    }
    if (dbdiff (qstr, infile, outfile, batchsize, &loaded_rows,
            logfile, keycols, verbosity, tabname, gzcompr)!=0) {
      return 1;
    }
    if (benchm) {
      t2=time(NULL);
      i=t2-t1;
      if (logfilename[0]) {
        fprintf(logfile, "Loading finished at %s",ctime(&t2));
        fprintf(logfile, "Elapsed seconds: %d\n",i);
      }
      fprintf(stdout, "Loading finished at %s",ctime(&t2));
      fprintf(stdout, "Elapsed seconds: %d\n",i);
    }
    break; /* currently only 1 input file can be specified for dbdiff */
  } while (++argind <argc);
    
  if (logfilename[0]) {
    fclose(logfile);
  }

  return 0;
}


int usage( char *prog )
{


  fprintf( stdout, "\n");
  fprintf( stdout, "%s - compare data in *.unl file with table.\n\n", prog );

  fprintf( stdout, "Usage: \n" );
#ifdef HAVE_ZLIB
  fprintf( stdout, 
           "\t%s [-V] -d <db_name> [-s <server> -u <user> -p <password>]\n "
           "(-e <select stmt>|-t <table>|-c <sql file>) [-n <batch size>]\n "
           "-k <key columns> [-o output file] [-r]\n "
           "[-i <# bad rows>] [-l <logfile>] [-B] [-g] [-v <verbosity>] \n "
           "<unload file (input)>\n\n", prog);
#else
  fprintf( stdout, 
           "\t%s [-V] -d <db_name> [-s <server> -u <user> -p <password>]\n "
           "(-e <select stmt>|-t <table>|-c <sql file>) [-n <batch size>]\n "
           "-k <key columns> [-o output file] [-r]\n "
           "[-i <# bad rows>] [-l <logfile>] [-B] [-v <verbosity>]\n "
           "<unload file (input)>\n\n", prog);
#endif

  fprintf( stdout, "\t\t-V - version information\n" ); 
  fprintf( stdout, "\t\t-d - database name\n" );
  fprintf( stdout, "\t\t-s - informix server name\n" );
  fprintf( stdout, "\t\t-u - user\n" );
  fprintf( stdout, "\t\t-p - password\n" );
  fprintf( stdout, "\t\t-e - SQL insert statement in quotes\n" );
  fprintf( stdout, "\t\t-t - table name\n" );
  fprintf( stdout, "\t\t-c - file containing SQL insert statement\n" );
  fprintf( stdout, "\t\t-n - batch size\n" );
  fprintf( stdout, "\t\t-k - list of key column names or column numbers\n" );
  fprintf( stdout, "\t\t-o - name of output file; if not specified stdout is used\n" );
  fprintf( stdout, "\t\t-r - reverse comparison\n" );
  fprintf( stdout, "\t\t-i - isolation level\n" );
  fprintf( stdout, "\t\t   0 - dirty read (default)\n" );
  fprintf( stdout, "\t\t   1 - commited read\n" );
  fprintf( stdout, "\t\t   2 - cursor stability\n" );
  fprintf( stdout, "\t\t   3 - repeatable read\n" );
  fprintf( stdout, "\t\t-l - log file name\n" );
  fprintf( stdout, "\t\t-B - benchmark\n" );
#ifdef HAVE_ZLIB
  fprintf( stdout, "\t\t-g - gzipped unload\n" );
#endif
  fprintf( stdout, "\t\t-v - verbosity:\n" );
  fprintf( stdout, "\t\t     0 - completely quiet\n" );
  fprintf( stdout, "\t\t     1 - show errors only\n" );
  fprintf( stdout, "\t\t     2 - summary messages\n" );
  fprintf( stdout, "\t\t     3 - detailed messages (default)\n" );
  fprintf( stdout, "\t\t     4 - debug info\n" );
  fprintf( stdout, "\t\t-a - table alias for progress messages\n" );

  return 0;
} 

int unl_open2(unl_handle_t *unl_handle, short gzcompr)
{
  unl_handle_t *h;
  char escapech;
  void *h1;
/*  EXEC SQL BEGIN DECLARE SECTION;
    char stmt_id[128];
    char curs_id[128];
  EXEC SQL END DECLARE SECTION;
*/
  h=unl_handle;
/*  sprintf(stmt_id,"ustmt_id%ld",h->instance_id);
  sprintf(curs_id,"ucurs_id%ld",h->instance_id); 

#ifdef DEBUG_ALLOC
printf("Declare cursor: %s\n", curs_id);
#endif
  EXEC SQL declare :curs_id cursor with hold for :stmt_id;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(h->errstr,"Cannot declare cursor: %ld:%ld:%s\n",sqlca.sqlcode,
            sqlca.sqlerrd[1], sqlca.sqlerrm);
    h->onlogerror(__FILE__, __LINE__, h->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, h->rownum, 0, 0, h->logfile, h->verbosity, 10);
    cleanup(h);
    return 1;
  }
 
  EXEC SQL open :curs_id;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(h->errstr,"Cannot open cursor: %ld:%ld:%s\n",sqlca.sqlcode,
            sqlca.sqlerrd[1], sqlca.sqlerrm);
    h->onlogerror(__FILE__, __LINE__, h->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, h->rownum, 0, 0, h->logfile, h->verbosity, 10);
    cleanup(h);
    return 1;
  } */
  h->iscursoropen=1;
/*  h->fetarridx=h->fetarrsize; */

  h1=h->datahandle;
  if (h->oninit(h->fname, h->tablename, h->batchsize, &h1, &escapech,
                h->logfile, h->fieldsnum, h->slctstmt2, h->onlogerror,
                h->verbosity, h->datahandleoptions )!=0) {
    exit(1);
  }
  h->escapechar=escapech;
  h->datahandle=h1;
  h->rownum=1;
  h->unloaded_rows=0;
  return 0;
}

int dbdiff (const char * slctstmt2,
            const char *ifname, const char * ofname,
            const int batchsize, unsigned long int *unloaded_rows,
            const void *logfile, const char *collist,
            const int verbosity, const char *tablename,
            const short gzcompr)
{
  unl_handle_t *unl_handle, *unl_handle2;
  int i, k;
  char buffer[32767+1];
  int batchsize2;
  int fieldsnum;
  int all_records1, all_records2;
  unsigned long int bytesread;
  int sqlind;
  loader_handle_t *loader_handle;
  int all_fields;
  char sqlname2[MAX_FIELDNAME]; /* fieldname read from file */
  int emptyread;
  int badrecord;
  unsigned long int loaded_rows, unloaded_rows2;
  int cmp_result;
  int cnt1, cnt2;
    
  buffer[0]=0;

  unl_handle=unl_create(); /* input data from database */
  if (unl_handle==NULL) {
    return 1;
  }
  unl_handle2=unl_create(); /* resulting output data */
  if (unl_handle==NULL) {
    return 1;
  }

  loader_handle=loader_create (); /* input data from stdin */
  if (loader_handle==NULL) {
    return 1;
  }

  k=unl_setstmt(unl_handle, slctstmt2);
  if (k!=0) {
    return 1;
  }

  k=unl_setstmt(unl_handle2, slctstmt2);
  if (k!=0) {
    return 1;
  }

  k=loader_setstmt(loader_handle, "insert into nonexistent values(?)" );
  if (k!=0) {
    return 1;
  }

  k=unl_setdatahandlename(unl_handle, ""); /* file name not used */
  if (k!=0) {
    return 1;
  }

  k=unl_setdatahandlename(unl_handle2, ofname);
  if (k!=0) {
    return 1;
  }

  k=loader_setdatahandlename(loader_handle, ifname);
  if (k!=0) {
    return 1;
  }

  if (batchsize<0) {
    batchsize2=0;
  } else {
    batchsize2=batchsize;
  }
  unl_setbatchsize(unl_handle, batchsize2);
  unl_setbatchsize(unl_handle2, batchsize2);
  loader_setbatchsize(loader_handle, batchsize2);


  if (logfile==NULL) {
    unl_setloghandle(unl_handle, NULL);
    unl_setloghandle(unl_handle2, NULL);
    loader_setloghandle(loader_handle, NULL);
  } else {
    unl_setloghandle(unl_handle, logfile);
    unl_setloghandle(unl_handle2, logfile);
    loader_setloghandle(loader_handle, logfile);
  }

  switch (verbosity) { 
    case 0: 
            unl_setverbosity(unl_handle, 0);
            unl_setverbosity(unl_handle2, 0);
            loader_setverbosity(loader_handle, 0);
            break;
    case 1: 
            unl_setverbosity(unl_handle, 20);
            unl_setverbosity(unl_handle2, 20);
            loader_setverbosity(loader_handle, 20);
            break;
    case 2: 
            unl_setverbosity(unl_handle, 30);
            unl_setverbosity(unl_handle2, 30);
            loader_setverbosity(loader_handle, 30);
            break;
    case 3: 
            unl_setverbosity(unl_handle, 40);
            unl_setverbosity(unl_handle2, 40);
            loader_setverbosity(loader_handle, 40);
            break;
    case 4: 
            unl_setverbosity(unl_handle, 50);
            unl_setverbosity(unl_handle2, 50);
            loader_setverbosity(loader_handle, 50);
            break;
    default: 
            unl_setverbosity(unl_handle, 30);
            unl_setverbosity(unl_handle2, 30);
            loader_setverbosity(loader_handle, 30);
            break;
  }
#ifdef HAVE_ZLIB
  if (gzcompr) {
    loader_setoninit(loader_handle, &oninitgz);
    loader_setonreadfield(loader_handle, &onreadfieldgz);
    loader_setoncrlf(loader_handle, &oncrlfgz);
    loader_setoncleanup(loader_handle, &oncleanupgz);
  }
#endif
/*
    loader_setoninit(loader_handle, &oninit_std);
    loader_setonbatch(loader_handle, &onbatch_std);
    loader_setonfinish(loader_handle, &onfinish_std);
    loader_setonreadfield(loader_handle, &onreadfield_std);
    loader_setoncleanup(loader_handle, &oncleanup_std);
    loader_setonlogerror(loader_handle, &onlogerror_std);
    loader_setoncrlf(loader_handle, &oncrlf_std);
*/

  unl_settablename(unl_handle, tablename);
  unl_settablename(unl_handle2, tablename);


  *unloaded_rows=0;
  loaded_rows=0;
  unloaded_rows2=0;


  k=unl_prepare(unl_handle);
  if (k!=0) {
    return 1;
  }
  k=unl_open(unl_handle);
  if (k!=0) {
    return 1;
  }
  k=loader_dup(unl_handle, loader_handle, unl_handle2);
  if (k!=0) {
    return 1;
  }
  loader_setkeycols(loader_handle, collist);
  convert_keycols(loader_handle);

  fieldsnum=unl_getfieldsnum(unl_handle);

  i=unl_handle2->verbosity;
  unl_handle2->verbosity=20;
  k=unl_open2(unl_handle2, gzcompr);
  if (k!=0) {
    return 1;
  }
  unl_handle2->verbosity=i;

  i=loader_handle->verbosity;
  loader_handle->verbosity=20; /* to suppress initial messages */
  k=loader_open(loader_handle);
  if (k!=0) {
    return 1;
  }
  loader_handle->verbosity=i;

  all_records1=0;
  all_records2=0;
  cmp_result=0; /* initial state */
  cnt1=0;
  cnt2=0;
  while (1) {
    if (cmp_result==0 || cmp_result==1) {
      /* get record from file */
      i=0; /* current field number */
      while (1) {
        k=loader_readfield(loader_handle, &i,
                           buffer, sizeof(buffer)-1,
                           &bytesread, &all_records2,
                           sqlname2, MAX_FIELDNAME, &emptyread,
                           &all_fields, &badrecord, &sqlind);
       if (k!=0) {
          return 1;
        }
        if (all_records2 && i==0) {
          break;
        }

        k=loader_putfield(loader_handle, i, buffer, bytesread, sqlind,
                          &badrecord);
        if (k!=0) {
          return 1;
        }
        i++;
        if (all_fields) {
          i=0;
          break;
        }
      }
      if (all_records2) {
        break;
      }
      cnt2++;
    }
    if (cmp_result==-1 || cmp_result==0) {
      /* get record from database */
      k=unl_getrecord(unl_handle, &all_records1);
      if (k!=0) {
        return 1;
      }
      if (all_records1) {
        break;
      }
      unl_handle->rownum++;
      cnt1++;
    }

    cmp_result=compare_keys(unl_handle, loader_handle);
    if (reverse_comparison) {
      switch (cmp_result) {
        case 0: if (!same_records(unl_handle, loader_handle)) {
                  putrec1(0, unl_handle, unl_handle2);
                  writerec(unl_handle2);
                };
                break;
        case 1: putrec2(2, loader_handle, unl_handle2);
                writerec(unl_handle2);
                break;
        case -1: putrec1(1, unl_handle, unl_handle2);
                writerec(unl_handle2);
                break;
       }
     } else {
      switch (cmp_result) {
        case 0: if (!same_records(unl_handle, loader_handle)) {
                  putrec2(0, loader_handle, unl_handle2);
                  writerec(unl_handle2);
                };
                break;
        case 1: putrec2(1, loader_handle, unl_handle2);
                writerec(unl_handle2);
                break;
        case -1: putrec1(2, unl_handle, unl_handle2);
                writerec(unl_handle2);
                break;
       }
     }
  }
  while (!all_records2) {
    if (all_records1 && (cmp_result==-1 || cmp_result==0) ) {
      if (reverse_comparison) {
        putrec2(2, loader_handle, unl_handle2);
       } else {
        putrec2(1, loader_handle, unl_handle2);
      }
      writerec(unl_handle2);
    }
    i=0; /* current field number */
    while (1) {
      k=loader_readfield(loader_handle, &i,
                         buffer, sizeof(buffer)-1,
                         &bytesread, &all_records2,
                         sqlname2, MAX_FIELDNAME, &emptyread,
                         &all_fields, &badrecord, &sqlind);
      if (k!=0) {
        return 1;
      }
      if (all_records2 && i==0) {
        break;
      }

      k=loader_putfield(loader_handle, i, buffer, bytesread, sqlind,
                        &badrecord);
      if (k!=0) {
        return 1;
      }
 
      i++;
      if (all_fields) {
        if (reverse_comparison) {
          putrec2(2, loader_handle, unl_handle2);
        } else {
          putrec2(1, loader_handle, unl_handle2);
        }
        writerec(unl_handle2);
        i=0;
      }
    }
    if (all_records2) {
      break;
    }
  }
  while (!all_records1) {
    if (all_records2 && cmp_result==1) {
      if (reverse_comparison) {
        putrec1(1, unl_handle, unl_handle2);
       } else {
        putrec1(2, unl_handle, unl_handle2);
      }
      writerec(unl_handle2);
    }
    k=unl_getrecord(unl_handle, &all_records1);
    if (k!=0) {
      return 1;
    }
    if (all_records1) {
      break;
    }
    unl_handle->rownum++;
    if (reverse_comparison) {
      putrec1(1, unl_handle, unl_handle2);
    } else {
      putrec1(2, unl_handle, unl_handle2);
    }
    writerec(unl_handle2);
  }
  i=loader_handle->verbosity;
  loader_handle->verbosity=20;
  if (loader_close(loader_handle)) {
    return 1;
  }
  loader_handle->verbosity=i;
  loaded_rows=loader_getloadedrows(loader_handle);
  if (loader_free(loader_handle)) { /* = loader_unprepare */
    return 1;
  }
  loader_handle=NULL;

  i=unl_handle->verbosity;
  unl_handle->verbosity=20;
  if (unl_close(unl_handle)) {
    return 1;
  }
  unl_handle->verbosity=i;

  *unloaded_rows=unl_getunloadedrows(unl_handle);
  if (unl_free(unl_handle)) { /* = unloader_unprepare */
    return 1;
  }
  unl_handle=NULL;

  if (unl_close(unl_handle2)) {
    return 1;
  }
  *unloaded_rows=unl_getunloadedrows(unl_handle2);
/*  if (unl_free(unl_handle2)) {*/ /* = unloader_unprepare */ /*
    return 1;
  } */
  unl_handle2=NULL;

  return 0; 
}


int loader_dup(unl_handle_t *unl_handle, loader_handle_t *loader_handle,
               unl_handle_t *unl_handle2)
{
 unl_handle_t *h1, *h3;
 loader_handle_t *h2;
 int i;
#if MI_PTR_SIZE == 64
 mlong pos, cp;
#else
 mint pos, cp;
#endif
 struct sqlvar_struct *col1;
 unsigned long memsize;

 h1=unl_handle;
 h2=loader_handle;
 h3=unl_handle2;

 h2->delim=h1->delim;
 h2->batchsize=h1->batchsize;
 h2->usecursor=0;
 h2->usecommit=0;
 h2->maxerrors=0;
 h2->verbosity=h1->verbosity;
 strcpy(h2->tablename, h1->tablename);
 h2->fieldsnum=h1->fieldsnum;
 h2->indicators=(int *)load2_malloc((h2->fieldsnum)*sizeof(int), __FILE__, __LINE__);
 if ( h2->indicators == NULL ) {
   sprintf(h2->errstr,"Out of memory");
   h2->onlogerror(__FILE__, __LINE__, h2->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
              sqlca.sqlerrm, 0, 0, 0, h2->logfile, h2->verbosity, 10);
   return 1;
 }
 for (i=0;i<h2->fieldsnum;i++) {
   h2->indicators[i]=0;
 }

 h2->nullables=(short *)load2_malloc((h2->fieldsnum) * sizeof(short), __FILE__, __LINE__);
 if ( h2->nullables == NULL ) {
   sprintf(h2->errstr,"Out of memory");
   h2->onlogerror(__FILE__, __LINE__, h2->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
              sqlca.sqlerrm, 0, 0, 0, h2->logfile, h2->verbosity, 10);
   return 1;
 }
 for (i=0;i<h2->fieldsnum;i++) {
   h2->nullables[i]=1; /* 1 - NULL allowed, 0 - disallowed */
 }

 h2->keycols=(short *)load2_malloc((h1->udesc->sqld)* sizeof(short), __FILE__, __LINE__);
 if ( h2->keycols == NULL ) {
   sprintf(h2->errstr,"Out of memory");
   h2->onlogerror(__FILE__, __LINE__, h2->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
              sqlca.sqlerrm, 0, 0, 0, h2->logfile, h2->verbosity, 10);
   return 1;
 }
 for (i=0; i<h1->udesc->sqld; i++) {
   h2->keycols[i]=0;
 }

 h2->ldesc=(struct sqlda *)load2_malloc(sizeof(struct sqlda), __FILE__, __LINE__);
 if ( h2->ldesc == NULL ) {
   sprintf(h2->errstr,"Out of memory");
   h2->onlogerror(__FILE__, __LINE__, h2->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
              sqlca.sqlerrm, 0, 0, 0, h2->logfile, h2->verbosity, 10);
   return 1;
 }
 h2->ldesc->sqld=h1->udesc->sqld;
 bycopy((void *)h1->udesc, (void *)h2->ldesc, sizeof(struct sqlda));

 h2->ldesc->sqlvar=(struct sqlvar_struct *)load2_malloc(sizeof(struct sqlvar_struct)*(h2->ldesc->sqld), __FILE__, __LINE__);
 if ( h2->ldesc->sqlvar == NULL ) {
   sprintf(h2->errstr,"Out of memory");
   h2->onlogerror(__FILE__, __LINE__, h2->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
              sqlca.sqlerrm, 0, 0, 0, h2->logfile, h2->verbosity, 10);
   return 1;
 }
 bycopy((void *)h1->udesc->sqlvar, (void *)h2->ldesc->sqlvar, sizeof(struct sqlvar_struct)*(h2->ldesc->sqld));

 h2->blobs=0;
 pos=0;
 for (col1 = h2->ldesc->sqlvar, i = 0; i < h2->ldesc->sqld; col1++, i++) {
   col1->sqlind = (short *)(h2->indicators + i);
   if (col1->sqltype==CFIXBINTYPE) {
     if (col1->sqlxid==XID_BLOB || col1->sqlxid==XID_CLOB ) {
       memsize = col1->sqllen;
       col1->sqldata=load2_malloc(memsize, __FILE__, __LINE__);
       if ( col1->sqldata == NULL ) {
         sprintf(h2->errstr,"Out of memory\n");
         h2->onlogerror(__FILE__, __LINE__, h2->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                    sqlca.sqlerrm, h2->rownum, 0, 0, h2->logfile, h2->verbosity, 10);
         return 1;
       }
     }
   } else {
     memsize = col1->sqllen;
     pos = rtypalign( pos, col1->sqltype) + memsize;

     switch(col1->sqltype) {
        case SQLTEXT:
        case SQLBYTES:
/*          col->sqltype = CLOCATORTYPE; */
          (h2->blobs)++;
          break;
     }
   }
 }

 if (pos!=0) {
   h2->recordbuffer = (char *)load2_malloc(pos, __FILE__, __LINE__);
   if ( h2->recordbuffer == NULL ) {
     sprintf(h2->errstr,"Out of memory");
     h2->onlogerror(__FILE__, __LINE__, h2->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                   sqlca.sqlerrm, 0, 0, 0, h2->logfile, h2->verbosity, 10);
     return 1;
   }
 }
#if MI_PTR_SIZE == 64
 cp = (mlong)h2->recordbuffer;
#else
 cp = (mint)h2->recordbuffer;
#endif
 for (col1 = h2->ldesc->sqlvar, i = 0;
      i < h2->ldesc->sqld; col1++, i++) {
   if (col1->sqltype==CFIXBINTYPE) {
     continue;
   }
   cp = rtypalign( cp, col1->sqltype);
   memsize = col1->sqllen;
   col1->sqldata = (char *)cp;

   cp += memsize;
   if (col1->sqltype==CCHARTYPE || col1->sqltype==CVCHARTYPE
   || col1->sqltype==CLVCHARTYPE /* || col1->sqltype==CFIXCHARTYPE */) { 
     col1->sqllen=memsize;
   }
   if (col1->sqltype==CDTIMETYPE) {
     ((dtime_t *)col1->sqldata)->dt_qual=(int2)col1->sqllen;
   }
   if (col1->sqltype==CINVTYPE) {
      ((intrvl_t *)col1->sqldata)->in_qual=(int2)col1->sqllen;
   }
 }


 h3->delim=h1->delim;
 h3->batchsize=h1->batchsize;
 h3->fetcharray=0;
 h3->verbosity=h1->verbosity;
 strcpy(h3->tablename, h1->tablename);
 h3->fieldsnum=h1->fieldsnum+1;

 h3->typmsizes=(int *)load2_malloc(sizeof(int)*((h1->udesc->sqld)+1), __FILE__, __LINE__);
 if ( h3->typmsizes == NULL ) {
   sprintf(h3->errstr,"Out of memory\n");
   h3->onlogerror(__FILE__, __LINE__, h3->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, h3->rownum, 0, 0, h3->logfile, h3->verbosity, 10);
   return 1;
 }
 bycopy((void *)(h1->typmsizes),(void *)(h3->typmsizes+1),h1->udesc->sqld*sizeof(int));
 h3->typmsizes[0]=2;

 h3->indicators=(int *)load2_malloc((h3->fieldsnum)*sizeof(int), __FILE__, __LINE__);
 if ( h3->indicators == NULL ) {
   sprintf(h3->errstr,"Out of memory");
   h3->onlogerror(__FILE__, __LINE__, h3->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
              sqlca.sqlerrm, 0, 0, 0, h3->logfile, h3->verbosity, 10);
   return 1;
 }
 for (i=0;i<h3->fieldsnum;i++) {
   h3->indicators[i]=0;
 }


 h3->udesc=(struct sqlda *)load2_malloc(sizeof(struct sqlda), __FILE__, __LINE__);
 if ( h3->udesc == NULL ) {
   sprintf(h3->errstr,"Out of memory");
   h3->onlogerror(__FILE__, __LINE__, h3->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
              sqlca.sqlerrm, 0, 0, 0, h3->logfile, h3->verbosity, 10);
   return 1;
 }
 bycopy((void *)h1->udesc, (void *)h3->udesc, sizeof(struct sqlda));
 h3->udesc->sqld=h1->udesc->sqld+1;
 

 h3->udesc->sqlvar=(struct sqlvar_struct *)load2_malloc(sizeof(struct sqlvar_struct)*(h3->udesc->sqld), __FILE__, __LINE__);
 if ( h3->udesc->sqlvar == NULL ) {
   sprintf(h3->errstr,"Out of memory");
   h3->onlogerror(__FILE__, __LINE__, h3->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
              sqlca.sqlerrm, 0, 0, 0, h3->logfile, h3->verbosity, 10);
   return 1;
 }
/* bycopy((void *)h1->udesc->sqlvar, (void *)h3->udesc->sqlvar+1, sizeof(struct sqlvar_struct)*h1->udesc->sqld); */
  for (i=0;i<h1->udesc->sqld;i++) {
    (h3->udesc->sqlvar)[i+1]=(h1->udesc->sqlvar)[i];
  }

 h3->udesc->sqlvar->sqlname=strdup("flag");
 if ( h3->udesc->sqlvar->sqlname == NULL ) {
   sprintf(h3->errstr,"Out of memory");
   h3->onlogerror(__FILE__, __LINE__, h3->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
              sqlca.sqlerrm, 0, 0, 0, h3->logfile, h3->verbosity, 10);
   return 1;
 }
 h3->udesc->sqlvar->sqllen=2;
 h3->udesc->sqlvar->sqltype=CSHORTTYPE;

 pos = 0;
 for (col1 = h3->udesc->sqlvar, i = 0; i < h3->udesc->sqld; col1++, i++) {
   col1->sqlind = (short *)(h3->indicators + i);
   if (col1->sqltype==CFIXBINTYPE) {
     if (col1->sqlxid==XID_BLOB || col1->sqlxid==XID_CLOB ) {
       memsize = col1->sqllen;
       col1->sqldata=load2_malloc(memsize, __FILE__, __LINE__);
       if ( col1->sqldata == NULL ) {
         sprintf(h2->errstr,"Out of memory\n");
         h2->onlogerror(__FILE__, __LINE__, h2->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                    sqlca.sqlerrm, h2->rownum, 0, 0, h2->logfile, h2->verbosity, 10);
         return 1;
       }
     }
   } else {
     memsize = col1->sqllen;
     pos = rtypalign( pos, col1->sqltype) + memsize;
     switch(col1->sqltype) {
        case SQLTEXT:
        case SQLBYTES:
/*          col->sqltype = CLOCATORTYPE; */
          (h3->blobs)++;
          break;
     }
   }
 }

 h3->recordbuffer = (char *)load2_malloc(pos, __FILE__, __LINE__);
 if ( h3->recordbuffer == NULL ) {
   sprintf(h3->errstr,"Out of memory");
   h3->onlogerror(__FILE__, __LINE__, h3->errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, 0, 0, 0, h3->logfile, h3->verbosity, 10);
   return 1;
 }

#if MI_PTR_SIZE == 64
 cp = (mlong)h3->recordbuffer;
#else
 cp = (mint)h3->recordbuffer;
#endif
/* cp = (char *) rtypalign( (int) cp, SQLSMINT);
 memsize = rtypmsize(h3->udesc->sqlvar->sqltype, h3->udesc->sqlvar->sqllen);
 h3->udesc->sqlvar->sqldata=cp;
 cp += memsize; */

 for (col1 = h3->udesc->sqlvar, i = 0;
      i < h3->udesc->sqld; col1++, i++) {
   if (col1->sqltype==CFIXBINTYPE) {
     continue;
   }
   cp = rtypalign( cp, col1->sqltype);
   memsize = col1->sqllen;
   col1->sqldata = (char *)cp;
   cp += memsize;
   if (col1->sqltype==CCHARTYPE || col1->sqltype==CVCHARTYPE
   || col1->sqltype==CLVCHARTYPE /* || col1->sqltype==CFIXCHARTYPE */) { 
     col1->sqllen=memsize;
   }
   if (col1->sqltype==CDTIMETYPE) {
     ((dtime_t *)col1->sqldata)->dt_qual=(int2)col1->sqllen;
   }
   if (col1->sqltype==CINVTYPE) {
      ((intrvl_t *)col1->sqldata)->in_qual=(int2)col1->sqllen;
   }
 }
 h3->fieldstates=load2_malloc(h3->fieldsnum*sizeof(void *), __FILE__, __LINE__);
 if (h3->fieldstates==NULL) {
   sprintf(h3->errstr, "No memory for field states table");
   h3->onlogerror(__FILE__, __LINE__, h3->errstr, sqlca.sqlcode,
       sqlca.sqlerrd[1], sqlca.sqlerrm, 0, 0, 0, h3->logfile, h3->verbosity, 10);
   return 1;
 }

 byfill((char *)h3->fieldstates,h3->fieldsnum*sizeof(void *),0);
 for (col1 = h3->udesc->sqlvar, i = 0; i < h3->fieldsnum; col1++, i++) {
   if (col1->sqltype==CFIXBINTYPE &&
       (col1->sqlxid==XID_BLOB || col1->sqlxid==XID_CLOB )) {
     h3->fieldstates[i]=load2_malloc(sizeof(slob_handle_t), __FILE__, __LINE__);
     if (h3->fieldstates[i]==NULL) {
       snprintf(h3->errstr, ERR_BUF_SIZE, "No memory for slob handle");
       h3->onlogerror(__FILE__, __LINE__, h3->errstr, sqlca.sqlcode,
           sqlca.sqlerrd[1], sqlca.sqlerrm, 0, 0, 0, h3->logfile, h3->verbosity, 10);
       return 1;
     }
     byfill(h3->fieldstates[i],sizeof(slob_handle_t),0);
   }
 }

 return 0;
}

int compare_keys(unl_handle_t *unl_handle, loader_handle_t *loader_handle)
{ unl_handle_t *h1;
  loader_handle_t *h2;
  int i,j;
  struct sqlvar_struct *col1, *col2;

  h1=unl_handle;
  h2=loader_handle;
  for (i=0, col1=h1->udesc->sqlvar, col2=h2->ldesc->sqlvar;
       i<h2->ldesc->sqld; i++,col1++,col2++) {

    if (h2->keycols[i]) {
      j=colcmp(col1,col2,h1);
      if (j!=0) {
        return j;
      }
    }
  }
  return 0;
}

int same_records(unl_handle_t *unl_handle, loader_handle_t *loader_handle)
{ unl_handle_t *h1;
  loader_handle_t *h2;
  int i,j;
  struct sqlvar_struct *col1, *col2;

  h1=unl_handle;
  h2=loader_handle;
  for (i=0, col1=h1->udesc->sqlvar, col2=h2->ldesc->sqlvar;
       i<h2->ldesc->sqld; i++,col1++,col2++) {

    j=colcmp(col1,col2,h1);
    if (j!=0) {
      return 0;
    }
  }
  return 1;
}
 
int colcmp(struct sqlvar_struct *col1, struct sqlvar_struct *col2, unl_handle_t *unl_handle)
{ double dbl1;
  int sqlind1, sqlind2;
  int i, len;
  char *cp;
  unl_handle_t *h;


  sqlind1 = *(short *)(col1->sqlind);
  sqlind2 = *(short *)(col2->sqlind);
  if (sqlind1 == -1 ) {
    if (sqlind2 == -1 ) { 
      return 0;
    } else {
      return -1;
    }
  } else {
    if (sqlind2 == -1) {
      return 1;
    }
  }

  h=unl_handle;
  switch (col1->sqltype) {
    case CFLOATTYPE:
      dbl1=*((float *) (col1->sqldata)) - *((float *) (col2->sqldata));
      if (dbl1==0) {
        return 0;
      } else {
        if (dbl1>0) {
          return 1;
        } else {
          return -1;
        }
      }
      break;
    case CDOUBLETYPE:
      dbl1=*((double *) (col1->sqldata)) - *((double *) (col2->sqldata));
      if (dbl1==0) {
        return 0;
      } else {
        if (dbl1>0) {
          return 1;
        } else {
          return -1;
        }
      }
      break;
    case CDECIMALTYPE:
      i=deccmp((dec_t *) (col1->sqldata), (dec_t *) (col2->sqldata));
      return i;
    case CINTTYPE:
      dbl1=*((int4 *) (col1->sqldata)) - *((int4 *) (col2->sqldata));
      if (dbl1==0) {
        return 0;
      } else {
        if (dbl1>0) {
          return 1;
        } else {
          return -1;
        }
      }
      break;
    case CSHORTTYPE:
      dbl1=*((int2 *) (col1->sqldata)) - *((int2 *) (col2->sqldata));
      if (dbl1==0) {
        return 0;
      } else {
        if (dbl1>0) {
          return 1;
        } else {
          return -1;
        }
      }
      break;
    case CINT8TYPE:
      i=ifx_int8cmp((struct ifx_int8 *) (col1->sqldata),
                       (struct ifx_int8 *) (col2->sqldata));
      return i;
    case CBOOLTYPE:
      if (*((char *)col1->sqldata)=='\0'){
        if (*((char *)col2->sqldata)=='\0') {
          i=0;
        } else {
          i=1;
        }
      } else {
        if (*((char *)col2->sqldata)=='\0') {
          i=-1;
        } else {
          i=0;
        }
      }
      return i;
      break;
    case CDATETYPE:
      dbl1=*((int4 *) (col1->sqldata)) - *((int4 *) (col2->sqldata));
      if (dbl1==0) {
        return 0;
      } else {
        if (dbl1>0) {
          return 1;
        } else {
          return -1;
        }
      }
      break;
    case CDTIMETYPE:
      i=deccmp((dec_t *)&((dtime_t *) col1->sqldata)->dt_dec,
                  (dec_t *)&((dtime_t *) col2->sqldata)->dt_dec);
      return i;
    case CINVTYPE:
      i=deccmp((dec_t *)&((intrvl_t *) col1->sqldata)->in_dec,
                  (dec_t *)&((intrvl_t *) col2->sqldata)->in_dec);
      return i;
    case CCHARTYPE:
      len = stleng((char *) col1->sqldata);
      cp = ((char *) col1->sqldata) + len - 1;
      while ( len > 1 && *cp == ' ') {
        *cp = '\0';
        len--, cp--;
      }

      len = stleng((char *) col2->sqldata);
      cp = ((char *) col2->sqldata) + len - 1;
      while ( len > 1 && *cp == ' ') {
        *cp = '\0';
        len--, cp--;
      }

      dbl1=stringcmp(((char *) col1->sqldata), ((char *) col2->sqldata));
      if (dbl1==0) {
        return 0;
      } else {
        if (dbl1>0) {
          return 1;
        } else {
          return -1;
        }
      }
      break;
    case CVCHARTYPE:
      dbl1=stringcmp(((char *) col1->sqldata), ((char *) col2->sqldata));
      if (dbl1==0) {
        return 0;
      } else {
        if (dbl1>0) {
          return 1;
        } else {
          return -1;
        }
      }
      break;
    case CLVCHARTYPE:
      dbl1=stringcmp(((char *) col1->sqldata), ((char *) col2->sqldata));
      if (dbl1==0) {
        return 0;
      } else {
        if (dbl1>0) {
          return 1;
        } else {
          return -1;
        }
      }
      break;
    case CLOCATORTYPE:
    case SQLTEXT:
    case SQLBYTES:
      return 0; /* all BLOBs are equal: enough for this version */
/*      bycopy(sqldata, fieldp, sizeof(loc_t));
      if (((loc_t *)fieldp)->loc_indicator==-1) {
        *sqlind=-1;
      } else {
        *sqlind=0;
      }
      break; */

    default:
      /*  Usupported data type.  */
      return 0;
/*      snprintf(h->errstr, ERR_BUF_SIZE, "unknown type:%s:%s:%d\n",
               col1->sqlname, rtypname(col1->sqltype), col1->sqltype);
      h->onlogerror(__FILE__, __LINE__, h->errstr, 0, 0, NULL, h->rownum, 0, 0,
                   h->logfile, h->verbosity, 10);
      exit(1); */
  }
  return 0;
}

int putrec1(int flag, unl_handle_t *unl_handle, unl_handle_t *unl_handle2)
{ unl_handle_t *h1, *h2;
  struct sqlvar_struct *col1, *col2;
  int i;
  loc_t simple_lo;

  h1=unl_handle;
  h2=unl_handle2;

  for (i=0, col1=h1->udesc->sqlvar, col2=h2->udesc->sqlvar+1;
       i<h1->udesc->sqld; i++,col1++,col2++) {

    if (*(col1->sqlind) == -1 ) {
      *(col2->sqlind)=-1;
      switch (col1->sqltype) {
        case CLOCATORTYPE:
        case SQLTEXT:
        case SQLBYTES: /* for these types rsetnull is not enough */
                       simple_lo.loc_indicator= -1; /* Null */
                       simple_lo.loc_bufsize=0;
                       simple_lo.loc_buffer=NULL;
                       simple_lo.loc_type=col1->sqltype;
                       simple_lo.loc_loctype=LOCMEMORY;
                       simple_lo.loc_size=0;
                       bycopy((void *)&simple_lo, col2->sqldata, sizeof(loc_t));
                       break;
      }
      rsetnull(col2->sqltype,col2->sqldata);
    } else {
      *(col2->sqlind)=*(col1->sqlind);
      bycopy(col1->sqldata, col2->sqldata, h1->typmsizes[i]);
    }
  }
  col2=h2->udesc->sqlvar;
  *(col2->sqlind)=0;
  bycopy((void *)&flag, col2->sqldata, h2->typmsizes[0]);
  return 0;
}

int putrec2(int flag, loader_handle_t *loader_handle, unl_handle_t *unl_handle2)
{ loader_handle_t *h1;
  unl_handle_t *h2;
  struct sqlvar_struct *col1, *col2;
  int i;
  loc_t simple_lo;

  h1=loader_handle;
  h2=unl_handle2;

  for (i=0, col1=h1->ldesc->sqlvar, col2=h2->udesc->sqlvar+1;
       i<h1->ldesc->sqld; i++,col1++,col2++) {

    if (*(col1->sqlind) == -1 ) {
      *(col2->sqlind)=-1;
      switch (col1->sqltype) {
        case CLOCATORTYPE:
        case SQLTEXT:
        case SQLBYTES: /* for these types rsetnull is not enough */
                       simple_lo.loc_indicator= -1; /* Null */
                       simple_lo.loc_bufsize=0;
                       simple_lo.loc_buffer=NULL;
                       simple_lo.loc_type=col1->sqltype;
                       simple_lo.loc_loctype=LOCMEMORY;
                       simple_lo.loc_size=0;
                       bycopy((void *)&simple_lo, col2->sqldata, sizeof(loc_t));
                       break;
      }
      rsetnull(col2->sqltype,col2->sqldata);
    } else {
      *(col2->sqlind)=*(col1->sqlind);
      if ((col1->sqltype)==CLOCATORTYPE
       || (col1->sqltype)==SQLBYTES
       || (col1->sqltype)==SQLTEXT) {
        blob_copy2((loc_t *)(col1->sqldata),
                  (loc_t *)(col2->sqldata),
                  h1->rownum, h1->rownum2,
                  h1->logfile, h1->onlogerror, h1->verbosity);
      } else {
        if (col1->sqltype==CFIXBINTYPE &&
          (col1->sqlxid==XID_BLOB || col1->sqlxid==XID_CLOB)) {
          sblob_copy((ifx_lo_t *)((col1->sqldata)),
                     (ifx_lo_t *)(col2->sqldata),
                     h1->rownum, h1->rownum2,
                     h1->logfile, h1->onlogerror, h1->verbosity);
        } else {
          bycopy(col1->sqldata, col2->sqldata, h2->typmsizes[i+1]);
        }
      }
    }
  }
  col2=h2->udesc->sqlvar;
  *(col2->sqlind)=0;
  bycopy((void *) &flag, col2->sqldata, h2->typmsizes[0]);
  return 0;
}

int writerec(unl_handle_t *unl_handle)
{ unl_handle_t *h;
  int i,k;
  char buffer[32767+1];
  unsigned long int bytesread;
  int sqlind;

  i=0; /* current field number */
  h=unl_handle;
  while (i<h->fieldsnum) {
    k=unl_getfield(h, i, buffer, sizeof(buffer), &bytesread, &sqlind);
    if (k!=0) {
      return 1;
    }

    k=unl_writefield(h, i, buffer, bytesread, sqlind, h->fieldstates[i]);
    if (k!=0) {
      return 1;
    }
    i++;
  }
  if (i>=h->fieldsnum) {
    if (unl_afterrow(h)) {
      return 1;
    }
  }
  return 0;
}

int stringcmp(char *s1, char *s2)
{ int i;
  /* return strcmp(s1,s2); */
  i=0;
  while (s1[i]==s2[i]) {
    if (s1[i]=='\0' || s2[i]=='\0') {
      break;
    }
    i++;
  }
  if (s1[i]!=0 && s2[i]!=0) {
    return s1[i]-s2[i];
  } else {
    if (s1[i]=='\0') {
      while (s2[i]==' ') {
         i++;
      }
      return '\0' - s2[i];
    } else {
      while (s1[i]==' ') {
         i++;
      }
      return s1[i] - '\0';
    }
  }
}

int blob_copy(loc_t *unl_blob, loc_t *load_blob, unsigned long int rownum,
          unsigned long int loaded, const void *logfile,
          onlogerror_upt onlogerror, int verbose)
{ char errstr[ERR_BUF_SIZE];

  if (unl_blob->loc_position==1) {
    load_blob->loc_loctype=LOCMEMORY;
    load_blob->loc_bufsize=unl_blob->loc_bufsize;
    load_blob->loc_size=unl_blob->loc_size;
    load_blob->loc_mflags=0;
    load_blob->loc_oflags=0;
    load_blob->loc_buffer=unl_blob->loc_buffer;
  } else {
    if (unl_blob->loc_position!=2) {
      sprintf(errstr,"Incorrect BLOB storage: "INT4_FORMAT"", unl_blob->loc_position);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
         0, logfile, verbose, 10);
      return 1; 
    }
    load_blob->loc_loctype=LOCFNAME;
    load_blob->loc_size=-1;
    load_blob->loc_oflags=LOC_RONLY;
    load_blob->loc_fname=(char *)load2_malloc(unl_blob->loc_bufsize, __FILE__, __LINE__);
    if (load_blob->loc_fname==NULL) {
      sprintf(errstr,"Out of memory for blob");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
         0, logfile, verbose, 10);
      return 1;
    }
    strcpy(load_blob->loc_fname, unl_blob->loc_buffer);
    load2_free(unl_blob->loc_buffer, __FILE__, __LINE__);
    unl_blob->loc_buffer=NULL;
  }
  load_blob->loc_indicator=unl_blob->loc_indicator;
  load_blob->loc_type=unl_blob->loc_type;
  return 0;
}

int blob_copy2(loc_t *load_blob, loc_t *unl_blob, unsigned long int rownum,
          unsigned long int loaded, const void *logfile,
          onlogerror_upt onlogerror, int verbose)
{ char errstr[ERR_BUF_SIZE];

  unl_blob->loc_indicator=load_blob->loc_indicator;
  if (load_blob->loc_loctype==LOCMEMORY) {
    unl_blob->loc_loctype=LOCMEMORY;
    unl_blob->loc_position=1;
    unl_blob->loc_size=load_blob->loc_size;
    if (load_blob->loc_buffer != NULL && load_blob->loc_indicator!=-1
     && load_blob->loc_size!=0) {
      unl_blob->loc_buffer=(char *)load2_malloc(load_blob->loc_bufsize, __FILE__,__LINE__);
      if (unl_blob->loc_buffer==NULL) {
        sprintf(errstr,"Out of memory for blob");
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
           0, logfile, verbose, 10);
        return 1;
      }
      bycopy(load_blob->loc_buffer, unl_blob->loc_buffer, load_blob->loc_bufsize);
    } else {
      unl_blob->loc_buffer=NULL;
    }
  } else {
    unl_blob->loc_position=2;
    unl_blob->loc_bufsize=strlen(load_blob->loc_fname);
    unl_blob->loc_buffer=(char *)load2_malloc(unl_blob->loc_bufsize, __FILE__, __LINE__);
    if (unl_blob->loc_buffer==NULL) {
      sprintf(errstr,"Out of memory for blob");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
         0, logfile, verbose, 10);
      return 1;
    }
    strcpy(unl_blob->loc_buffer, load_blob->loc_fname);
  }
  load_blob->loc_indicator=unl_blob->loc_indicator;
  load_blob->loc_type=unl_blob->loc_type;
  return 0;
}

int sblob_copy(ifx_lo_t *unl_blob, ifx_lo_t *load_blob, unsigned long int rownum,
               unsigned long int loaded, const void *logfile,
               onlogerror_upt onlogerror, int verbose)
{
  bycopy((char *)unl_blob, (char *)load_blob, sizeof(ifx_lo_t));
/* it just works */  
  return 0;
}
