/******************************************************************************
*
* Standard event handlers for loading functions - gz format
*
* Author: Robert Sosnowski, Robert.Sosnowski@bzwbk.pl
*
* Version 1.70, 17-10-2005
*
* Copyright (c) 2001 - 2007. All Rights Reserved.
*
* Permission granted to use & distribute for free. Changes in source code
* require Author's acceptance.
*
* This software can be used only at your own risk. Author is not responsible 
* for any damage caused by using this software.
* 
******************************************************************************/

#include "load2cfg.h"
#include "tempfile.h"

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif


#ifdef HAVE_IO_H
#include <io.h>
/* _mktemp, creat prototypes in MSVC */
#endif

#include <stdio.h>
#include "load2.h"

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <stdlib.h>

#include <fcntl.h>
#include <zlib.h>

EXEC SQL include locator;
EXEC SQL include sqltypes;

#define READBUF_SIZE 10000
typedef struct {
  gzFile fd;
  void *readbuf;
  int readbufind;
  int readbufsize;
  int readbufmax;
} buffered_file;

  

int oninitgz(const char *fname, const char *tablename, const int batchsize,
             void **handle, const void *logfile, const int fieldsnum,
             const char *insstmt2,
             onlogerror_lpt onlogerror, int verbose,
             const char * datahandleoptions);
int onreadfieldgz(const void *fd, const int sqltype, const int4 sqlxid,
                  const char *filename,
                  char **dst, const unsigned int maxdstbuf, const char delim,
                  const char escapech, int * eol, int * sqlind,
                  const unsigned long int rownum,
                  const unsigned long int loaded, const int fieldnum,
                  const char * fieldname, const int max_fld_name,
                  char *fieldname2, const void *logfile,
                  onlogerror_lpt onlogerror, int verbose,
                  const char *datahandleoptions);
int oncrlfgz(const void *fd, const char *filename,
             const unsigned long int rownum, const unsigned long int loaded,
             const int fields_number, const int bequiet, const char escapech,
             const void *logfile,
             onlogerror_lpt onlogerror, int verbose,
             const char *datahandleoptions);
void oncleanupgz(const char *fname, void *handle, const void *logfile,
                 onlogerror_lpt onlogerror, int verbose,
                 const char *datahandleoptions);


int onblobfreegz(loc_t *simple_lo, const void *logfile, onlogerror_lpt onlogerror,
                   int verbose);

static int onreadtextblob(const void *fd, const char *filename,
                          char *dst, const unsigned int maxdstbuf,
                          const char delim, const char escapech,
                          int *eol, int *sqlind,
                          const unsigned long int rownum,
                          const unsigned long int loaded,
                          const int fieldnum, const char * fieldname,
                          const int max_fld_name, char *fieldname2,
                          const void *logfile, onlogerror_lpt onlogerror,
                          int verbose);

static int onreadbyteblob(const void *fd, const char *filename,
                    char *dst, const unsigned int maxdstbuf, const char delim,
                    int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose);

static int onreadsmartblob(const void *fd, const char *filename,
                    const int4 sqlxid, char *dst,
                    const unsigned int maxdstbuf, const char delim,
                    const char escapech, int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose);

static int myfgetc(buffered_file *fd);

static char *id="@(#) $Id: load2gz.ec 2521 2007-11-21 08:55:56Z sosnor01 $";


int onblobfreegz(loc_t *simple_lo, const void *logfile,
                   onlogerror_lpt onlogerror, int verbose)
{
  if (simple_lo->loc_loctype==LOCMEMORY) {
    if (simple_lo->loc_buffer!=NULL) {
      free(simple_lo->loc_buffer);
    }
    simple_lo->loc_buffer=NULL;
    simple_lo->loc_size=0;
    simple_lo->loc_bufsize=0;
  }
  if (simple_lo->loc_loctype==LOCFNAME) {
    unlink(simple_lo->loc_fname);
    free(simple_lo->loc_fname);
    simple_lo->loc_fname=NULL;
  }
  return 0;
}


static int hexdigit(int c)
{ int n;

  if (isdigit(c)) {
    n=c-'0';    
  } else {
    if (c>='a' && c<='f') {
      n=c-'a'+10;
    } else {
      if (c>='A' && c<='F') {
        n=c-'A'+10;
      } else {
        n=-1;
      }
    }
  }
  return n;
}

static short hex2int8(char *hex, off_t *result)
{
  int i;
  short j;
  off_t k;
  int ch;

  i=0;
  k=0;
  while ((ch=hex[i++])) {
    j=hexdigit(ch);
    if (j==-1) {
      return -1;
    }
    k=(k<<4) + j;
  }
  *result=k;
  return 0;
}

static int onreadbyteblob(const void *fd, const char *filename,
                    char *dst, const unsigned int maxdstbuf, const char delim,
                    int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose)

/* returns: length if >=0;
 *          -1 if error;
 *          -2 if no more data (like sqlcode 100 in SQL)
 * dst : pointer to the buffer;
 * if pointer is null then procedure allocates this else given pointer is used
 */
{
  unsigned int i, j, k;
  int ch;
  buffered_file *fd1;
  char errstr[ERR_BUF_SIZE];
  char *bufp, *blobbufp, *p;
  unsigned long maxbuf;
  loc_t simple_lo;
  char tmpfilename[256];
  FILE * ft;
  int destination;
  int parity, non_hex;
  char *envp;

  ft=NULL;
  fd1=(buffered_file *)fd;
  destination=0; /* memory */
  ch=myfgetc(fd1);
  i=0;
  fieldname2[0]=0;
  bufp=dst;
  maxbuf=maxdstbuf-1;
#if defined(HAVE_GETENV)
  envp=getenv("DBBLOBBUF");
  if (envp!=NULL && isdigit(envp[0])) {
    k=atol(envp);
    if (k*1024<maxdstbuf) {
      maxbuf=k*1024;
    }
  }
#endif
  parity=0;
  non_hex=0;
  k=0;
  

  while ( fd1->readbufmax ) {
    if (i>=maxbuf) {
      if (destination==0) {
        destination=1; /* file */
        p=creatmpfile();
        strcpy(tmpfilename, p);
        if (tmpfilename==NULL) {
          i=errno;
          sprintf(errstr,"can't create temporary file at line %ld, errno=%d (%s)",
                  rownum, i, tmpfilename);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                     logfile, verbose, 10);
          return -1;
        }
        ft=fopen(tmpfilename, "wb");
        if (ft==NULL) {
          i=errno;
          sprintf(errstr,"can't open temporary file at line %ld, errno=%d (%s)",
                  rownum, i, tmpfilename);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                     logfile, verbose, 10);
          return -1;
        }
      }
      i=fwrite(bufp,1, i, ft);
      if (ferror(ft) ) {
        i=errno;
        sprintf(errstr,"can't write temporary file at line %ld, errno=%d (%s)",
                rownum, i, tmpfilename);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                   logfile, verbose, 10);
        return -1;
      }
      i=0;
    }
    if (ch=='\r') {
      ch=myfgetc(fd1);
      if (ch=='\n') {
        break;
      } else {
        bufp[i]='\r';
        i++;
        continue;
      }
    }
    if (ch=='\n') {
      break;
    }
    if (ch==delim) {
      break;
    }
    if (ch==-1 || ch==-2) {
      continue;
    }
    j=(unsigned int)hexdigit(ch);
    if (j==-1) {
      non_hex=1;
    }
    if (!parity) {
      k=j<<4;
    } else {
      k=k | j;
      bufp[i]=k;
      i++;
    }
    parity=!parity;
    ch=myfgetc(fd1);
  }
  if (ch==-2) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
    return -1;
  }
  if (non_hex) {
    sprintf(errstr,"non-hex BYTE at line %ld", rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, 0,
               logfile, verbose, 10);
    return -1;
  }
  if (parity!=0) {
    j=k>>4;
    bufp[i]=j;
    i++;
  }
  k=i;
  if (destination==1) {
    i=fwrite(bufp,1, i, ft);
    if (ferror(ft) ) {
      i=errno;
      sprintf(errstr,"can't write temporary file at line %ld, errno=%d (%s)",
              rownum, i, tmpfilename);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                 logfile, verbose, 10);
      return -1;
    }
    i=fclose(ft);
    if ( i != 0 ) {
      i=errno;
      sprintf(errstr,"can't close temporary file at line %ld, errno=%d (%s)",
                      rownum, i, tmpfilename);
              onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                         logfile, verbose, 10);
              return -1;
            }
          }
          if (destination==0) {
            blobbufp=NULL;
            if (i>0) {
              blobbufp=(char *)load2_malloc(k+1, __FILE__, __LINE__);
              if (blobbufp==NULL) {
                sprintf(errstr,"Out of memory");
                onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
                   0, logfile, verbose, 10);
                return -1;
              }
              bycopy(bufp,blobbufp,k);
            }
            simple_lo.loc_loctype=LOCMEMORY;
            simple_lo.loc_bufsize=k+1;
            simple_lo.loc_size=k;
            simple_lo.loc_buffer=blobbufp;
            simple_lo.loc_mflags=0;
            simple_lo.loc_oflags=0;
          } else { /* destination */
            blobbufp=NULL;
            i=strlen(tmpfilename);
    blobbufp=(char *)load2_malloc(i+1, __FILE__, __LINE__);
    if (blobbufp==NULL) {
      sprintf(errstr,"Out of memory");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
         0, logfile, verbose, 10);
      return -1;
    }
    strcpy(blobbufp,tmpfilename);
    simple_lo.loc_loctype=LOCFNAME;
    simple_lo.loc_fname=blobbufp;
    simple_lo.loc_oflags=LOC_RONLY;
    simple_lo.loc_size=-1;
  }
  if (k==0 && simple_lo.loc_loctype==LOCMEMORY) {
    simple_lo.loc_indicator= -1; /* Null */
    *sqlind=-1;
    simple_lo.loc_bufsize=0;
    simple_lo.loc_buffer=NULL;
  } else {
    simple_lo.loc_indicator= k;
    *sqlind=0;
  }
  simple_lo.loc_type=SQLBYTES;
  bycopy((void *)&simple_lo, dst, sizeof(loc_t));

  if (ch == -1 || ch == '\n' ) {
    *eol = 1;
  } else {
    *eol = 0;
  }
  if (k==0 && destination==0) { /* value -2 have to be retuned only for the case when 
                                   no field data read */
    if ( !fd1->readbufmax ) {
      return -2;
    }
  }
  if (destination!=0 && k==0) {
    k=1;
  }
  return k; /* ok only for destination==0 */
}


static int onreadtextblob(const void *fd, const char *filename,
                    char *dst, const unsigned int maxdstbuf, const char delim,
                    const char escapech, int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose)

/* returns: length if >=0;
 *          -1 if error;
 *          -2 if no more data (like sqlcode 100 in SQL)
 * dst : pointer to the buffer;
 * if pointer is null then procedure allocates this else given pointer is used
 */
{
  unsigned int i, k;
  int ch;
  int escaped;
  int empty_string;
  buffered_file *fd1;
  char errstr[ERR_BUF_SIZE];
  char *bufp, *blobbufp, *p;
  unsigned long maxbuf;
  loc_t simple_lo;
  char tmpfilename[256];
  FILE * ft;
  int destination;
  char *envp;

  ft=NULL;
  fd1=(buffered_file *)fd;
  destination=0; /* memory */
  ch=myfgetc(fd1);
  i=0;
  escaped=0;
  empty_string=0;
  fieldname2[0]=0;
  bufp=dst;
  maxbuf=maxdstbuf-1;
#if defined(HAVE_GETENV)
  envp=getenv("DBBLOBBUF");
  if (envp!=NULL && isdigit(envp[0])) {
    k=atol(envp);
    if (k*1024<maxdstbuf) {
      maxbuf=k*1024;
    }
  }
#endif
  
  while ( fd1->readbufmax) {
    if (i>=maxbuf) {
      bufp[maxbuf]=0;
      if (destination==0) {
        destination=1; /* file */
        p=creatmpfile();
        strcpy(tmpfilename, p);
        if (tmpfilename==NULL) {
          i=errno;
          sprintf(errstr,"can't create temporary file at line %ld, errno=%d (%s)",
                  rownum, i, tmpfilename);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                     logfile, verbose, 10);
          return -1;
        }
        ft=fopen(tmpfilename, "wb");
        if (ft==NULL) {
          i=errno;
          sprintf(errstr,"can't open temporary file at line %ld, errno=%d (%s)",
                  rownum, i, tmpfilename);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                     logfile, verbose, 10);
          return -1;
        }
      }
      i=fwrite(bufp,1, maxbuf, ft);
      if (ferror(ft) ) {
        i=errno;
        sprintf(errstr,"can't write temporary file at line %ld, errno=%d (%s)",
                rownum, i, tmpfilename);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                   logfile, verbose, 10);
        return -1;
      }
      i=0;
    }
    if ( !escaped ) {
      if ( ch==escapech ) {
        escaped=1;
        ch=myfgetc(fd1);
        if (i==0 && ch==' ') {
          empty_string=1;
        }
        continue;
      }
      if (ch=='\r') {
        ch=myfgetc(fd1);
        if (ch=='\n') {
          break;
        } else {
          bufp[i]='\r';
          i++;
          continue;
        }
      }
      if (ch=='\n') {
        break;
      }
      if (ch==delim) {
        break;
      }
    } else {
      escaped=0;
    }
    if (ch==-1 || ch==-2 ) {
      continue;
    }
    bufp[i]=ch;
    i++;
    ch=myfgetc(fd1);
  }
  if (ch==-2) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
    return -1;
  }
  if (destination==1) {
    i=fwrite(bufp,1, i, ft);
    if (ferror(ft) ) {
      i=errno;
      sprintf(errstr,"can't write temporary file at line %ld, errno=%d (%s)",
              rownum, i, tmpfilename);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                 logfile, verbose, 10);
      return -1;
    }
    i=fclose(ft);
    if ( i != 0 ) {
      i=errno;
      sprintf(errstr,"can't close temporary file at line %ld, errno=%d (%s)",
              rownum, i, tmpfilename);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                 logfile, verbose, 10);
      return -1;
    }
  }
  bufp[i]=0;
  if (i>1){
    empty_string=0;
  }
  if (empty_string) {
    i=0;
    bufp[0]=0;
  }
  if (destination==0) {
    blobbufp=NULL;
    if (i>0) {
      blobbufp=(char *)load2_malloc(i+1, __FILE__, __LINE__);
      if (blobbufp==NULL) {
        sprintf(errstr,"Out of memory");
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
           0, logfile, verbose, 10);
        return -1;
      }
      memcpy(blobbufp,bufp,i);
    }
    simple_lo.loc_loctype=LOCMEMORY;
    simple_lo.loc_bufsize=i+1;
    simple_lo.loc_size=i;
    simple_lo.loc_buffer=blobbufp;
    simple_lo.loc_mflags=0;
    simple_lo.loc_oflags=0;
  } else { /* destination */
    blobbufp=NULL;
    i=strlen(tmpfilename);
    blobbufp=(char *)load2_malloc(i+1, __FILE__, __LINE__);
    if (blobbufp==NULL) {
      sprintf(errstr,"Out of memory");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
         0, logfile, verbose, 10);
      return -1;
    }
    strcpy(blobbufp,tmpfilename);
    simple_lo.loc_loctype=LOCFNAME;
    simple_lo.loc_fname=blobbufp;
    simple_lo.loc_oflags=LOC_RONLY;
    simple_lo.loc_size=-1;
  }
  if (bufp[0]==0 && !empty_string && simple_lo.loc_loctype==LOCMEMORY) {
    simple_lo.loc_indicator= -1; /* Null */
    simple_lo.loc_bufsize=0;
    simple_lo.loc_buffer=NULL;
    *sqlind=-1;
  } else {
    simple_lo.loc_indicator= i;
    *sqlind=0;
  }
  simple_lo.loc_type=SQLTEXT;
  bycopy((void *)&simple_lo, dst, sizeof(loc_t));

  if (ch == -1 || ch == '\n' ) {
    *eol = 1;
  } else {
    *eol = 0;
  }
  if (i==0 && destination==0) { /* value -2 have to be retuned only for the case when 
                                   no field data read */
    if ( !fd1->readbufmax ) {
      return -2;
    }
  }
  if (destination!=0 && i==0) {
    i=1;
  }
  return i; /* ok only for destination==0 */
}

static int onreadsmartblob(const void *fd, const char *filename,
                    const int4 sqlxid, char *dst,
                    const unsigned int maxdstbuf, const char delim,
                    const char escapech, int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose)
/* returns: length if >=0;
 *          -1 if error;
 *          -2 if no more data (like sqlcode 100 in SQL)
 */
{
  char errstr[ERR_BUF_SIZE];
  mint lofd;
  int4 ifx_error;
  mint ifx_error2;
  off_t fsize64;
  gzFile *f;
  buffered_file *fd1;
  char filename2[256];
  unsigned int stat;
  ifx_lo_create_spec_t *create_spec;
  off_t offset8;
  char buf[MAXMEMBLOB+1];
  ifx_lo_t descr;
  char ch;
  unsigned int i;
  short overbuf;
  char *bufp, *p1, *p2;
  unsigned int maxbuf;
  int numbytes;
  int buflen;
  char hex1[20], hex2[20];
  int n;
  ifx_int8_t t1;

  fd1=(buffered_file *)fd;
  ch=myfgetc(fd1);
  i=0;
  fieldname2[0]=0;
  overbuf=0;
  bufp=dst;
  maxbuf=maxdstbuf;
  maxbuf--;
  
  while ( fd1->readbufmax ) {
    if (ch=='\r') { /* \r\n recognized as end-of-field */
      ch=myfgetc(fd1);
      if (ch=='\n') {
        break;
      } else {
        if (i<maxbuf) {
          bufp[i]='\r';
          i++;
        } else {
          overbuf=1;
        }
        continue;
      }
    }
    if (ch=='\n') { /* \n recognized as end-of-field */
      break;
    }
    if (ch==delim) {
      break;
    }
    if (ch==-1 || ch==-2) {
      continue;
    }
    if (i<maxbuf) {
      bufp[i]=ch;
      i++;
    } else {
      overbuf=1;
    }
    ch=myfgetc(fd1);
  }
  if (ch==-2) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
    return -1;
  }
  if (overbuf) {
    bufp[maxbuf]=0;
    sprintf(errstr,"field %s (field no %d) longer than %ud at line %ld",
            fieldname, fieldnum, maxbuf, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  bufp[i]=0;
  if (bufp[0]==0) { /* if NULL */
    *sqlind=-1;
  } else {
    *sqlind=0;
  }
  if (ch == -1 || ch == '\n' ) {
    *eol = 1;
  } else {
    *eol = 0;
  }
  if (i==0) { /* value -2 have to be retuned only for the case when 
                 no field data read */
    if ( !fd1->readbufmax ) {
      return -2;
    }
  }
  if (bufp[0]==0) { /* field is NULL */
    return 0;
  }
  p1=strchr(bufp,',');
  if (p1==NULL) {
    sprintf(errstr,"Smart blob field doesn't contain ','");
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  i=p1-bufp-1;
  if (i>sizeof(hex1)-1) {
    sprintf(errstr,"Too long hex value in string: %s", bufp);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  (*p1)=0;
  strcpy(hex1, bufp);
  (*p1)=',';
  p1++;
  p2=strchr(p1,',');
  if (p2==NULL) {
    sprintf(errstr,"Smart blob field doesn't contain second ','");
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  i=p2-p1-1;
  if (i>sizeof(hex2)-1) {
    sprintf(errstr,"Too long second hex value in string: %s", bufp);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  (*p2)=0;
  strcpy(hex2, p1);
  (*p2)=',';
  p2++;
  strcpy(filename2, p2++);
  
  stat=hex2int8(hex1, &offset8);
  if (stat==-1) {
    sprintf(errstr,"Non hex digit in hex number: %s", hex1);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  stat=hex2int8(hex2, &fsize64);
  if (stat==-1) {
    sprintf(errstr,"Non hex digit in hex number: %s", hex2);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  ifx_error = ifx_lo_def_create_spec(&create_spec);
  if (ifx_error < 0) {
    sprintf(errstr,"ERROR ifx_lo_def_create_spec:"INT4_FORMAT","INT4_FORMAT",%s",
           ifx_error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, ifx_error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, loaded, 0, logfile, verbose, 10);
    return -1;
  }

  ifx_error=off_t2int8(fsize64, &t1);
  if (ifx_error < 0) {
    sprintf(errstr,"ERROR converting off_t into int8:"INT4_FORMAT"", ifx_error);
    onlogerror(__FILE__, __LINE__, errstr, ifx_error, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  ifx_error=ifx_lo_specset_estbytes(create_spec, &t1);
  if (ifx_error < 0) {
    sprintf(errstr,"ERROR ifx_lo_specset_estbytes:"INT4_FORMAT","INT4_FORMAT",%s",
            ifx_error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, ifx_error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, loaded, 0, logfile, verbose, 10);
    return -1;
  }
  
/* Create an LO-specification structure for the smart large object */
  if ((lofd = ifx_lo_create(create_spec, LO_WRONLY, &descr, &ifx_error2)) == -1) {
    sprintf(errstr,"ERROR ifx_lo_create:%d,"INT4_FORMAT",%s",
            ifx_error2, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, ifx_error2, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, loaded, 0, logfile, verbose, 10);
    return -1;
  }

/* Copy data into the open smart
 * large object that lofd points to. */
  f=gzopen(filename2,"rb");
  if (f==NULL) {
    sprintf(errstr,"gzOS error in gzopen: %d", errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       errno, logfile, verbose, 10);
    return -1;
  }

  stat=gzseek(f, (z_off_t)offset8, SEEK_SET);
  if (stat<0) {
    sprintf(errstr,"gzOS error in fseek:%d", errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       errno, logfile, verbose, 10);
    return -1;
  }
  while (fsize64>0) {
    if (MAXMEMBLOB-1>fsize64) {
      n=fsize64;
    } else {
      n=MAXMEMBLOB-1;
    }
    buflen=gzread(f, buf,n);
    if (buflen<n) {
      if (buflen==0) {
        sprintf(errstr,"unexpected eof in gzread");
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
           errno, logfile, verbose, 10);
        return -1;
      } else {
        sprintf(errstr,"OS error in gzread:%d", errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
           errno, logfile, verbose, 10);
        return -1;
      }
    }

    numbytes = ifx_lo_write(lofd, buf, buflen, &ifx_error2);
    if ( numbytes < buflen ) {
      sprintf(errstr,"ERROR ifx_lo_write:%d,"INT4_FORMAT",%s",
              ifx_error2, sqlca.sqlerrd[1], sqlca.sqlerrm);
      onlogerror(__FILE__, __LINE__, errstr, ifx_error2, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, rownum, loaded, errno, logfile, verbose, 10);
      return -1;
    }
    fsize64-=n;
  }
  gzclose(f);
/* Close the LO file descriptor */
  ifx_lo_close(lofd);
/* Free LO-specification structure */
  ifx_lo_spec_free(create_spec);
  bycopy((char *) &descr, bufp, sizeof(descr));
  return sizeof(descr);
}

int onreadfieldgz(const void *fd, const int sqltype, const int4 sqlxid,
                    const char *filename,
                    char **dst, const int unsigned maxdstbuf, const char delim,
                    const char escapech, int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose, const char *datahandleoptions)

/* returns: length if >=0;
 *          -1 if error;
 *          -2 if no more data (like sqlcode 100 in SQL)
 * dst : address of pointer to the buffer;
 * if pointer is null then procedure allocates this else given pointer is used
 */
{
  char buf[32767+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int i;
  int ch;
  int escaped;
  int empty_string;
  int overbuf;
  buffered_file *fd1;
  char errstr[ERR_BUF_SIZE];
  char *bufp;
  unsigned long maxbuf;

  if (sqltype==SQLTEXT || sqltype==SQLBYTES ) {
    if ((*dst)==NULL) {
      if (sqltype==SQLTEXT) {
        return onreadtextblob(fd, filename, (void *) &buf, sizeof(buf),
                      delim, escapech, eol, sqlind, rownum, loaded,
                      fieldnum, fieldname, max_fld_name, fieldname2,
                      logfile, onlogerror, verbose);
      } else {
        return onreadbyteblob(fd, filename, (void *) &buf, sizeof(buf),
                      delim, eol, sqlind, rownum, loaded,
                      fieldnum, fieldname, max_fld_name, fieldname2,
                      logfile, onlogerror, verbose);
      }
    } else {
      if (sqltype==SQLTEXT) {
        return onreadtextblob(fd, filename, (*dst), maxdstbuf, delim,
                      escapech, eol, sqlind, rownum, loaded, fieldnum,
                      fieldname, max_fld_name, fieldname2, logfile,
                      onlogerror, verbose);
      } else {
        return onreadbyteblob(fd, filename, (*dst), maxdstbuf, delim,
                      eol, sqlind, rownum, loaded, fieldnum,
                      fieldname, max_fld_name, fieldname2, logfile,
                      onlogerror, verbose);
      }
    }
  }
  if (sqltype==CFIXBINTYPE && (sqlxid==XID_BLOB || sqlxid==XID_CLOB)) {
    if ((*dst)==NULL) {
      return onreadsmartblob(fd, filename, sqlxid, (void *) &buf, sizeof(buf),
                    delim, escapech, eol, sqlind, rownum, loaded,
                    fieldnum, fieldname, max_fld_name, fieldname2,
                    logfile, onlogerror, verbose);
    } else {
      return onreadsmartblob(fd, filename, sqlxid, (*dst), maxdstbuf, delim,
                    escapech, eol, sqlind, rownum, loaded, fieldnum,
                    fieldname, max_fld_name, fieldname2, logfile,
                    onlogerror, verbose);
    }
  }
  fd1=(buffered_file *)fd;
  ch=myfgetc(fd1);
  i=0;
  escaped=0;
  empty_string=0;
  fieldname2[0]=0;
  overbuf=0;
  if ((*dst)==NULL ) {
    bufp=buf;
    maxbuf=sizeof(buf);
  } else {
    bufp=(*dst);
    maxbuf=maxdstbuf;
  }
  maxbuf--;
  
  while ( fd1->readbufmax ) {
    if ( !escaped ) {
      if ( ch==escapech ) {
        escaped=1;
        ch=myfgetc(fd1);
        if (i==0 && ch==' ') {
          empty_string=1;
        }
        continue;
      }
      if (ch=='\r') {
        ch=myfgetc(fd1);
        if (ch=='\n') {
          break;
        } else {
          if (i<maxbuf) {
            bufp[i]='\r';
            i++;
          } else {
            overbuf=1;
          }
          continue;
        }
      }
      if (ch=='\n') {
        break;
      }
      if (ch==delim) {
        break;
      }
    } else {
      escaped=0;
    }
    if (ch==-1 || ch==-2) {
      continue;
    }
    if (i<maxbuf) {
      bufp[i]=ch;
      i++;
    } else {
      overbuf=1;
    }
    ch=myfgetc(fd1);
  }
  if (ch==-2) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
    return -1;
  }
  if (overbuf) {
    bufp[maxbuf]=0;
    sprintf(errstr,"field %s (field no %d) longer than %ld at line %ld",
            fieldname, fieldnum, maxbuf, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  bufp[i]=0;
  if (i>1){
    empty_string=0;
  }
  if (empty_string) {
    i=0;
    bufp[0]=0;
  }
  if (ch==-2) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
    return -1;
  }
  /* sqlind should be set as following: -1 if value to be inserted is NULL,
                                         0 otherwise */
  /* In Informix unl format '| |' means field containing 1 space,
   *                        '||' means field containing NULL,
   *                        '|\ |' means empty string
   * This convention is used in dbaccess & dbload but not in HPL */
  
  if (bufp[0]=='\0' && !empty_string) { /* if NULL */
    *sqlind=-1;
    if ((*dst) != NULL) {
      (*dst)[0]='\0';
    }
  } else {
    if ((*dst) == NULL) {
      (*dst)=(char *)load2_malloc(i+1, __FILE__, __LINE__);
      if ((*dst) == NULL) {
        sprintf(errstr,"Out of memory");
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, 0,
                   logfile, verbose, 10);
        return -1;
      }
    }
    if (bufp!=(*dst)) {
      strcpy((*dst),bufp);
    }
    *sqlind=0;
  }
  if (ch == -1 || ch == '\n' ) {
    *eol = 1;
  } else {
    *eol = 0;
  }
  if (i==0) { /* value -2 have to be retuned only for the case when 
                 no field data read */
    if ( !fd1->readbufmax ) {
      return -2;
    }
  }
  return i;
}

int oninitgz(const char *fname, const char *tablename, const int batchsize,
             void **handle, const void *logfile, const int fieldsnum,
             const char *insstmt2, onlogerror_lpt onlogerror, int verbose,
             const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  gzFile f;
  buffered_file *p;
  void *p2;

  if ( (*handle) == NULL ) {
    if (fname[0]==0) {
      if (verbose>=10) {
        sprintf(errstr, "file=<stdin>, tablename=%s", tablename);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
                   verbose, 30);
      }
      f=gzdopen(0,"rb");
    } else {
      sprintf(errstr, "file=%s, tablename=%s", fname, tablename);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
                 verbose, 30);
      f = gzopen(fname, "rb");
    }
    if ( f == NULL ) {
      sprintf(errstr,"Cannot open file '%s:%d'", fname, errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 10);
      return 1;
    }
    p2=load2_malloc(READBUF_SIZE, __FILE__, __LINE__);
    if (p2==NULL) {
      gzclose(f);
      sprintf(errstr,"No memory for read buffer");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
                 verbose, 10);
      return 1;
    }
    p=(buffered_file*)load2_malloc(sizeof(buffered_file), __FILE__, __LINE__);
    if (p==NULL) {
      gzclose(f);
      load2_free(p2, __FILE__, __LINE__);
      sprintf(errstr,"Out of memory");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
                 verbose, 10);
      return 1;
    }

    p->fd=f;
    p->readbuf=p2;
    p->readbufind=-1;
    p->readbufsize=READBUF_SIZE;
    p->readbufmax=-1;
    (*handle)=p;
  }
  return 0;
}


void oncleanupgz(const char *fname, void *handle, const void *logfile,
                 onlogerror_lpt onlogerror, int verbose,
                 const char *datahandleoptions)
{ buffered_file * fd1;
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;

  fd1=(buffered_file *)handle;
  if (fd1!=NULL) {
    stat=gzclose(fd1->fd);
    if (stat==-1) {
      sprintf(errstr, "OS error %d while closing file", errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
                 logfile, verbose, 10);
    }
    load2_free(fd1->readbuf, __FILE__, __LINE__);
    fd1->readbuf=NULL;
    load2_free(fd1, __FILE__, __LINE__);
  }
}

int oncrlfgz(const void *fd, const char *filename,
             const unsigned long int rownum, const unsigned long int loaded,
             const int fields_number, const int bequiet, const char escapech,
             const void *logfile, onlogerror_lpt onlogerror, int verbose,
             const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int i;
  int ch;
  buffered_file *fd1;
  int escaped;

  fd1=(buffered_file *)fd;
  ch=myfgetc(fd1);
  i=0;
  escaped=0;
  while ( fd1->readbufmax) {
    if ( !escaped ) {
      if ( ch==escapech ) {
        i++;
        escaped=1;
        ch=myfgetc(fd1);
        continue;
      }
      if (ch=='\r') {
        ch=myfgetc(fd1);
        if (ch=='\n') {
          break;
        } else {
          i++;
          if (ch==escapech) {
            escaped=1;
            i++;
            ch=myfgetc(fd1);
            continue;
          }
        }
      }
      if (ch==-1 || ch==-2) {
        continue;
      }
      if (ch=='\n') {
        break;
      }
    } else {
      escaped=0;
    }
    i++;
    ch=myfgetc(fd1);
  }
  if (ch==-2 ) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
/*    perror("I/O error"); */
    return 1;
  }
  if (i>0 && !bequiet) {
    sprintf(errstr,
            "At line %ld: number of columns>expected %d : last %d characters",
            rownum,fields_number, i);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, 0,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

static int myfgetc(buffered_file *fd)
{ unsigned int ch;
  int i;
  /* i=getc(fd); */
  i=fd->readbufind;
  if (i<0 || i>=fd->readbufmax ) {
    fd->readbufmax=gzread(fd->fd, fd->readbuf, fd->readbufsize);
    if (fd->readbufmax==0) {
      return -1;
    }
    if (fd->readbufmax==-1) {
      fd->readbufmax=0;
      return -2;
    }
    ch=((unsigned char *)fd->readbuf)[0];
    fd->readbufind=1;
  } else {
    ch=((unsigned char *)fd->readbuf)[i];
    fd->readbufind++;
  }
  return ch;
}

