/******************************************************************************
*
* Standard event handlers for loading functions
*
* Author: Robert Sosnowski, Robert.Sosnowski@bzwbk.pl
*
* Version 1.70, 17-10-2005
*
* Copyright (c) 2001 - 2007. All Rights Reserved.
*
* Permission granted to use & distribute for free. Changes in source code
* require Author's acceptance.
*
* This software can be used only at your own risk. Author is not responsible 
* for any damage caused by using this software.
* 
******************************************************************************/

#include "load2cfg.h"
#include "tempfile.h"

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_CTYPE_H
#include <ctype.h>
#endif

#ifdef HAVE_IO_H
#include <io.h>
/* _mktemp, creat prototypes in MSVC */
#endif

#include <stdio.h>
#include "load2.h"

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <stdlib.h>

#include <fcntl.h>

EXEC SQL include locator;
EXEC SQL include sqltypes;

#ifndef HAVE_SNPRINTF
#ifdef HAVE__SNPRINTF
#define snprintf _snprintf
#else
#include <stdarg.h>
int snprintf(char *str, size_t count, ...)
{int i;
 va_list args;
 i= sprintf(str, args);
 va_end(args);
return i;
} 
#endif
#endif

#define READBUF_SIZE 10000
typedef struct {
  int fd;
  void *readbuf;
  int readbufind;
  int readbufsize;
  int readbufmax;
} buffered_file;

  

int oninit_std(const char *fname, const char *tablename, const int batchsize,
               void **handle, const void *logfile, const int fieldsnum,
               const char *insstmt2,
               onlogerror_lpt onlogerror, int verbose,
               const char *datahandleoptions);
int onbatch_std(const char *fname, const char *tablename, const void *handle,
              const unsigned long int rownum, const unsigned long int loaded,
              const unsigned long int inserted,
              const unsigned long int updated,
              const unsigned long int deleted, const void *logfile,
              onlogerror_lpt onlogerror, int verbose);
int onfinish_std(const char *fname, const char *tablename, void *handle,
                 const unsigned long int rownum,
                 const unsigned long int rownum2,
                 const unsigned long int inserted,
                 const unsigned long int updated,
                 const unsigned long int deleted,
                 const void *logfile,
                 onlogerror_lpt onlogerror, int verbose);
int onreadfield_std(const void *fd, const int sqltype, const int4 sqlxid,
                    const char *filename,
                    char **dst, const unsigned int maxdstbuf, const char delim,
                    const char escapech, int * eol, int * sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded, const int fieldnum,
                    const char * fieldname, const int max_fld_name,
                    char *fieldname2, const void *logfile,
                    onlogerror_lpt onlogerror, int verbose,
                    const char *datahandleoptions);
int onlogerror_std(const char * srcfile, const int pline, const char *errstr,
                   const long int sqlcode, const long int isamcode,
                   const char *ifx_str, const unsigned long int rownum,
                   const unsigned long int loaded, const int os_errno,
                   const void *logfile, int verbose, int urgency);
int oncrlf_std(const void *fd, const char *filename,
               const unsigned long int rownum, const unsigned long int loaded,
               const int fields_number, const int bequiet, const char escapech,
               const void *logfile,
               onlogerror_lpt onlogerror, int verbose,
               const char *daahandleoptions);
int onrecordbegin_std(const void *fd, const char *filename,
                      const unsigned long int rownum,
		      const unsigned long int loaded,
                      const int fields_number, const int bequiet,
                      const char escapech, const void *logfile,
                      onlogerror_lpt onlogerror, const int verbose,
                      const char *datahandleoptions);
int is_critical_std(const long int sqlcode, const long int isamcode,
                    const int errtype, const int errcnt, const int maxerrors,
                    int *errinc);

int onblobfree_std(loc_t *simple_lo, const void *logfile, onlogerror_lpt onlogerror,
                   int verbose);

static int onreadtextblob(const void *fd, const char *filename,
                    char *dst, const unsigned int maxdstbuf, const char delim,
                    const char escapech, int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose);

static int onreadbyteblob(const void *fd, const char *filename,
                    char *dst, const unsigned int maxdstbuf, const char delim,
                    int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose);

static int onreadsmartblob(const void *fd, const char *filename,
                    const int4 sqlxid, char *dst,
                    const unsigned int maxdstbuf, const char delim,
                    const char escapech, int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose);

static int myfgetc(buffered_file *fd);

static char *id="@(#) $Id: load2h.ec 2521 2007-11-21 08:55:56Z sosnor01 $";



int onblobfree_std(loc_t *simple_lo, const void *logfile,
                   onlogerror_lpt onlogerror, int verbose)
/* this function is called after succesfull insert
   currently it is not called in general cleanup procedure
   - to be fixed */
{
  if (simple_lo->loc_loctype==LOCMEMORY) {
    if (simple_lo->loc_buffer!=NULL) {
      free(simple_lo->loc_buffer);
    }
    simple_lo->loc_buffer=NULL;
    simple_lo->loc_size=0;
    simple_lo->loc_bufsize=0;
  }
  if (simple_lo->loc_loctype==LOCFNAME) {
    unlink(simple_lo->loc_fname);
    free(simple_lo->loc_fname);
    simple_lo->loc_fname=NULL;
  }
  return 0;
}

static int hexdigit(int c)
{ int n;

  if (isdigit(c)) {
    n=c-'0';    
  } else {
    if (c>='a' && c<='f') {
      n=c-'a'+10;
    } else {
      if (c>='A' && c<='F') {
        n=c-'A'+10;
      } else {
        n=-1;
      }
    }
  }
  return n;
}


static short hex2int8(char *hex, off_t *result)
{
  int i;
  short j;
  off_t k;
  int ch;

  i=0;
  k=0;
  while ((ch=hex[i++])) {
    j=hexdigit(ch);
    if (j==-1) {
      return -1;
    }
    k=(k<<4) + j;
  }
  *result=k;
  return 0;
}

static int onreadbyteblob(const void *fd, const char *filename,
                    char *dst, const unsigned int maxdstbuf, const char delim,
                    int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose)

/* returns: length if >=0;
 *          -1 if error;
 *          -2 if no more data (like sqlcode 100 in SQL)
 */
{
  unsigned int i, j, k;
  int ch;
  buffered_file *fd1;
  char errstr[ERR_BUF_SIZE];
  char *bufp, *blobbufp, *p;
  unsigned long maxbuf;
  loc_t simple_lo;
  char tmpfilename[256];
  FILE *ft;
  int destination;
  int parity, non_hex;
  char *envp;

  ft=NULL;
  fd1=(buffered_file *)fd;
  destination=0; /* memory */
  ch=myfgetc(fd1);
  i=0;
  fieldname2[0]=0;
  bufp=dst;
  maxbuf=maxdstbuf-1;
#if defined(HAVE_GETENV)
  envp=getenv("DBBLOBBUF");
  if (envp!=NULL && isdigit(envp[0])) {
    k=atol(envp);
    if (k*1024<maxdstbuf) {
      maxbuf=k*1024;
    }
  }
#endif
  parity=0;
  non_hex=0;
  k=0;
  

  while ( fd1->readbufmax ) {
    if (i>=maxbuf) {
      if (destination==0) {
        destination=1; /* file */
        p=creatmpfile();
        strcpy(tmpfilename, p);
        if (tmpfilename==NULL) {
          i=errno;
          sprintf(errstr,"can't create temporary file at line %ld, errno=%d (%s)",
                  rownum, i, tmpfilename);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                     logfile, verbose, 10);
          return -1;
        }
        ft=fopen(tmpfilename, "wb");
        if (ft==NULL) {
          i=errno;
          sprintf(errstr,"can't open temporary file at line %ld, errno=%d (%s)",
                  rownum, i, tmpfilename);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                     logfile, verbose, 10);
          return -1;
        }
      }
      i=fwrite(bufp,1, i, ft);
      if (ferror(ft) ) {
        i=errno;
        sprintf(errstr,"can't write temporary file at line %ld, errno=%d (%s)",
                rownum, i, tmpfilename);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                   logfile, verbose, 10);
        return -1;
      }
      i=0;
    }
    if (ch=='\r') {
      ch=myfgetc(fd1);
      if (ch=='\n') {
        break;
      } else {
        bufp[i]='\r';
        i++;
        continue;
      }
    }
    if (ch=='\n') {
      break;
    }
    if (ch==delim) {
      break;
    }
    if (ch==-1 || ch==-2) {
      continue;
    }
    j=(unsigned int)hexdigit(ch);
    if (j==-1) {
      non_hex=1;
    }
    if (!parity) {
      k=j<<4;
    } else {
      k=k | j;
      bufp[i]=k;
      i++;
    }
    parity=!parity;
    ch=myfgetc(fd1);
  }
  if (ch==-2) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
    return -1;
  }
  if (non_hex) {
    sprintf(errstr,"non-hex BYTE at line %ld", rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, 0,
               logfile, verbose, 10);
    return -1;
  }
  if (parity!=0) {
    j=k>>4;
    bufp[i]=j;
    i++;
  }
  k=i;
  if (destination==1) {
    i=fwrite(bufp,1, i, ft);
    if (ferror(ft) ) {
      i=errno;
      sprintf(errstr,"can't write temporary file at line %ld, errno=%d (%s)",
              rownum, i, tmpfilename);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                 logfile, verbose, 10);
      return -1;
    }
    i=fclose(ft);
    if ( i != 0 ) {
      i=errno;
      sprintf(errstr,"can't close temporary file at line %ld, errno=%d (%s)",
                      rownum, i, tmpfilename);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                 logfile, verbose, 10);
      return -1;
    }
  }
  if (destination==0) {
    blobbufp=NULL;
    if (i>0) {
      blobbufp=(char *)load2_malloc(k+1, __FILE__, __LINE__);
      if (blobbufp==NULL) {
        sprintf(errstr,"Out of memory");
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
                   0, logfile, verbose, 10);
        return -1;
      }
      bycopy(bufp,blobbufp,k);
    }
    simple_lo.loc_loctype=LOCMEMORY;
    simple_lo.loc_bufsize=k+1;
    simple_lo.loc_size=k;
    simple_lo.loc_buffer=blobbufp;
    simple_lo.loc_mflags=0;
    simple_lo.loc_oflags=0;
  } else { /* destination */
    blobbufp=NULL;
    i=strlen(tmpfilename);
    blobbufp=(char *)load2_malloc(i+1, __FILE__, __LINE__);
    if (blobbufp==NULL) {
      sprintf(errstr,"Out of memory");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
                 0, logfile, verbose, 10);
      return -1;
    }
    strcpy(blobbufp,tmpfilename);
    simple_lo.loc_loctype=LOCFNAME;
    simple_lo.loc_fname=blobbufp;
    simple_lo.loc_oflags=LOC_RONLY;
    simple_lo.loc_size=-1;
  }
  if (k==0 && simple_lo.loc_loctype==LOCMEMORY) {
    simple_lo.loc_indicator= -1; /* Null */
    *sqlind=-1;
    simple_lo.loc_bufsize=0;
    simple_lo.loc_buffer=NULL;
  } else {
    simple_lo.loc_indicator= k;
    *sqlind=0;
  }
  simple_lo.loc_type=SQLBYTES;
  bycopy((void *)&simple_lo, dst, sizeof(loc_t));

  if (ch == -1 || ch == '\n' ) {
    *eol = 1;
  } else {
    *eol = 0;
  }
  if (k==0 && destination==0) { /* value -2 have to be retuned only for the case when 
                                   no field data read */
    if ( !fd1->readbufmax ) {
      return -2;
    }
  }
  if (destination!=0 && k==0) {
    k=1;
  }
  return k; /* ok only for destination==0 */
}


static int onreadtextblob(const void *fd, const char *filename,
                    char *dst, const unsigned int maxdstbuf, const char delim,
                    const char escapech, int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose)

/* returns: length if >=0;
 *          -1 if error;
 *          -2 if no more data (like sqlcode 100 in SQL)
 */
{
  unsigned int i, k;
  int ch;
  int escaped;
  int empty_string;
  buffered_file *fd1;
  char errstr[ERR_BUF_SIZE];
  char *bufp, *blobbufp, *p;
  unsigned long maxbuf;
  loc_t simple_lo;
  char tmpfilename[256];
  FILE * ft;
  int destination;
  char *envp;

  ft=NULL;
  fd1=(buffered_file *)fd;
  destination=0; /* memory */
  ch=myfgetc(fd1);
  i=0;
  escaped=0;
  empty_string=0;
  fieldname2[0]=0;
  bufp=dst;
  maxbuf=maxdstbuf-1;
#if defined(HAVE_GETENV)
  envp=getenv("DBBLOBBUF");
  if (envp!=NULL && isdigit(envp[0])) {
    k=atol(envp);
    if (k*1024<maxdstbuf) {
      maxbuf=k*1024;
    }
  }
#endif
  
  while ( fd1->readbufmax) {
    if (i>=maxbuf) {
      bufp[maxbuf]=0;
      if (destination==0) {
        destination=1; /* file */
        p=creatmpfile();
        strcpy(tmpfilename, p);
        if (tmpfilename==NULL) {
          i=errno;
          sprintf(errstr,"can't create temporary file at line %ld, errno=%d (%s)",
                  rownum, i, tmpfilename);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                     logfile, verbose, 10);
          return -1;
        }
        ft=fopen(tmpfilename, "wb");
        if (ft==NULL) {
          i=errno;
          sprintf(errstr,"can't open temporary file at line %ld, errno=%d (%s)",
                  rownum, i, tmpfilename);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                     logfile, verbose, 10);
          return -1;
        }
      }
      i=fwrite(bufp,1, maxbuf, ft);
      if (ferror(ft) ) {
        i=errno;
        sprintf(errstr,"can't write temporary file at line %ld, errno=%d (%s)",
                rownum, i, tmpfilename);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                   logfile, verbose, 10);
        return -1;
      }
      i=0;
    }
    if ( !escaped ) {
      if ( ch==escapech ) {
        escaped=1;
        ch=myfgetc(fd1);
        if (i==0 && ch==' ') {
          empty_string=1;
        }
        continue;
      }
/* commented out because:
   If TEXT field is last field in the row then it can contain \r or \n fields inside its content.
   Therefore neither \n nor \r can be treated as end of such a field.

      if (ch=='\r') {
        ch=myfgetc(fd1);
        if (ch=='\n') {
          break;
        } else {
          bufp[i]='\r';
          i++;
          continue;
        }
      } 
      if (ch=='\n') {
        break;
      } */
      if (ch==delim) {
        break;
      }
    } else {
      escaped=0;
    }
    if (ch==-1 || ch==-2 ) {
      continue;
    }
    bufp[i]=ch;
    i++;
    ch=myfgetc(fd1);
  }
  if (ch==-2) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
    return -1;
  }
  if (destination==1) {
    i=fwrite(bufp,1, i, ft);
    if (ferror(ft) ) {
      i=errno;
      sprintf(errstr,"can't write temporary file at line %ld, errno=%d (%s)",
              rownum, i, tmpfilename);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                 logfile, verbose, 10);
      return -1;
    }
    i=fclose(ft);
    if ( i != 0 ) {
      i=errno;
      sprintf(errstr,"can't close temporary file at line %ld, errno=%d (%s)",
              rownum, i, tmpfilename);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, i,
                 logfile, verbose, 10);
      return -1;
    }
  }
  bufp[i]=0;
  if (i>1){
    empty_string=0;
  }
  if (empty_string) {
    i=0;
    bufp[0]=0;
  }
  if (destination==0) {
    blobbufp=NULL;
    if (i>0) {
      blobbufp=(char *)load2_malloc(i+1, __FILE__, __LINE__);
      if (blobbufp==NULL) {
        sprintf(errstr,"Out of memory");
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
           0, logfile, verbose, 10);
        return -1;
      }
      memcpy(blobbufp,bufp,i);
    }
    simple_lo.loc_loctype=LOCMEMORY;
    simple_lo.loc_bufsize=i+1;
    simple_lo.loc_size=i;
    simple_lo.loc_buffer=blobbufp;
    simple_lo.loc_mflags=0;
    simple_lo.loc_oflags=0;
  } else { /* destination */
    blobbufp=NULL;
    i=strlen(tmpfilename);
    blobbufp=(char *)load2_malloc(i+1, __FILE__, __LINE__);
    if (blobbufp==NULL) {
      sprintf(errstr,"Out of memory");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
         0, logfile, verbose, 10);
      return -1;
    }
    strcpy(blobbufp,tmpfilename);
    simple_lo.loc_loctype=LOCFNAME;
    simple_lo.loc_fname=blobbufp;
    simple_lo.loc_oflags=LOC_RONLY;
    simple_lo.loc_size=-1;
  }
  if (bufp[0]==0 && !empty_string && simple_lo.loc_loctype==LOCMEMORY) {
    simple_lo.loc_indicator= -1; /* Null */
    simple_lo.loc_bufsize=0;
    simple_lo.loc_buffer=NULL;
    *sqlind=-1;
  } else {
    simple_lo.loc_indicator= i;
    *sqlind=0;
  }
  simple_lo.loc_type=SQLTEXT;
  bycopy((void *)&simple_lo, dst, sizeof(loc_t));

  if (ch == -1 || ch == '\n' ) {
    *eol = 1;
  } else {
    *eol = 0;
  }
  if (i==0 && destination==0) { /* value -2 have to be retuned only for the case when 
                                   no field data read */
    if ( !fd1->readbufmax ) {
      return -2;
    }
  }
  if (destination!=0 && i==0) {
    i=1;
  }
  return i; /* ok only for destination==0 */
}

static int onreadsmartblob(const void *fd, const char *filename,
                    const int4 sqlxid, char *dst,
                    const unsigned int maxdstbuf, const char delim,
                    const char escapech, int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose)
/* returns: length if >=0;
 *          -1 if error;
 *          -2 if no more data (like sqlcode 100 in SQL)
 */
{
  char errstr[ERR_BUF_SIZE];
  mint lofd;
  int4 ifx_error;
  mint ifx_error2;
  off_t fsize64;
  FILE *f;
  buffered_file *fd1;
  char filename2[256];
  unsigned int stat;
  ifx_lo_create_spec_t *create_spec;
  off_t offset8;
  char buf[MAXMEMBLOB+1];
  ifx_lo_t descr;
  char ch;
  unsigned int i;
  short overbuf;
  char *bufp, *p1, *p2;
  unsigned int maxbuf;
  int numbytes;
  int buflen;
  char hex1[20], hex2[20];
  int n;
  ifx_int8_t t1;

  fd1=(buffered_file *)fd;
  ch=myfgetc(fd1);
  i=0;
  fieldname2[0]=0;
  overbuf=0;
  bufp=dst;
  maxbuf=maxdstbuf;
  maxbuf--;
  
  while ( fd1->readbufmax ) {
    if (ch=='\r') { /* \r\n recognized as end-of-field */
      ch=myfgetc(fd1);
      if (ch=='\n') {
        break;
      } else {
        if (i<maxbuf) {
          bufp[i]='\r';
          i++;
        } else {
          overbuf=1;
        }
        continue;
      }
    }
    if (ch=='\n') { /* \n recognized as end-of-field */
      break;
    }
    if (ch==delim) {
      break;
    }
    if (ch==-1 || ch==-2) {
      continue;
    }
    if (i<maxbuf) {
      bufp[i]=ch;
      i++;
    } else {
      overbuf=1;
    }
    ch=myfgetc(fd1);
  }
  if (ch==-2) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
    return -1;
  }
  if (overbuf) {
    bufp[maxbuf]=0;
    sprintf(errstr,"field %s (field no %d) longer than %ud at line %ld",
            fieldname, fieldnum, maxbuf, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  bufp[i]=0;
  if (bufp[0]==0) { /* if NULL */
    *sqlind=-1;
  } else {
    *sqlind=0;
  }
  if (ch == -1 || ch == '\n' ) {
    *eol = 1;
  } else {
    *eol = 0;
  }
  if (i==0) { /* value -2 have to be retuned only for the case when 
                 no field data read */
    if ( !fd1->readbufmax ) {
      return -2;
    }
  }
  if (bufp[0]==0) { /* field is NULL */
    return 0;
  }
  p1=strchr(bufp,',');
  if (p1==NULL) {
    sprintf(errstr,"Smart blob field doesn't contain ','");
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  i=p1-bufp-1;
  if (i>sizeof(hex1)-1) {
    sprintf(errstr,"Too long hex value in string: %s", bufp);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  (*p1)=0;
  strcpy(hex1, bufp);
  (*p1)=',';
  p1++;
  p2=strchr(p1,',');
  if (p2==NULL) {
    sprintf(errstr,"Smart blob field doesn't contain second ','");
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  i=p2-p1-1;
  if (i>sizeof(hex2)-1) {
    sprintf(errstr,"Too long second hex value in string: %s", bufp);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  (*p2)=0;
  strcpy(hex2, p1);
  (*p2)=',';
  p2++;
  strcpy(filename2, p2++);
  
  stat=hex2int8(hex1, &offset8);
  if (stat==-1) {
    sprintf(errstr,"Non hex digit in hex number: %s", hex1);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  stat=hex2int8(hex2, &fsize64);
  if (stat==-1) {
    sprintf(errstr,"Non hex digit in hex number: %s", hex2);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  ifx_error = ifx_lo_def_create_spec(&create_spec);
  if (ifx_error < 0) {
    sprintf(errstr,"ERROR ifx_lo_def_create_spec:"INT4_FORMAT","INT4_FORMAT",%s",
                   ifx_error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, ifx_error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, loaded, 0, logfile, verbose, 10);
    return -1;
  }

  ifx_error=off_t2int8(fsize64, &t1);
  if (ifx_error < 0) {
    sprintf(errstr,"ERROR converting off_t into int8:"INT4_FORMAT"", ifx_error);
    onlogerror(__FILE__, __LINE__, errstr, ifx_error, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  ifx_error=ifx_lo_specset_estbytes(create_spec, &t1);
  if (ifx_error < 0) {
    sprintf(errstr,"ERROR ifx_lo_specset_estbytes:"INT4_FORMAT","INT4_FORMAT",%s",
                   ifx_error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, ifx_error, sqlca.sqlerrd[1],
       sqlca.sqlerrm, rownum, loaded, 0, logfile, verbose, 10);
    return -1;
  }
  
/* Create an LO-specification structure for the smart large object */
  if ((lofd = ifx_lo_create(create_spec, LO_WRONLY, &descr, &ifx_error2)) == -1) {
    sprintf(errstr,"ERROR ifx_lo_create:%d,"INT4_FORMAT",%s", ifx_error2, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, ifx_error2, sqlca.sqlerrd[1], sqlca.sqlerrm, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }

/* Copy data into the open smart
 * large object that lofd points to. */
  f=fopen(filename2,"rb");
  if (f==NULL) {
    sprintf(errstr,"OS error in fopen: %d", errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       errno, logfile, verbose, 10);
    return -1;
  }

  stat=fseek8(f, offset8, SEEK_SET);
  if (stat!=0) {
    sprintf(errstr,"OS error in fseek:%d", errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       errno, logfile, verbose, 10);
    return -1;
  }
  while (fsize64>0) {
    if (MAXMEMBLOB-1>fsize64) {
      n=fsize64;
    } else {
      n=MAXMEMBLOB-1;
    }
    buflen=fread(buf,1,n, f);
    if (buflen<n) {
      if (feof(f)) {
        sprintf(errstr,"unexpected eof in freed");
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
           errno, logfile, verbose, 10);
        return -1;
      } else {
        sprintf(errstr,"OS error in fread:%d", errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
           errno, logfile, verbose, 10);
        return -1;
      }
    }

    numbytes = ifx_lo_write(lofd, buf, buflen, &ifx_error2);
    if ( numbytes < buflen ) {
      sprintf(errstr,"ERROR ifx_lo_write:%d,"INT4_FORMAT",%s",
                     ifx_error2, sqlca.sqlerrd[1], sqlca.sqlerrm);
      onlogerror(__FILE__, __LINE__, errstr, ifx_error2, 0, NULL, rownum, loaded,
         errno, logfile, verbose, 10);
      return -1;
    }
    fsize64-=n;
  }
  fclose(f);
/* Close the LO file descriptor */
  ifx_lo_close(lofd);
/* Free LO-specification structure */
  ifx_lo_spec_free(create_spec);
  bycopy((char *) &descr, bufp, sizeof(descr));
  return sizeof(descr);
}

int onreadfield_std(const void *fd, const int sqltype, const int4 sqlxid,
                    const char *filename,
                    char **dst, const int unsigned maxdstbuf, const char delim,
                    const char escapech, int *eol, int *sqlind,
                    const unsigned long int rownum,
                    const unsigned long int loaded,
                    const int fieldnum, const char * fieldname,
                    const int max_fld_name, char *fieldname2,
                    const void *logfile, onlogerror_lpt onlogerror,
                    int verbose, const char *datahandleoptions)

/* returns: length if >=0;
 *          -1 if error;
 *          -2 if no more data (like sqlcode 100 in SQL)
 * dst : address of pointer to the buffer;
 * if pointer is null then procedure allocates this else given pointer is used
 */
{
  char buf[32767+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int i;
  int ch;
  int escaped;
  int empty_string;
  int overbuf;
  buffered_file *fd1;
  char errstr[ERR_BUF_SIZE];
  char *bufp;
  unsigned long maxbuf;

  if (sqltype==SQLTEXT || sqltype==SQLBYTES ) {
    if ((*dst)==NULL) {
      if (sqltype==SQLTEXT) {
        return onreadtextblob(fd, filename, (void *) &buf, sizeof(buf),
                      delim, escapech, eol, sqlind, rownum, loaded,
                      fieldnum, fieldname, max_fld_name, fieldname2,
                      logfile, onlogerror, verbose);
      } else {
        return onreadbyteblob(fd, filename, (void *) &buf, sizeof(buf),
                      delim, eol, sqlind, rownum, loaded,
                      fieldnum, fieldname, max_fld_name, fieldname2,
                      logfile, onlogerror, verbose);
      }
    } else {
      if (sqltype==SQLTEXT) {
        return onreadtextblob(fd, filename, (*dst), maxdstbuf, delim,
                      escapech, eol, sqlind, rownum, loaded, fieldnum,
                      fieldname, max_fld_name, fieldname2, logfile,
                      onlogerror, verbose);
      } else {
        return onreadbyteblob(fd, filename, (*dst), maxdstbuf, delim,
                      eol, sqlind, rownum, loaded, fieldnum,
                      fieldname, max_fld_name, fieldname2, logfile,
                      onlogerror, verbose);
      }
    }
  }
  if (sqltype==CFIXBINTYPE && (sqlxid==XID_BLOB || sqlxid==XID_CLOB)) {
    if ((*dst)==NULL) {
      return onreadsmartblob(fd, filename, sqlxid, (void *) &buf, sizeof(buf),
                    delim, escapech, eol, sqlind, rownum, loaded,
                    fieldnum, fieldname, max_fld_name, fieldname2,
                    logfile, onlogerror, verbose);
    } else {
      return onreadsmartblob(fd, filename, sqlxid, (*dst), maxdstbuf, delim,
                    escapech, eol, sqlind, rownum, loaded, fieldnum,
                    fieldname, max_fld_name, fieldname2, logfile,
                    onlogerror, verbose);
    }
  }

  fd1=(buffered_file *)fd;
  ch=myfgetc(fd1);
  i=0;
  escaped=0;
  empty_string=0;
  fieldname2[0]=0;
  overbuf=0;
  if ((*dst)==NULL ) {
    bufp=buf;
    maxbuf=sizeof(buf);
  } else {
    bufp=(*dst);
    maxbuf=maxdstbuf;
  }
  maxbuf--;
  
  while ( fd1->readbufmax ) {
    if ( !escaped ) {
      if ( ch==escapech ) {
        escaped=1;
        ch=myfgetc(fd1);
        if (i==0 && ch==' ') {
          empty_string=1;
        }
        continue;
      }
      if (ch=='\r' && sqltype!=CCHARTYPE && sqltype!=CVCHARTYPE
      && sqltype!=CLVCHARTYPE) {
/* \r\n recognized as end-of-field only in case of non-char fields,
   in case of character fields \r always belongs to the field content */
        ch=myfgetc(fd1);
        if (ch=='\n') {
          break;
        } else {
          if (i<maxbuf) {
            bufp[i]='\r';
            i++;
          } else {
            overbuf=1;
          }
          continue;
        }
      }
      if (ch=='\n' && sqltype!=CCHARTYPE && sqltype!=CVCHARTYPE
      && sqltype!=CLVCHARTYPE) {
/* \n recognized as end-of-field only in case of non-char fields,
   in case of character fields \n always belongs to the field content */
        break;
      }
      if (ch==delim) {
        break;
      }
    } else {
      escaped=0;
    }
    if (ch==-1 || ch==-2) {
      continue;
    }
    if (i<maxbuf) {
      bufp[i]=ch;
      i++;
    } else {
      overbuf=1;
    }
    ch=myfgetc(fd1);
  }
  if (ch==-2) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
    return -1;
  }
  if (overbuf) {
    bufp[maxbuf]=0;
    sprintf(errstr,"field %s (field no %d) longer than %ld at line %ld",
            fieldname, fieldnum, maxbuf, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
       0, logfile, verbose, 10);
    return -1;
  }
  bufp[i]=0;
  if (i>1){
    empty_string=0;
  }
  if (empty_string) {
    i=0;
    bufp[0]=0;
  }
  if (ch==-2) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
    return -1;
  }
  /* sqlind should be set as following: -1 if value to be inserted is NULL,
                                         0 otherwise */
  /* In Informix unl format '| |' means field containing 1 space,
   *                        '||' means field containing NULL,
   *                        '|\ |' means empty string
   * This convention is used in dbaccess & dbload but not in HPL */
  
  if (bufp[0]=='\0' && !empty_string) { /* if NULL */
    *sqlind=-1;
    if ((*dst) != NULL) {
      (*dst)[0]='\0';
    }
  } else {
    if ((*dst) == NULL) {
      (*dst)=(char *)load2_malloc(i+1, __FILE__, __LINE__);
      if ((*dst) == NULL) {
        sprintf(errstr,"Out of memory");
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, 0,
                   logfile, verbose, 10);
        return -1;
      }
    }
    if (bufp!=(*dst)) {
      strcpy((*dst),bufp);
    }
    *sqlind=0;
  }
  if (ch == -1 || ch == '\n' ) {
    *eol = 1;
  } else {
    *eol = 0;
  }
  if (i==0) { /* value -2 have to be retuned only for the case when 
                 no field data read */
    if ( !fd1->readbufmax ) {
      return -2;
    }
  }
  return i;
}

int oninit_std(const char *fname, const char *tablename, const int batchsize,
               void **handle, const void *logfile, const int fieldsnum,
               const char *insstmt2, onlogerror_lpt onlogerror, int verbose,
               const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  int f;
  buffered_file *p;
  void *p2;

  if ( (*handle) == NULL ) {
    if (fname[0]==0) {
      if (verbose>=10) {
        sprintf(errstr, "file=<stdin>, tablename=%s", tablename);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
                   verbose, 30);
      }
      f=0;
    } else {
      sprintf(errstr, "file=%s, tablename=%s", fname, tablename);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
                 verbose, 30);
/* use O_BINARY if such an option is available; if it is not available then usually it is by default */

#ifdef O_LARGEFILE
#  ifdef O_BINARY
       f = open(fname, O_RDONLY|O_LARGEFILE|O_BINARY);
#  else
       f = open(fname, O_RDONLY|O_LARGEFILE);
#  endif
#else
#  ifdef O_BINARY
       f = open(fname, O_RDONLY|O_BINARY);
#  else
       f = open(fname, O_RDONLY);
#  endif
#endif
    }
    if ( f == -1 ) {
      sprintf(errstr,"Cannot open file '%s:%d'", fname, errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 10);
      return 1;
    }
    p2=load2_malloc(READBUF_SIZE, __FILE__, __LINE__);
    if (p2==NULL) {
      close(f);
      sprintf(errstr,"No memory for read buffer");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
                 verbose, 10);
      return 1;
    }
    p=(buffered_file*)load2_malloc(sizeof(buffered_file), __FILE__, __LINE__);
    if (p==NULL) {
      close(f);
      load2_free(p2, __FILE__, __LINE__);
      sprintf(errstr,"Out of memory");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
                 verbose, 10);
      return 1;
    }

    p->fd=f;
    p->readbuf=p2;
    p->readbufind=-1;
    p->readbufsize=READBUF_SIZE;
    p->readbufmax=-1;
    (*handle)=p;
  }
  return 0;
}

int onbatch_std(const char *fname, const char *tablename, const void *handle,
                const unsigned long int rownum, const unsigned long int loaded,
                const unsigned long int inserted,
                const unsigned long int updated,
                const unsigned long int deleted,
                const void *logfile, onlogerror_lpt onlogerror, int verbose)
{
  char errstr[ERR_BUF_SIZE];

  if (updated==0 && deleted==0) {
    sprintf(errstr, "Loaded succesfully %ld records at row %ld", loaded, rownum);
  } else {
    if (deleted==0) {
      sprintf(errstr, "Inserted %ld, updated %ld at row %ld", inserted,
              updated, rownum);
    } else {
      sprintf(errstr, "Inserted %ld, updated %ld, deleted %ldat row %ld",
              inserted, updated, deleted, rownum);
    }
  }
  onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
             verbose, 40);
  return 0;
}

int onfinish_std(const char *fname, const char *tablename, void *handle,
                 const unsigned long int rownum,
                 const unsigned long int rownum2,
                 const unsigned long int inserted,
                 const unsigned long int updated,
                 const unsigned long int deleted,
                 const void *logfile,
                 onlogerror_lpt onlogerror, int verbose)
{
  char errstr[ERR_BUF_SIZE];

  if (updated==0 && deleted==0) {
    snprintf(errstr, ERR_BUF_SIZE, "%ld records were read, succesfully loaded %ld" ,
             rownum, rownum2);
  } else {
    if (deleted==0) {
      snprintf(errstr, ERR_BUF_SIZE, "%ld records were read, succesfully inserted %ld, updated %ld" ,
              rownum, inserted, updated);
    } else {
      snprintf(errstr, ERR_BUF_SIZE, "%ld records were read, succesfully inserted %ld, updated %ld, deleted %ld" ,
              rownum, inserted, updated, deleted);
    }
  }
  onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
             verbose, 30);
  return 0;
}


void oncleanup_std(const char *fname, void *handle, const void *logfile,
                   onlogerror_lpt onlogerror, int verbose,
                   const char *datahandleoptions)
{ buffered_file * fd1;
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;

  fd1=(buffered_file *)handle;
  if (fd1!=NULL && fd1->fd!=0) { /* 0 = stdin */
    stat=close(fd1->fd);
    if (stat==-1) {
      sprintf(errstr, "OS error %d while closing file", errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
                 logfile, verbose, 10);
    }
    load2_free(fd1->readbuf, __FILE__, __LINE__);
    fd1->readbuf=NULL;
    load2_free(fd1, __FILE__, __LINE__);
  }
}

int onlogerror_std(const char * srcfile, const int pline, const char *errstr,
                   const long int sqlcode, const long int isamcode,
                   const char *ifx_str, const unsigned long int rownum,
                   const unsigned long int processed, const int os_errno,
                   const void *logfile, int verbose, int urgency)
{ FILE *f;
  f=(FILE *)logfile;
  if (urgency<=verbose && verbose!=0) {
    if (f==NULL) {
      if (urgency>=30) {
        fprintf(stdout, errstr);
        fprintf(stdout, "\n");
      } else { /* error messages */
        fprintf(stderr, "ERROR:%s\n", errstr);
        if (verbose>=50) {
          fprintf(stderr, "File: %s, line:%d\n",srcfile, pline);
        }
      }
    } else {
      if (urgency>=30) {
        fprintf(f, errstr);
        fprintf(f, "\n");
        fprintf(stdout, errstr);
        fprintf(stdout, "\n");
      } else { /* error messages */
        fprintf(f, "ERROR:%s\n", errstr);
        fprintf(stderr, "ERROR:%s\n", errstr);
        if (verbose>=50) {
          fprintf(f, "File: %s, line:%d\n",srcfile, pline);
          fprintf(stderr, "File: %s, line:%d\n",srcfile, pline);
        }
      }
    }
  }
  return 0;
}

int oncrlf_std(const void *fd, const char *filename,
               const unsigned long int rownum, const unsigned long int loaded,
               const int fields_number, const int bequiet, const char escapech,
               const void *logfile, onlogerror_lpt onlogerror, int verbose,
               const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int i;
  int ch;
  buffered_file *fd1;
  int escaped;

  fd1=(buffered_file *)fd;
  ch=myfgetc(fd1);
  i=0;
  escaped=0;
  while ( fd1->readbufmax) {
    if ( !escaped ) {
      if ( ch==escapech ) {
        i++;
        escaped=1;
        ch=myfgetc(fd1);
        continue;
      }
      if (ch=='\r') {
        ch=myfgetc(fd1);
        if (ch=='\n') {
          break;
        } else {
          i++;
          if (ch==escapech) {
            escaped=1;
            i++;
            ch=myfgetc(fd1);
            continue;
          }
        }
      }
      if (ch==-1 || ch==-2) {
        continue;
      }
      if (ch=='\n') {
        break;
      }
    } else {
      escaped=0;
    }
    i++;
    ch=myfgetc(fd1);
  }
  if (ch==-2 ) { /* I/O error */
    sprintf(errstr, "OS error %d at line %ld",errno, rownum);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, errno,
       logfile, verbose, 10);
/*    perror("I/O error"); */
    return 1;
  }
  if (i>0 && !bequiet) {
    sprintf(errstr,
            "At line %ld: number of columns>expected %d : last %d characters",
            rownum,fields_number, i);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded, 0,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int is_critical_std(const long int sqlcode, const long int isamcode,
                    const int errtype, const int errcnt, const int maxerrors,
		    int *errinc)
{
/* errtype:
   0 - SQL error
   1 - inserting null into non-null column
   2 - conversion from ASCII format failed
   3 - error while reading (onreadfield, oncrlf)
   4 - wrong number of columns
   warning: if you set to ignore all errors then it may happen that
            procedure will go into infinite loop - this is because after
    an error procedure reads a file and encounters the same problem
   so: you can ignore all I/O errors if you want but upper limit of such errors
       should exist
*/
  (*errinc)=1;
  if (errcnt>=maxerrors) { /* test before errcnt++ */
    return 1;
  }
  return 0;
}

int onrecordbegin_std(const void *fd, const char *filename,
                      const unsigned long int rownum,
                      const unsigned long int loaded, const int fields_number,
                      const int bequiet, const char escapech,
                      const void *logfile, onlogerror_lpt onlogerror,
                      const int verbose, const char *datahandleoptions)
{
  return 0;
}

static int myfgetc(buffered_file *fd)
{ unsigned int ch;
  long int i;
  /* i=getc(fd); */
  i=fd->readbufind;
  if (i<0 || i>=fd->readbufmax ) {
    fd->readbufmax=read(fd->fd, fd->readbuf, fd->readbufsize);
    if (fd->readbufmax==0) {
      return -1;
    }
    if (fd->readbufmax==-1) {
      fd->readbufmax=0;
      return -2;
    }
    ch=((unsigned char *)fd->readbuf)[0];
    fd->readbufind=1;
  } else {
    ch=((unsigned char *)fd->readbuf)[i];
    fd->readbufind++;
  }
  return ch;
}

