/******************************************************************************
*
* Utility for updating database from .unl files
*
* Author: Robert Sosnowski, Robert.Sosnowski@bzwbk.pl
*
* Version 1.70, 17-10-2005
*
* Copyright (c) 2001 - 2007. All Rights Reserved.
*
* Permission granted to use & distribute for free.
*
* This software can be used only at your own risk. Author is not responsible 
* for any damage caused by using this software.
* 
******************************************************************************/

#include "load2cfg.h"

#ifndef HAVE_GETOPT
extern int      optind;
extern char    *optarg;
extern int	opterr;
int getopt(int argc, char *argv[], char *optionS);
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "load2.h"
#include <time.h>

extern char * optarg;
extern int optind;
static char *id="@(#) $Id: loadinc.ec 2521 2007-11-21 08:55:56Z sosnor01 $";

int usage( char * );
int loadinc (const char * insstmt2, const char *fname, const char delim,
             const int batchsize, const int usecommit,
             const int maxerrors,  unsigned long int *loaded_rows,
             const void *logfile, const char *collist, const short uploadmode,
             const short uploadmode2, const short verbosity,
             const short gzexpr);

void oncleanup_std(const char *fname, void *handle, const void *logfile,
                   onlogerror_lpt onlogerror, int verbose,
                   const char *datahandleoptions);

#ifdef HAVE_ZLIB
int oninitgz(const char *fname, const char *tablename, const int batchsize,
             void **handle, const void *logfile, const int fieldsnum,
             const char *insstmt2,
             onlogerror_lpt onlogerror, int verbose,
             const char *datahandleoptions);
int onreadfieldgz(const void *fd, const int sqltype, const int4 sqlxid,
                  const char *filename,
                  char **dst, const unsigned int maxdstbuf, const char delim,
                  const char escapech, int * eol, int * sqlind,
                  const unsigned long int rownum,
                  const unsigned long int loaded, const int fieldnum,
                  const char * fieldname, const int max_fld_name,
                  char *fieldname2, const void *logfile,
                  onlogerror_lpt onlogerror, const int verbose,
                  const char *datahandleoptions);
int oncrlfgz(const void *fd, const char *filename,
             const unsigned long int rownum, const unsigned long int loaded,
             const int fields_number, const int bequiet, const char escapech,
             const void *logfile,
             onlogerror_lpt onlogerror, int verbose,
             const char *datahandleoptions);
void oncleanupgz(const char *fname, void *handle, const void *logfile,
                 onlogerror_lpt onlogerror, int verbose,
                 const char *datahandleoptions);
#endif



int main( int argc, char **argv )
{

  EXEC SQL BEGIN DECLARE SECTION;
  static char dbname[TBLNAME_SIZE];
  static char server[TBLNAME_SIZE];
  static char server2[TBLNAME_SIZE+1];
  static char user[USERNAME_SIZE];
  static char password[USERNAME_SIZE];
  static char  tabname[TBLNAME_SIZE];
  EXEC SQL END DECLARE SECTION;

  static char  qstr[5000];
  static char  sqlfilename[128];
  static char  logfilename[128];
  static char  infile[128];
  static char  keycols[500];
  short use_commit, benchm;
  time_t t1, t2;
  int batchsize, maxerrors, argind;
  unsigned long int loaded_rows;
  int ch;
  FILE * fsqlfile, *logfile;
  int i;
  short uploadmode, uploadmode2;
  short verbosity,gzcompr;
  short no_repliation;

  verbosity=3; /* show errors and progress messages */
  logfilename[0]=0;
  sqlfilename[0]=0;
  batchsize=10000;
  server[0]=0;
  user[0]=0;
  password[0]=0;
  maxerrors=0;
  logfile=NULL;
  benchm=0;
  gzcompr=0;
  uploadmode=3;
  uploadmode2=0;
  no_repliation=0;

  setvbuf(stdout,NULL,_IONBF,0);
  if ( argc == 1 ) {
    usage( argv[0] );
    exit( -1 );
  }
    

#ifdef HAVE_ZLIB
  while ((ch = getopt( argc, argv, "Vd:s:u:p:e:t:c:n:k:m:orv:l:Bg" )) != EOF) {
#else
  while ((ch = getopt( argc, argv, "Vd:s:u:p:e:t:c:n:k:m:orv:l:B" )) != EOF) {
#endif
    switch (ch) {
      case 'V':
        fprintf( stdout,
         "\nloadinc ver 1.70, Copyright 2001 - 2007 Robert Sosnowski\n\n" );
        exit( 0 );
        break;
      case 'd':
        /* database name */
        strcpy( dbname, optarg );
        break;
      case 's':
        /* informix server */
        strcpy( server, optarg );
        break;
      case 'u':
        /* user */
        strcpy( user, optarg );
        break;
      case 'p':
        /* password */
        strcpy( password, optarg );
        break;
      case 'e': 
        /* SQL statement */
        strcpy( qstr, optarg );
        break;
      case 't': 
        /* table name */
        strcpy( tabname, optarg );
        break;
      case 'c': 
        /* file with SQL statement */
        strcpy( sqlfilename, optarg );
        break;
      case 'n':   
        /* batchsize */
        batchsize=(int)atoi( optarg );
        break;
      case 'k':
        /* key definition */
        strcpy(keycols, optarg);
        break;
      case 'm':
        /* upload mode */
        uploadmode=(int)atoi( optarg );
        break;
      case 'o':
        /* do select first */
        uploadmode2=1;
        break;
      case 'l':
        /* log file */
        strcpy( logfilename, optarg );
        break;
      case 'v':
        /* verbosity */
        verbosity=(int)atoi( optarg );
        break;
      case 'B':
        /* Benchmark mode */
	benchm=1;
        break;
      case 'g':
        /* gzipped */
        gzcompr=1;
        break;
      case 'r':
        /* Enterprise Replication disabled */
        no_repliation=1;
        break;
      default:
        usage( argv[0] );
        exit(2);
    }
  }

  if (strlen( dbname ) == 0) {
    fprintf( stdout, "Must supply database (-d)!\n" );
    usage( argv[0] );
    exit(2);
  }
  FetBufSize=30000;
    
  EXEC SQL WHENEVER ERROR CONTINUE;

  if (strlen(server)!=0 || strlen(user)!=0 || strlen(password)!=0) {
    if (strlen(server)==0) {
      fprintf( stderr, "Informix server should be specified with -s option\n");
      return 1;
    }
    strcpy(server2,"@");
    strcat(server2, server);
    if ( strlen(user)==0 && strlen(password)==0) {
      EXEC SQL CONNECT TO :server2 AS "conn1" WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting server %s:"INT4_FORMAT"\n", server,
	         sqlca.sqlcode);
	return 1;
      }
    } else {
      if ( strlen(user)==0 || strlen(password)==0) {
    	fprintf( stderr, "Both options: -u and -p should be provided\n");
        return 1;
      }
      EXEC SQL CONNECT TO :server2 AS "conn1" USER :user USING :password
               WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting server %s as user %s:"INT4_FORMAT"\n",
	         server, user, sqlca.sqlcode);
        return 1;
      }
    }
  } else {
    EXEC SQL CONNECT TO DEFAULT WITH CONCURRENT TRANSACTION;
    if (sqlca.sqlcode<0) {
      fprintf( stderr, "Error while connecting default server: "INT4_FORMAT"\n",
               sqlca.sqlcode);
      return 1;
    }
  }
  EXEC SQL DATABASE :dbname;
  if (sqlca.sqlcode < 0) {
    fprintf( stderr, "Cannot open database.  Code: "INT4_FORMAT", ISAM: "INT4_FORMAT".\n",
             sqlca.sqlcode, sqlca.sqlerrd[1] );
    return 1;
  }
  use_commit = (sqlca.sqlwarn.sqlwarn1 == 'W');

  if (use_commit && no_repliation) {
    use_commit=2;
  }
  if (strlen( qstr ) != 0 ) {
    if (strlen(tabname)!=0) {
      fprintf(stderr, "You can't specify both -e and -t options!\n");
      return 1;
    }
    if (strlen(sqlfilename)!=0) {
      fprintf(stderr, "You can't specify both -e and -c options!\n");
      return 1;
    }
  } else {
    if (strlen(tabname) != 0) {
      if (strlen(sqlfilename)!=0) {
        fprintf(stderr, "You can't specify both -t and -c options!\n");
        return 1;
      }
      sprintf(qstr, "insert into %s", tabname);
    } else {
      if (strlen(sqlfilename)!=0) {
        fsqlfile=fopen(sqlfilename, "r");
        if (fsqlfile) {
          i=fread(qstr, 1, sizeof(qstr), fsqlfile);
          qstr[i]=0;
          fclose(fsqlfile);
        } else {
          perror("Can't open sql file");
          return 1;
        }
      } else {
        fprintf(stderr, "You have to specify one of: -e -t or -c option!\n");
        return 1;
      }
    }
  }
  if (uploadmode<0 || uploadmode>5) {
    fprintf(stderr, "Wrong value for upload mode\n");
    return 1;
  }

  if (verbosity<0 || verbosity>4) {
    fprintf(stderr, "Wrong verbosity value\n");
    return 1;
  }
  if (gzcompr<0 || gzcompr>9) {
    fprintf(stderr, "Wrong gz compression value\n");
    return 1;
  }
  if (logfilename[0]) {
    logfile=fopen(logfilename,"a");
#ifdef _IONBF
    if (logfile!=NULL) {
      setvbuf(logfile, NULL, _IONBF,0);
    }
#endif
  }

  strcpy(infile,"-");
  argind=optind;
  do {
    if (argind<argc) {
      strcpy(infile, argv[argind]);
    }
    if (strcmp(infile,"-")==0) {
      infile[0]=0;
    }
    if (benchm) {
      t1=time(NULL);
      if (logfilename[0]) {
        fprintf(logfile, "Loading started at %s",ctime(&t1));
      }
      fprintf(stdout, "Loading started at %s",ctime(&t1));
    }
    if (loadinc (qstr, infile, '|', batchsize, use_commit,
             maxerrors,  &loaded_rows, logfile, keycols, uploadmode,
             uploadmode2, verbosity, gzcompr)) {
      return 1;
    }
    if (benchm) {
      t2=time(NULL);
      i=t2-t1;
      if (logfilename[0]) {
        fprintf(logfile, "Loading finished at %s",ctime(&t2));
        fprintf(logfile, "Elapsed seconds: %d\n",i);
      }
      fprintf(stdout, "Loading finished at %s",ctime(&t2));
      fprintf(stdout, "Elapsed seconds: %d\n",i);
    }
  } while (++argind <argc);
    
  if (logfilename[0]) {
    fclose(logfile);
  }

  return 0;
}


int usage( char *prog )
{


  fprintf( stdout, "\n");
  fprintf( stdout, "%s - apply insert/update/delete from *.unl file to Informix database.\n\n", prog );

  fprintf( stdout, "Usage: \n" );
#ifdef HAVE_ZLIB
  fprintf( stdout, 
           "\t%s [-V] -d <db_name> [-s <server> -u <user> -p <password>]\n "
           "(-e <insert stmt>|-t <table>|-c <sql file>) [-n <batch size>]\n "
           "-k <key columns> [-m <mode>] [-o] [-l <logfile>] [-B] [-g]\n "
           "[-v <verbosity>] [-r] <unload files>\n\n", prog);
#else
  fprintf( stdout, 
           "\t%s [-V] -d <db_name> [-s <server> -u <user> -p <password>]\n "
           "(-e <insert stmt>|-t <table>|-c <sql file>) [-n <batch size>]\n "
           "-k <key columns> [-m <mode>] [-o] [-l <logfile>] [-B]\n "
           "[-v <verbosity>] [-r] <unload files>\n\n", prog);
#endif
  fprintf( stdout, "\t\t-V - version information\n" ); 
  fprintf( stdout, "\t\t-d - database name\n" );
  fprintf( stdout, "\t\t-s - informix server name\n" );
  fprintf( stdout, "\t\t-u - user\n" );
  fprintf( stdout, "\t\t-p - password\n" );
  fprintf( stdout, "\t\t-e - SQL insert statement in quotes\n" );
  fprintf( stdout, "\t\t-t - table name\n" );
  fprintf( stdout, "\t\t-c - file containing SQL insert statement\n" );
  fprintf( stdout, "\t\t-n - batch size\n" );
  fprintf( stdout, "\t\t-k - list of key column names or column numbers\n" );
  fprintf( stdout, "\t\t-m - upload mode:\n" );
  fprintf( stdout, "\t\t     0 - plain load (insert only)\n" );
  fprintf( stdout, "\t\t     1 - try insert first, then update\n" );
  fprintf( stdout, "\t\t     2 - try update first, then insert\n" );
  fprintf( stdout, "\t\t     3 - insert/update order based on statistics (default)\n");
  fprintf( stdout, "\t\t           (more often first)\n" );
  fprintf( stdout, "\t\t     4 - use extra column to determine insert/delete/update\n" );
  fprintf( stdout, "\t\t     5 - only updates (inserts are skipped\n" );
  fprintf( stdout, "\t\t-o - use select statement to determine if insert\n");
  fprintf( stdout, "\t\t       or update should be used\n" );
  fprintf( stdout, "\t\t-l - log file name\n" );
  fprintf( stdout, "\t\t-B - benchmark\n" );
#ifdef HAVE_ZLIB
  fprintf( stdout, "\t\t-g - gzipped unload\n" );
#endif
  fprintf( stdout, "\t\t-v - verbosity:\n" );
  fprintf( stdout, "\t\t     0 - completely quiet\n" );
  fprintf( stdout, "\t\t     1 - show errors only\n" );
  fprintf( stdout, "\t\t     2 - summary messages\n" );
  fprintf( stdout, "\t\t     3 - detailed messages (default)\n" );
  fprintf( stdout, "\t\t     4 - debug info\n" );
  fprintf( stdout, "\t\t-r - loading without (Enterprise) replication\n" );

  return 0;
} 

int loadinc (const char * insstmt2, const char *fname, const char delim,
             const int batchsize, const int usecommit,
             const int maxerrors,  unsigned long int *loaded_rows,
             const void *logfile, const char *collist, const short uploadmode,
             const short uploadmode2, const short verbosity,
             const short gzcompr)

{
  loader_handle_t *loader_handle;
  int sqlind;
  int i, k;
  char buffer[32767+1];
  int all_fields;
  char sqlname2[TBLNAME_SIZE+1]; /* fieldname read from file */
  unsigned long int bytesread;
  int emptyread;
  int badrecord;
  int all_records;
  int batchsize2;
    
  buffer[0]=0;

  loader_handle=loader_create ();
  if (loader_handle==NULL) {
/*    sprintf(errstr,"Could not initialize loader - no memory\n");
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile);
    cleanup(fname, datahandle, 0, 0, udesc, indicators, nullables, used_tags,
            buffer, logfile, NULL, onlogerror); */
    return 1;
  }

  k=loader_setstmt(loader_handle, insstmt2);
  if (k!=0) {
    return 1;
  }

  loader_setdelimiter(loader_handle, delim);

  k=loader_setdatahandlename(loader_handle, fname);
  if (k!=0) {
    return 1;
  }

  if (batchsize<0) {
    batchsize2=0;
  } else {
    batchsize2=batchsize;
  }
  loader_setbatchsize(loader_handle, batchsize2);

  loader_setusecursor(loader_handle, 0);

  loader_setusecommit(loader_handle, usecommit);

  loader_setmaxerrors(loader_handle, maxerrors);

  if (logfile==NULL) {
    loader_setloghandle(loader_handle, NULL);
  } else {
    loader_setloghandle(loader_handle, logfile);
  }
  switch (verbosity) { 
    case 0: 
            loader_setverbosity(loader_handle, 0);
            break;
    case 1: 
            loader_setverbosity(loader_handle, 20);
            break;
    case 2: 
            loader_setverbosity(loader_handle, 30);
            break;
    case 3: 
            loader_setverbosity(loader_handle, 40);
            break;
    case 4: 
            loader_setverbosity(loader_handle, 50);
            break;
    default: 
            loader_setverbosity(loader_handle, 30);
            break;
  }

#ifdef HAVE_ZLIB
  if (gzcompr) {
    loader_setoninit(loader_handle, &oninitgz);
    loader_setonreadfield(loader_handle, &onreadfieldgz);
    loader_setoncleanup(loader_handle, &oncleanupgz);
    loader_setoncrlf(loader_handle, &oncrlfgz);
  } else {
#else
    loader_setoncleanup(loader_handle, &oncleanup_std);
#endif
#ifdef HAVE_ZLIB
  }
#endif
  loader_setkeycols(loader_handle, collist);

  loader_setuploadmode(loader_handle, uploadmode);

  loader_setuploadmode2(loader_handle, uploadmode2);

  *loaded_rows=0;
  k=loader_prepare(loader_handle);
  if (k!=0) {
    return 1;
  }

  k=loader_open(loader_handle);
  if (k!=0) {
    return 1;
  }
  i=0; /* current field number */
  if (loader_handle->uploadmode==4) {
    i=-1;
  }
  while (1) {
    if (i==0) {
      if (loader_beforerow(loader_handle)) {
        return 1;
      }
    }
    while (1) {
      k=loader_readfield(loader_handle, &i,
                       buffer, sizeof(buffer)-1,
                       &bytesread, &all_records,
                       sqlname2, TBLNAME_SIZE+1, &emptyread,
                       &all_fields, &badrecord, &sqlind);
      if (k!=0) {
        return 1;
      }
      if (all_records && i==0) {
        break;
      }
      if (emptyread && !all_fields && !all_records) {
        continue;
      }
      break;
    }
    /* assertion here: !emptyread || all_fields || all_records */

    if (badrecord) {
      i=0;
      continue;
    }
    if (!emptyread) {
      k=loader_putfield(loader_handle, i, buffer, bytesread, sqlind,
                        &badrecord);
      if (k!=0) {
        return 1;
      }
      i++;
    }
    if (all_fields) {
      k=loader_putrecord(loader_handle);
      if (k!=0) {
        return 1;
      }
      i=0;
      if (loader_handle->uploadmode==4) {
        i=-1;
      }
    }
    if (badrecord) {
      i=0;
      continue;
    }
    if (all_records) {
      break;
    }
  }
  if (loader_close(loader_handle)) {
    return 1;
  }
  if (loader_free(loader_handle)) { /* = loader_unprepare & destroy */
    return 1;
  }
  loader_handle=NULL;
  return 0;
}

