/******************************************************************************
*
* Utility for loading Informix database directly from XML data file.
*
* Author: Robert Sosnowski, Robert.Sosnowski@bzwbk.pl
*
* Version 1.70, 17-10-2005
*
* Copyright (c) 2001 - 2007. All Rights Reserved.
*
* Permission granted to use & distribute for free.
*
* This software can be used only at your own risk. Author is not responsible 
* for any damage caused by using this software.
* 
******************************************************************************/

#include "load2cfg.h"

#ifndef HAVE_GETOPT
extern int      optind;
extern char    *optarg;
extern int	opterr;
int getopt(int argc, char *argv[], char *optionS);
#endif

EXEC SQL include locator;
EXEC SQL include sqltypes;

static const char cb64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>


#ifdef HAVE_EXPAT_H
#include <expat.h>
#endif

#include "load2.h"
#include "tempfile.h"
#include <time.h>

#ifdef HAVE_CTYPE_H
#include <ctype.h>
/* for isdigit on MSVC */
#endif

#ifdef HAVE_ZLIB
#include <zlib.h>
#endif

extern char * optarg;
extern int optind;

#define BUFFSIZE	8192
#define FLDBUFFSIZE     (32767+1)

char Buff[BUFFSIZE];
char fldbuff[FLDBUFFSIZE];

static int within_rowset, within_row, within_text;
static char rowset_token[20], row_token[20];
static loader_handle_t *loader_handle;
static unsigned long int bytesread;
static char fieldname[19];
static int fieldnum;
static int fieldtype;
static int4 sqlxid;
static FILE *ft, *logfile;
static unsigned char block1[4];
static short blocksize;
static unsigned char cd[256];

static char *id="@(#) $Id: loadx.ec 2521 2007-11-21 08:55:56Z sosnor01 $";

int usage( char * );
int is_critical_dups(long int sqlcode, long int isamcode, int errtype,
                     int errcnt, int maxerrors, int *errinc);
int is_critical_std(long int sqlcode, long int isamcode, int errtype,
                    int errcnt, int maxerrors, int *errinc);

int loadx(char *infile,
          unsigned long int *loaded_rows,
          const short gzcompr);

void oncleanup_std(const char *fname, void *handle, const void *logfile,
                   onlogerror_lpt onlogerror, int verbose,
                   const char *datahandleoptions);
#ifdef HAVE_ZLIB
void oncleanupgzx(const char *fname, void *handle, const void *logfile,
                  onlogerror_lpt onlogerror, int verbose,
                  const char *datahandleoptions);
#endif



static void set_rowset_token(char *s)
{
  strcpy(rowset_token, s);
}

static void set_row_token(char *s)
{
  strcpy(row_token, s);
}

void decodeblock( unsigned char in[4], unsigned char out[3] )
{
  unsigned char ch1, ch2;

  ch1=in[2];
  ch2=in[1];
  out[ 2 ] = (unsigned char ) (((ch1 << 6) & 0xc0) | in[3]);
  out[ 1 ] = (unsigned char ) (ch2 << 4 | ch1 >> 2);
  out[ 0 ] = (unsigned char ) (in[0] << 2 | ch2 >> 4);
}

void start(void *data, const char *el, const char **attr) {
char *p;
int i;
void *blobbufp;
loc_t simple_lo;
char errstr[100];

  bytesread=0;
  if (within_rowset==0 && strcmp(el,rowset_token)==0) {
    within_rowset=1;
  } else {
    if (within_rowset && within_row==0 && strcmp(el,row_token)==0) {
       within_row=1;
    } else {
       /* handler for the column name */
       if (within_row && within_rowset) {
         sprintf (fieldname, "%s", el);
         fieldnum=loader_getfieldnum(loader_handle, el);
         if (fieldnum==-2) {
           exit(1);
         }
         if (fieldnum!=-3) {
           fieldtype=loader_getfieldtype(loader_handle, fieldnum);
         } else {
           fieldtype=-1;
         }
         if (fieldtype==CFIXBINTYPE) {
           sqlxid=loader_getfieldtypeext(loader_handle, fieldnum);
           if (sqlxid==-1) {
             fieldtype=-1;
           }
         }
         within_text=1;
         if (fieldtype==SQLTEXT || fieldtype==SQLBYTES ) {
           blocksize=0;
           p=creatmpfile();
           if (p==NULL) {
             i=errno;
             sprintf(errstr,"can't create temporary file at line %ld, errno=%d (%s)",
                     loader_handle->rownum, i, p);
             fprintf(stderr,errstr);
             if (logfile!=NULL) {
               fprintf(stderr,errstr);
             }
             exit(1);
           }
           ft=fopen(p, "wb");
           if (ft==NULL) {
             i=errno;
             sprintf(errstr,"can't open temporary file at line %ld, errno=%d (%s)",
                     loader_handle->rownum, i, p);
             fprintf(stderr,errstr);
             if (logfile!=NULL) {
               fprintf(stderr,errstr);
             }
             exit(1);
           }
           blobbufp=NULL;
           i=strlen(p);
           blobbufp=(char *)load2_malloc(i+1, __FILE__, __LINE__);
           if (blobbufp==NULL) {
             sprintf(errstr,"Out of memory\n");
             fprintf(stderr,errstr);
             if (logfile!=NULL) {
               fprintf(stderr,errstr);
             }
             exit(1);
           }
           strcpy(blobbufp,p);
           simple_lo.loc_loctype=LOCFNAME;
           simple_lo.loc_fname=blobbufp;
           simple_lo.loc_oflags=LOC_RONLY;
           simple_lo.loc_size=-1;
           simple_lo.loc_indicator= 1;
           simple_lo.loc_type=fieldtype;
           bycopy((void *)&simple_lo, fldbuff, sizeof(loc_t));
         }
         if (fieldtype==CFIXBINTYPE) {
           blocksize=0;
           p=creatmpfile();
           if (p==NULL) {
             i=errno;
             sprintf(errstr,"can't create temporary file at line %ld, errno=%d (%s)",
                     loader_handle->rownum, i, p);
             fprintf(stderr,errstr);
             if (logfile!=NULL) {
               fprintf(stderr,errstr);
             }
             exit(1);
           }
           ft=fopen(p, "wb");
           if (ft==NULL) {
             i=errno;
             sprintf(errstr,"can't open temporary file at line %ld, errno=%d (%s)",
                     loader_handle->rownum, i, p);
             fprintf(stderr,errstr);
             if (logfile!=NULL) {
               fprintf(stderr,errstr);
             }
             exit(1);
           }
           blobbufp=NULL;
           i=strlen(p);
           blobbufp=(char *)load2_malloc(i+1, __FILE__, __LINE__);
           if (blobbufp==NULL) {
             sprintf(errstr,"Out of memory\n");
             fprintf(stderr,errstr);
             if (logfile!=NULL) {
               fprintf(stderr,errstr);
             }
             exit(1);
           }
           strcpy(blobbufp,p);
  
           bycopy((char *)&blobbufp, fldbuff, sizeof(char *));
         }
       }
    }
  }
}  /* End of start handler */

void end(void *data, const char *el) {
int k;
int badrecord;
char *filename;
off_t fsize;
mint ifx_error;
ifx_lo_create_spec_t *create_spec;
ifx_lo_t descr;
mint lofd;
ifx_int8_t t1;
int j;
char errstr[256];


  if (within_rowset && within_row==0 && strcmp(el,rowset_token)==0) {
    within_rowset=0;
    within_row=0;
    within_text=0;
  } else {
    if (within_rowset && within_row && strcmp(el,row_token)==0) {
      /* handler for the row element */
      within_row=0;
      within_text=0;
      k=loader_putrecord(loader_handle);
      if (k!=0) {
	exit(-1);
      }
    } else {
      if (within_row) {
        /* handler for field element */
        within_text=0;
        if (fieldnum!=-3) {
          if (((fieldtype==CFIXBINTYPE && sqlxid==XID_BLOB) || fieldtype==SQLBYTES)
              && (blocksize==2 || blocksize==3)) {
            if (blocksize==2) {
              block1[2]=0;
              block1[3]=0;
              decodeblock(block1, block1);
              j=fputc(block1[0], ft);
            } else {
              block1[3]=0;
              decodeblock(block1, block1);
              j=fputc(block1[0], ft);
              if (j!=EOF) {
                j=fputc(block1[1], ft);
              }
            }
            if (j==EOF) {
              j=errno;
              sprintf(errstr,"error writing temporary file at line %ld, errno=%d",
                      loader_handle->rownum, j);
              fprintf(stderr,errstr);
              if (logfile!=NULL) {
                fprintf(stderr,errstr);
              }
              exit(1);
            }
          }
          filename=NULL;
          fsize=0;
          if (fieldtype==CFIXBINTYPE) {
            bycopy(fldbuff, (char *)&filename, sizeof(char *));
            fsize=ftell8(ft);
          }

          if (fieldtype==SQLTEXT || fieldtype==SQLBYTES || fieldtype==CFIXBINTYPE) {
            fclose(ft);
          } else {
            fldbuff[bytesread]=0;
          }

          if (fieldtype==CFIXBINTYPE) {
            ifx_error = ifx_lo_def_create_spec(&create_spec);
            if (ifx_error < 0) {
              sprintf(errstr,"ERROR ifx_lo_def_create_spec:%d,"INT4_FORMAT",%s",
                      ifx_error, sqlca.sqlerrd[1], sqlca.sqlerrm);
              fprintf(stderr,errstr);
              if (logfile!=NULL) {
                fprintf(stderr,errstr);
              }
              exit(1);
            }

            ifx_error=off_t2int8(fsize, &t1);
            if (ifx_error < 0) {
              sprintf(errstr,"ERROR converting off_t into int8:%d", ifx_error);
              fprintf(stderr,errstr);
              if (logfile!=NULL) {
                fprintf(stderr,errstr);
              }
              exit(1);
            }
            ifx_error=ifx_lo_specset_estbytes(create_spec, &t1);
            if (ifx_error < 0) {
              sprintf(errstr,"ERROR ifx_lo_specset_estbytes:%d,"INT4_FORMAT",%s",
                             ifx_error, sqlca.sqlerrd[1], sqlca.sqlerrm);
              fprintf(stderr,errstr);
              if (logfile!=NULL) {
                fprintf(stderr,errstr);
              }
              exit(1);
            }
  
          /* Create an LO-specification structure for the smart large object */
            if ((lofd = ifx_lo_create(create_spec, LO_WRONLY, &descr, &ifx_error)) == -1) {
              sprintf(errstr,"ERROR ifx_lo_create:%d,"INT4_FORMAT",%s", ifx_error, sqlca.sqlerrd[1], sqlca.sqlerrm);
              fprintf(stderr,errstr);
              if (logfile!=NULL) {
                fprintf(stderr,errstr);
              }
              exit(1);
            }

          /* Copy data into the open smart
           * large object that lofd points to. */
            ifx_error=ifx_lo_copy_to_lo(lofd, filename, LO_CLIENT_FILE);
            if (ifx_error < 0) {
              sprintf(errstr,"ERROR ifx_lo_copy_to_lo:%d,"INT4_FORMAT",%s",
                             ifx_error, sqlca.sqlerrd[1], sqlca.sqlerrm);
              fprintf(stderr,errstr);
              if (logfile!=NULL) {
                fprintf(stderr,errstr);
              }
              exit(1);
            }
            load2_free(filename,__FILE__,__LINE__);
          /* Close the LO file descriptor */
            ifx_lo_close(lofd);
          /* Free LO-specification structure */
            ifx_lo_spec_free(create_spec);
            bycopy((char *)&descr, fldbuff, sizeof(descr));
          }
          k=loader_putfield(loader_handle, fieldnum, fldbuff, bytesread, 0,
                            &badrecord);
          if (k!=0) {
	    exit(-1);
          }
        }
      }
    }
  }
}  /* End of end handler */

void characterData(void *data, const char *el, int len)
{ char ch;
  int i;
  unsigned int j;
  char errstr[100];
  unsigned char ch2;

  if (within_text) {
    if (fieldtype==SQLTEXT || fieldtype==SQLBYTES || fieldtype==CFIXBINTYPE) {
      if (fieldtype==SQLTEXT || (fieldtype==CFIXBINTYPE && sqlxid==XID_CLOB)) {
        for(i=0;i<len;i++) {
          j=fputc(el[i], ft);
          if (j==EOF) {
             j=errno;
             sprintf(errstr,"error writing temporary file at line %ld, errno=%d",
                     loader_handle->rownum, j);
             fprintf(stderr,errstr);
             if (logfile!=NULL) {
               fprintf(stderr,errstr);
             }
             exit(1);
          }
          bytesread++;
        }
      } else {
        for(i=0;i<len;i++) {
          ch2=cd[(int)el[i]];
          if (ch2==255) {
            continue;
          }
          if (ch2==254 && blocksize<2) {
            continue;
          }
          if (blocksize==3 && block1[2]==254 && ch2!=254) {
            blocksize--;
          }
          block1[blocksize++]=ch2;
          if (blocksize==4) {
            if (block1[3]==254) {
              if (block1[2]==254) {
                block1[2]=0;
                block1[3]=0;
                decodeblock(block1, block1);
                j=fputc(block1[0], ft);
              } else {
                block1[3]=0;
                decodeblock(block1, block1);
                j=fputc(block1[0], ft);
                if (j!=EOF) {
                  j=fputc(block1[1], ft);
                }
              }
            } else {
              decodeblock(block1,block1);
              j=fputc(block1[0], ft);
              if (j!=EOF) {
                j=fputc(block1[1], ft);
              }
              if (j!=EOF) {
                j=fputc(block1[2], ft);
              }
            }
            if (j==EOF) {
               j=errno;
               sprintf(errstr,"error writing temporary file at line %ld, errno=%d",
                     loader_handle->rownum, j);
             fprintf(stderr,errstr);
             if (logfile!=NULL) {
               fprintf(stderr,errstr);
             }
             exit(1);
            }
            bytesread++;
            blocksize=0;
          }
        }
      }
    } else {
      for(i=0;i<len;i++) {
        if (bytesread<FLDBUFFSIZE) {
          ch=el[i];
          fldbuff[bytesread]=ch;
          bytesread++;
        }
      }
    }
  }
}


int main( int argc, char **argv )
{

  EXEC SQL BEGIN DECLARE SECTION;
  static char dbname[TBLNAME_SIZE];
  static char server[TBLNAME_SIZE];
  static char server2[TBLNAME_SIZE+1];
  static char user[USERNAME_SIZE];
  static char password[USERNAME_SIZE];
  static char  tabname[TBLNAME_SIZE];
  EXEC SQL END DECLARE SECTION;

  static char  qstr[5000];
  static char  sqlfilename[128];
  static char  logfilename[128];
  static char  infile[128];

  short use_ins_curs, use_commit, benchm;
  time_t t1, t2;
  int batchsize, maxerrors, argind;
  int ch;
  FILE * fsqlfile;
  int i, k;
  char *p;
  int (*is_critical)(long int sqlcode, long int isamcode, int errtype,
                     int errcnt, int maxerrors, int *errinc);
  unsigned long int loaded_rows;
  short verbosity, gzcompr;

  is_critical=&is_critical_std;
  verbosity=3; /* show errors and progress messages */
  use_ins_curs=0;
  logfilename[0]=0;
  sqlfilename[0]=0;
  batchsize=10000;
  server[0]=0;
  user[0]=0;
  password[0]=0;
  maxerrors=0;
  logfile=NULL;
  benchm=0;
  gzcompr=0;

  for (i=0;i<256;i++) {
    cd[i]=255;
  }
  for (i=0;i<64;i++) {
    cd[(int)cb64[i]]=i;
  }
  cd[(unsigned char)'=']= 254;
  
/*  setvbuf(stdout,NULL,_IONBF,0); */
  if ( argc == 1 ) {
    usage( argv[0] );
    exit( -1 );
  }
    
#ifdef HAVE_ZLIB
  while ((ch = getopt( argc, argv, "Vd:s:u:p:e:t:c:n:kzi:v:l:Bg" )) != EOF) {
#else
  while ((ch = getopt( argc, argv, "Vd:s:u:p:e:t:c:n:kzi:v:l:B" )) != EOF) {
#endif
    switch (ch) {
      case 'V':
        fprintf( stdout,
         "\nloadx ver 1.70, Copyright 2001 - 2007 Robert Sosnowski\n\n" );
        exit( 0 );
        break;
      case 'd':
        /* database name */
        strcpy( dbname, optarg );
        break;
      case 's':
        /* informix server */
        strcpy( server, optarg );
        break;
      case 'u':
        /* user */
        strcpy( user, optarg );
        break;
      case 'p':
        /* password */
        strcpy( password, optarg );
        break;
      case 'e': 
        /* SQL statement */
        strcpy( qstr, optarg );
        break;
      case 't': 
        /* table name */
        strcpy( tabname, optarg );
        break;
      case 'c': 
        /* file with SQL statement */
        strcpy( sqlfilename, optarg );
        break;
      case 'n':   
        /* batchsize */
        batchsize=(int)atoi( optarg );
        break;
      case 'k':
        /* ignore unique key violation */
        is_critical=&is_critical_dups;
        break;
      case 'z':
        /* use insert cursor */
        use_ins_curs = 1;
        break;
      case 'i':
        /* ignore # bad rows */
        maxerrors = (int)atoi( optarg );
        break;
      case 'l':
        /* log file */
        strcpy( logfilename, optarg );
        break;
      case 'v':
        /* verbosity */
        verbosity=(int)atoi( optarg );
        break;
      case 'B':
        /* Benchmark mode */
	benchm=1;
        break;
      case 'g':
        /* gzipped */
        gzcompr=1;
        break;
      default:
        usage( argv[0] );
        exit(2);
    }
  }

  if (strlen( dbname ) == 0) {
    fprintf( stdout, "Must supply database (-d)!\n" );
    usage( argv[0] );
    exit(2);
  }
  FetBufSize=30000;
    
  EXEC SQL WHENEVER ERROR CONTINUE;

  if (strlen(server)==0 && (p = strstr(dbname, "@")))
  {
    p[0] = '\0';
    strcpy(server, p+1);
  }

  if (strlen(server)!=0 || strlen(user)!=0 || strlen(password)!=0) {
    if (strlen(server)==0) {
      fprintf( stderr, "Informix server should be specified with -s option\n");
      return 1;
    }
    strcpy(server2,"@");
    strcat(server2, server);
    if ( strlen(user)==0 && strlen(password)==0) {
      EXEC SQL CONNECT TO :server2 AS "conn1" WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting server %s:"INT4_FORMAT"\n", server,
	         sqlca.sqlcode);
	return 1;
      }
    } else {
      if ( strlen(user)==0 || strlen(password)==0) {
    	fprintf( stderr, "Both options: -u and -p should be provided\n");
        return 1;
      }
      EXEC SQL CONNECT TO :server2 AS "conn1" USER :user USING :password
               WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting server %s as user %s:"INT4_FORMAT"\n",
	         server, user, sqlca.sqlcode);
        return 1;
      }
    }
  } else {
    EXEC SQL CONNECT TO DEFAULT WITH CONCURRENT TRANSACTION;
    if (sqlca.sqlcode<0) {
      fprintf( stderr, "Error while connecting default server: "INT4_FORMAT"\n",
               sqlca.sqlcode);
      return 1;
    }
  }
  EXEC SQL DATABASE :dbname;
  if (sqlca.sqlcode < 0) {
    fprintf( stderr, "Cannot open database.  Code: "INT4_FORMAT", ISAM: "INT4_FORMAT".\n",
             sqlca.sqlcode, sqlca.sqlerrd[1] );
    return 1;
  }
  use_commit = (sqlca.sqlwarn.sqlwarn1 == 'W');

  if (strlen( qstr ) != 0 ) {
    if (strlen(tabname)!=0) {
      fprintf(stderr, "You can't specify both -e and -t options!\n");
      return 1;
    }
    if (strlen(sqlfilename)!=0) {
      fprintf(stderr, "You can't specify both -e and -c options!\n");
      return 1;
    }
  } else {
    if (strlen(tabname) != 0) {
      if (strlen(sqlfilename)!=0) {
        fprintf(stderr, "You can't specify both -t and -c options!\n");
        return 1;
      }
      sprintf(qstr, "insert into %s", tabname);
    } else {
      if (strlen(sqlfilename)!=0) {
        fsqlfile=fopen(sqlfilename, "r");
        if (fsqlfile) {
          i=fread(qstr, 1, sizeof(qstr), fsqlfile);
          qstr[i]=0;
          fclose(fsqlfile);
        } else {
          perror("Can't open sql file");
          return 1;
        }
      } else {
        fprintf(stderr, "You have to specify one of: -e -t or -c option!\n");
        return 1;
      }
    }
  }

  strcpy(infile,"-");
  argind=optind;

  loader_handle=loader_create ();
  if (loader_handle==NULL) {
    fprintf(stderr,"Could not initialize loader - no memory\n");
    exit(-1);
  }

  if (logfilename[0]) {
    logfile=fopen(logfilename,"a");
#ifdef _IONBF
    if (logfile!=NULL) {
      setvbuf(logfile, NULL, _IONBF,0);
    }
#endif
  }

  k=loader_setstmt(loader_handle, qstr);
  if (k!=0) {
    return 1;
  }

  if (batchsize<0) {
    batchsize=0;
  }
  loader_setbatchsize(loader_handle, batchsize);

  loader_setusecursor(loader_handle, use_ins_curs);

  loader_setusecommit(loader_handle, use_commit);

  loader_setmaxerrors(loader_handle, maxerrors);

  if (logfile==NULL) {
    loader_setloghandle(loader_handle, NULL);
  } else {
    loader_setloghandle(loader_handle, logfile);
  }
  switch (verbosity) { 
    case 0: 
            loader_setverbosity(loader_handle, 0);
            break;
    case 1: 
            loader_setverbosity(loader_handle, 20);
            break;
    case 2: 
            loader_setverbosity(loader_handle, 30);
            break;
    case 3: 
            loader_setverbosity(loader_handle, 40);
            break;
    case 4: 
            loader_setverbosity(loader_handle, 50);
            break;
    default: 
            fprintf(stderr, "Wrong verbosity value\n");
            return 1;
  }

#ifdef HAVE_ZLIB
  if (gzcompr) {
    loader_setoncleanup(loader_handle, oncleanupgzx);
  } else {
#endif
    loader_setoncleanup(loader_handle, oncleanup_std);
#ifdef HAVE_ZLIB
  }
#endif

  loader_setunload_type(loader_handle,3);

  loader_setextrafields(loader_handle, 0);

  loader_setiscritical(loader_handle, is_critical);

  k=loader_prepare(loader_handle);

  if (k!=0) {
    return 1;
  }

  do {
    if (argind<argc) {
      strcpy(infile, argv[argind]);
    }
    if (strcmp(infile,"-")==0) {
      infile[0]=0;
    }
    if (benchm) {
      t1=time(NULL);
      if (logfilename[0]) {
        fprintf(logfile, "Loading started at %s",ctime(&t1));
      }
      fprintf(stdout, "Loading started at %s",ctime(&t1));
    }
    printf("%s\n",infile);
    if (loadx(infile, 
             &loaded_rows, gzcompr)!=0) {
      return 1;
    }
    if (benchm) {
      t2=time(NULL);
      i=t2-t1;
      if (logfilename[0]) {
        fprintf(logfile, "Loading finished at %s",ctime(&t2));
        fprintf(logfile, "Elapsed seconds: %d\n",i);
      }
      fprintf(stdout, "Loading finished at %s",ctime(&t2));
      fprintf(stdout, "Elapsed seconds: %d\n",i);
    }
  } while (++argind <argc);

  if (loader_free(loader_handle)) { /* = loader_unprepare & destroy */
    return 1;
  }
  loader_handle=NULL;
    
  if (logfilename[0]) {
    fclose(logfile);
  }

  return 0;
}


int usage( char *prog )
{


  fprintf( stdout, "\n");
  fprintf( stdout, "%s - load *.xml into Informix tables.\n\n", prog );

  fprintf( stdout, "Usage: \n" );
#ifdef HAVE_ZLIB
  fprintf( stdout, 
           "\t%s [-V] -d <db_name> [-s <server> -u <user> -p <password>]\n "
           "(-e <insert stmt>|-t <table>|-c <sql file>) [-n <batch size>]\n "
           "[-k] [-z] [-i <# bad rows>] [-l <logfile>] [-B] [-g] [-v <verbosity>]\n "
           "<xml unload files>\n\n", prog);
#else
  fprintf( stdout, 
           "\t%s [-V] -d <db_name> [-s <server> -u <user> -p <password>]\n "
           "(-e <insert stmt>|-t <table>|-c <sql file>) [-n <batch size>]\n "
           "[-k] [-z] [-i <# bad rows>] [-l <logfile>] [-B] [-v <verbosity>]\n "
           "<xml unload files>\n\n", prog);
#endif

  fprintf( stdout, "\t\t-V - version information\n" ); 
  fprintf( stdout, "\t\t-d - database name\n" );
  fprintf( stdout, "\t\t-s - informix server name\n" );
  fprintf( stdout, "\t\t-u - user\n" );
  fprintf( stdout, "\t\t-p - password\n" );
  fprintf( stdout, "\t\t-e - SQL insert statement in quotes\n" );
  fprintf( stdout, "\t\t-t - table name\n" );
  fprintf( stdout, "\t\t-c - file containing SQL insert statement\n" );
  fprintf( stdout, "\t\t-n - batch size\n" );
  fprintf( stdout, "\t\t-k - ignore unique key violation\n" );
  fprintf( stdout, "\t\t-z - use insert cursor\n" );
  fprintf( stdout, "\t\t-i - ignore # bad rows\n" );
  fprintf( stdout, "\t\t-l - log file name\n" );
  fprintf( stdout, "\t\t-B - benchmark\n" );
#ifdef HAVE_ZLIB
  fprintf( stdout, "\t\t-g - gzipped data\n" );
#endif
  fprintf( stdout, "\t\t-v - verbosity:\n" );
  fprintf( stdout, "\t\t     0 - completely quiet\n" );
  fprintf( stdout, "\t\t     1 - show errors only\n" );
  fprintf( stdout, "\t\t     2 - summary messages\n" );
  fprintf( stdout, "\t\t     3 - detailed messages (default)\n" );
  fprintf( stdout, "\t\t     4 - debug info\n" );

  return 0;
} 

int is_critical_dups(long int sqlcode, long int isamcode, int errtype,
                     int errcnt, int maxerrors, int *errinc)
{
/* errtype:
   0 - SQL error
   1 - inserting null into non-null column
   2 - conversion from ASCII format failed
   3 - error while reading (onreadfield, oncrlf)
   4 - wrong number of columns
*/
  (*errinc)=1;
  if (errtype==0 && (sqlcode==-239 || sqlcode == -268)) {
    (*errinc)=0;
    return 0;
  }
  if (errcnt>=maxerrors) { /* test before errcnt++ */
    return 1;
  }
  return 0;
}



int loadx(char *infile,
          unsigned long int *loaded_rows,
          const short gzexpr)
{ FILE *f;
#ifdef HAVE_ZLIB
  gzFile f2;
#endif
  int k;
  unsigned long int loaded_rows2;
  XML_Parser p;
  int done;
  int len;


  f=NULL;
#ifdef HAVE_ZLIB
  f2=NULL;
#endif
  *loaded_rows=0;
  loaded_rows2=0;

  k=loader_setdatahandlename(loader_handle, infile);
  if (k!=0) {
    return 1;
  }

  p = XML_ParserCreate("ISO-8859-1");
  if (! p) {
    fprintf(stderr, "Couldn't allocate memory for XML parser\n");
    exit(-1);
  }

  set_rowset_token("ROWSET");
  set_row_token("ROW");

  XML_SetElementHandler(p, start, end);
  XML_SetCharacterDataHandler(p, characterData);

  if (strcmp(infile,"-")==0) {
    infile[0]=0;
  }
  if (gzexpr) {
#ifdef HAVE_ZLIB
    if (infile[0] !=0) {
      f2=gzopen(infile,"rb");
    } else {
      f2=gzdopen(0,"rb");
    }
    if (f2==NULL) {
      fprintf(stderr, "File %s cannot be opened for reading\n", infile);
      return 1;
    }
#endif
  } else {
    if (infile[0] !=0) {
      f=fopen(infile,"r");
      if (f==NULL) {
        fprintf(stderr, "File %s cannot be opened for reading\n", infile);
        return 1;
      }
    } else {
      f=stdin;
    }
  }

  k=loader_open(loader_handle);
  if (k!=0) {
    return 1;
  }

  for (;;) {
    done=0;
    len=0;

    if (gzexpr) {
#ifdef HAVE_ZLIB
      len = gzread(f2, Buff, BUFFSIZE);
      if (len<0) {
        fprintf(stderr, "Read error\n");
        exit(-1);
      }
      done = gzeof(f2);
#endif
    } else {
      len = fread(Buff, 1, BUFFSIZE, f);
      if (ferror(f)) {
        fprintf(stderr, "Read error\n");
        exit(-1);
      }
      done = feof(f);
    }

    if (! XML_Parse(p, Buff, len, done)) {
      fprintf(stderr, "Parse error at line %d:\n%s\n",
	      XML_GetCurrentLineNumber(p),
	      XML_ErrorString(XML_GetErrorCode(p)));
      exit(-1);
    }

    if (done)
      break;
  }

  if (gzexpr) {
#ifdef HAVE_ZLIB
    gzclose(f2);
#endif
  } else {
    fclose(f);
  }
  if (loader_close(loader_handle)) {
    return 1;
  }
  *loaded_rows=loaded_rows2;

  return 0;
}

void oncleanupgzx(const char *fname, void *handle, const void *logfile,
                  onlogerror_lpt onlogerror, int verbose,
                  const char *datahandleoptions)
{
}

