/******************************************************************************
*
* Utility for copying data between Informix tables
*
* Author: Robert Sosnowski, Robert.Sosnowski@bzwbk.pl
*
* Version 1.70, 17-10-2005
*
* Copyright (c) 2001 - 2007. All Rights Reserved.
*
* Permission granted to use & distribute for free.
*
* This software can be used only at your own risk. Author is not responsible 
* for any damage caused by using this software.
* 
******************************************************************************/

#include "load2cfg.h"

#ifndef HAVE_GETOPT
extern int      optind;
extern char    *optarg;
extern int	opterr;
int getopt(int argc, char *argv[], char *optionS);
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "load2.h"
#include "unload2.h"
#include "tempfile.h"
#include <time.h>

#define MAX_FIELDNAME TBLNAME_SIZE+1

EXEC SQL include sqltypes;

#ifndef HAVE_SNPRINTF
#ifdef HAVE__SNPRINTF
#define snprintf _snprintf
#else
#include <stdarg.h>
int snprintf(char *str, size_t count, ...)
{int i;
 va_list args;
 i= sprintf(str, args);
 va_end(args);
return i;
} 
#endif
#endif


static char *id="@(#) $Id: tbcopy.ec 2521 2007-11-21 08:55:56Z sosnor01 $";

int usage( char * );
int is_critical_std(long int sqlcode, long int isamcode, int errtype,
                    int errcnt, int maxerrors, int *errinc);
int oninit_copy_unl(char *fname, const char *tablename, int batchsize,
                    void **handle, char *escapech, const void *logfile,
                    int fieldsnum, char *slctstmt2,
                    onlogerror_upt onlogerror, int verbose,
                    const char *datahandleoptions);
int oninit_copy_load(const char *fname, const char *tablename, const int batchsize,
                     void **handle, const void *logfile, const int fieldsnum,
                     const char *insstmt2, onlogerror_lpt onlogerror, int verbose,
                     const char *datahandleoptions);
int onbatch_copy(const char *fname, const char *tablename, const void *handle,
                 const unsigned long int rownum, const unsigned long int loaded,
                 const unsigned long int inserted,
                 const unsigned long int updated,
                 const unsigned long int deleted,
                 const void *logfile, onlogerror_lpt onlogerror, int verbose);
int onfinish_copy_load(const char *fname, const char *tablename, const void *handle,
                  const unsigned long int rownum,
                  const unsigned long int rownum2,
                  const unsigned long int inserted,
                  const unsigned long int updated,
                  const unsigned long int deleted,
                  const void *logfile,
                  onlogerror_lpt onlogerror, const int verbose);
int onfinish_copy_unl(char *fname, char *tablename, void *handle,
                  unsigned long int rownum, const void *logfile,
                  onlogerror_upt onlogerror, int verbose);
int blob_copy(loc_t *unl_blob, loc_t *load_blob, unsigned long int rownum,
          unsigned long int loaded, const void *logfile,
          onlogerror_upt onlogerror, int verbose);


int main( int argc, char **argv )
{

  EXEC SQL BEGIN DECLARE SECTION;
  static char sdbname[TBLNAME_SIZE];
  static char sserver[TBLNAME_SIZE];
  static char sserver2[TBLNAME_SIZE+1];
  static char suser[USERNAME_SIZE];
  static char spassword[USERNAME_SIZE];
  static char stabname[TBLNAME_SIZE], tabalias[TBLNAME_SIZE];
  static char tdbname[TBLNAME_SIZE];
  static char tserver[TBLNAME_SIZE];
  static char tserver2[TBLNAME_SIZE+1];
  static char tuser[USERNAME_SIZE];
  static char tpassword[USERNAME_SIZE];
  static char ttabname[TBLNAME_SIZE];
  static char cmd[300];
  EXEC SQL END DECLARE SECTION;

  static char  sqstr[5000];
  static char  ssqlfilename[128];
  static char  tqstr[5000];
  static char  tsqlfilename[128];
  static char  logfilename[128];
  short verbosity, use_ins_curs, use_commit, benchm;
  time_t t1, t2;
  int batchsize, batchsize2;
  int ch;
  FILE * fsqlfile, *logfile;
  unsigned long int i;
  char isolation[3];
  int same_server, tdef_serv, sdef_serv, same_db;
  unl_handle_t *unl_handle;
  loader_handle_t *loader_handle;
  int j,k,l;
  int fieldsnum;
  unsigned long int unloaded_rows, loaded_rows;
  int all_records;
  char lockedtables[200], *p, *q;
  int lockmodewait;
  struct sqlvar_struct *col;



  verbosity=3; /* show errors and progress messages */
  use_ins_curs=0;
  logfilename[0]=0;
  ssqlfilename[0]=0;
  tsqlfilename[0]=0;
  batchsize=10000;
  sserver[0]=0;
  suser[0]=0;
  spassword[0]=0;
  tserver[0]=0;
  tuser[0]=0;
  tpassword[0]=0;
  logfile=NULL;
  benchm=0;
  isolation[0]=0;
  strcpy(tabalias, "<table>");
  lockedtables[0]=0;
  lockmodewait=0;

 
  setvbuf(stdout,NULL,_IONBF,0);
  if ( argc == 1 ) {
    usage( argv[0] );
    exit( -1 );
  }
    

  while ((ch = getopt( argc, argv, "Vd:s:u:p:e:t:c:D:S:U:P:E:T:C:n:i:zv:l:Ba:r:w" )) != EOF) {
    switch (ch) {
      case 'V':
        fprintf( stdout,
         "\ntbcopy ver 1.70, Copyright 2001 - 2007 Robert Sosnowski\n\n" );
        exit( 0 );
        break;
      case 'd':
        /* source database name */
        strcpy( sdbname, optarg );
        break;
      case 's':
        /* source informix server */
        strcpy( sserver, optarg );
        break;
      case 'u':
        /* source user */
        strcpy( suser, optarg );
        break;
      case 'p':
        /* source password */
        strcpy( spassword, optarg );
        break;
      case 'e': 
        /* source SQL statement */
        strcpy( sqstr, optarg );
        break;
      case 't': 
        /* source table name */
        strcpy( stabname, optarg );
        strcpy( tabalias, optarg);
        break;
      case 'c': 
        /* source file with SQL statement */
        strcpy( ssqlfilename, optarg );
        break;
      case 'D':
        /* target database name */
        strcpy( tdbname, optarg );
        break;
      case 'S':
        /* target informix server */
        strcpy( tserver, optarg );
        break;
      case 'U':
        /* target user */
        strcpy( tuser, optarg );
        break;
      case 'P':
        /* target password */
        strcpy( tpassword, optarg );
        break;
      case 'E': 
        /* target SQL statement */
        strcpy( tqstr, optarg );
        break;
      case 'T': 
        /* target table name */
        strcpy( ttabname, optarg );
        break;
      case 'C': 
        /* target file with SQL statement */
        strcpy( tsqlfilename, optarg );
        break;
      case 'n':   
        /* batchsize */
        batchsize=(int)atoi( optarg );
        break;
      case 'z':
        /* use insert cursor */
        use_ins_curs = 1;
        break;
      case 'i':
        /* isolation level */
        strncpy( isolation, optarg, 1 );
	isolation[1]=0;
        break;
      case 'l':
        /* log file */
        strcpy( logfilename, optarg );
        break;
      case 'v':
        /* verbosity */
        verbosity=(int)atoi( optarg );
        break;
      case 'B':
        /* Benchmark mode */
	benchm=1;
        break;
      case 'a':
        /* Table alias for progress messages */
        strcpy( tabalias, optarg);
        break;
      case 'r':
        /* Lock tables in share mode */
        strcpy(lockedtables,optarg);
        break;
      case 'w':
        /* Set lock mode to wait */
        lockmodewait=1;
        break;
      default:
        usage( argv[0] );
        exit(2);
    }
  }

  if (strlen( sdbname ) == 0 && strlen( tdbname ) == 0) {
    fprintf( stdout, "Must supply database (-d)!\n" );
    usage( argv[0] );
    exit(2);
  }
  if (strlen( sdbname ) == 0) {
    strcpy(sdbname, tdbname);
  }
  if (strlen( tdbname ) == 0) {
    strcpy(tdbname, sdbname);
  }
  same_db=0;
  if (!strcmp(sdbname, tdbname)) {
    same_db=1;
  }

  FetBufSize=30000;
    
  EXEC SQL WHENEVER ERROR CONTINUE;

  if (strlen(sserver)==0 && (p = strstr(sdbname, "@")))
  {
    p[0] = '\0';
    strcpy(sserver, p+1);
  }

  if (strlen(tserver)==0 && (p = strstr(tdbname, "@")))
  {
    p[0] = '\0';
    strcpy(tserver, p+1);
  }

  same_server=0;
  tdef_serv=0;
  sdef_serv=0;
  if (strlen(sserver)==0 && strlen(tserver)==0) {
    /* server is not specified at all */
    /* this is allowed in case both source and target is default server */
    /* plus database have to be the same: switching database within one connection */
    /* causes cursor(s) to be closed */
    if (!same_db) {
      fprintf( stderr, "Both connections cannot be default\n");
      return 1;
    }
    same_server=1;
    tdef_serv=1;
    sdef_serv=1;
  }

  if (strlen(sserver)!=0 || strlen(suser)!=0 || strlen(spassword)!=0) {
    if (strlen(sserver)==0) {
      fprintf( stderr, "Informix server should be specified with -s option\n");
      return 1;
    }
    strcpy(sserver2,"@");
    strcat(sserver2, sserver);
    if ( strlen(suser)==0 && strlen(spassword)==0) {
      EXEC SQL CONNECT TO :sserver2 AS "sconn" WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        if ( strlen(tuser)!=0 && strlen(tpassword)!=0) {
          fprintf( stderr, "Error while connecting server %s with no user: "INT4_FORMAT"\n",
                   sserver, sqlca.sqlcode);
          return 1;
        }
      }
    } else {
      if ( strlen(suser)==0 || strlen(spassword)==0) {
    	fprintf( stderr, "Both options: -u and -p should be provided\n");
        return 1;
      }

      EXEC SQL CONNECT TO :sserver2 AS "sconn" USER :suser USING :spassword
               WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting server %s as user %s:"INT4_FORMAT"\n",
	         sserver, suser, sqlca.sqlcode);
        return 1;
      }
    }
  } else {
    EXEC SQL CONNECT TO DEFAULT WITH CONCURRENT TRANSACTION;
    if (sqlca.sqlcode<0) {
      fprintf( stderr, "Error while connecting default server: "INT4_FORMAT"\n",
               sqlca.sqlcode);
      return 1;
    }
    sdef_serv=1;
  }
  if (strlen(tserver)!=0 || strlen(tuser)!=0 || strlen(tpassword)!=0) {
    if (strlen(tserver)==0) {
      fprintf( stderr, "Informix server should be specified with -S option\n");
      return 1;
    }
    strcpy(tserver2,"@");
    strcat(tserver2, tserver);
    if ( strlen(tuser)==0 && strlen(tpassword)==0) {
      EXEC SQL CONNECT TO :tserver2 AS "tconn" WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
          fprintf( stderr, "Error while connecting server %s with no user:"INT4_FORMAT"\n",
                   tserver, sqlca.sqlcode);
          return 1;
      }
    } else {
      if ( strlen(tuser)==0 || strlen(tpassword)==0) {
    	fprintf( stderr, "Both options: -U and -P should be provided\n");
        return 1;
      }

      EXEC SQL CONNECT TO :tserver2 AS "tconn" USER :tuser USING :tpassword
               WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting server %s as user %s:"INT4_FORMAT"\n",
                 tserver, tuser, sqlca.sqlcode);
        return 1;
      }
    }
  } else {
    if (sdef_serv!=1) {
      /* source server is connected non-default */
      EXEC SQL CONNECT TO DEFAULT WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting default server: "INT4_FORMAT"\n",
                 sqlca.sqlcode);
        return 1;
      }
      tdef_serv=1;
    } else {
      /* source and target server are both default */
      if (same_db) {
        same_server=1;
      } else {
        fprintf( stderr, "Both connections cannot be default\n");
        return 1;
      }
    }
  }
  if ( !same_server && tdef_serv!=1 && sdef_serv!=1) {
    if (strcmp(sserver,tserver)==0) {
      same_server=1;
    }
  }
  if ( !same_server) {
    same_db=0;
    if (sdef_serv) {
      EXEC SQL SET CONNECTION DEFAULT;
    } else {
      EXEC SQL SET CONNECTION 'sconn';
    }
    EXEC SQL DATABASE :sdbname;
    if (sqlca.sqlcode < 0) {
      fprintf( stderr, "Cannot open source database %s.  Code: "INT4_FORMAT", ISAM: "INT4_FORMAT".\n",
               sdbname, sqlca.sqlcode, sqlca.sqlerrd[1] );
      return 1;
    }

    if (tdef_serv) {
      EXEC SQL SET CONNECTION DEFAULT;
    } else {
      EXEC SQL SET CONNECTION 'tconn';
    }
    EXEC SQL DATABASE :tdbname;
    if (sqlca.sqlcode < 0) {
      fprintf( stderr, "Cannot open target database %s.  Code: "INT4_FORMAT", ISAM: "INT4_FORMAT".\n",
               tdbname, sqlca.sqlcode, sqlca.sqlerrd[1] );
      return 1;
    }
    use_commit = (sqlca.sqlwarn.sqlwarn1 == 'W');
  } else {
    EXEC SQL DATABASE :sdbname;
    if (sqlca.sqlcode < 0) {
      fprintf( stderr, "Cannot open source database %s.  Code: "INT4_FORMAT", ISAM: "INT4_FORMAT".\n",
               sdbname, sqlca.sqlcode, sqlca.sqlerrd[1] );
      return 1;
    }
    EXEC SQL DATABASE :tdbname;
    if (sqlca.sqlcode < 0) {
      fprintf( stderr, "Cannot open target database %s.  Code: "INT4_FORMAT", ISAM: "INT4_FORMAT".\n",
               tdbname, sqlca.sqlcode, sqlca.sqlerrd[1] );
      return 1;
    }
    use_commit = (sqlca.sqlwarn.sqlwarn1 == 'W');

  }


  if (strlen(sqstr)==0 && strlen(ssqlfilename)==0 && strlen(stabname)==0) {
    strcpy(stabname,ttabname);
  }
  if (strlen( sqstr ) != 0 ) {
    if (strlen(stabname)!=0) {
      fprintf(stderr, "You can't specify both -e and -t options!\n");
      return 1;
    }
    if (strlen(ssqlfilename)!=0) {
      fprintf(stderr, "You can't specify both -e and -c options!\n");
      return 1;
    }
  } else {
    if (strlen(stabname) != 0) {
      if (strlen(ssqlfilename)!=0) {
        fprintf(stderr, "You can't specify both -t and -c options!\n");
        return 1;
      }
      sprintf(sqstr, "select * from %s", stabname);
    } else {
      if (strlen(ssqlfilename)!=0) {
        fsqlfile=fopen(ssqlfilename, "r");
        if (fsqlfile) {
          i=fread(sqstr, 1, sizeof(sqstr), fsqlfile);
          sqstr[i]=0;
          fclose(fsqlfile);
        } else {
          perror("Can't open sql file");
          return 1;
        }
      } else {
        fprintf(stderr, "You have to specify one of: -e -t or -c option!\n");
        return 1;
      }
    }
  }

  if (strlen(tqstr)==0 && strlen(tsqlfilename)==0 && strlen(ttabname)==0) {
    strcpy(ttabname,stabname);
  }
  if (strlen( tqstr ) != 0 ) {
    if (strlen(ttabname)!=0) {
      fprintf(stderr, "You can't specify both -E and -T options!\n");
      return 1;
    }
    if (strlen(tsqlfilename)!=0) {
      fprintf(stderr, "You can't specify both -E and -C options!\n");
      return 1;
    }
  } else {
    if (strlen(ttabname) != 0) {
      if (strlen(tsqlfilename)!=0) {
        fprintf(stderr, "You can't specify both -T and -C options!\n");
        return 1;
      }
      sprintf(tqstr, "insert into %s", ttabname);
    } else {
      if (strlen(tsqlfilename)!=0) {
        fsqlfile=fopen(tsqlfilename, "r");
        if (fsqlfile) {
          i=fread(tqstr, 1, sizeof(tqstr), fsqlfile);
          tqstr[i]=0;
          fclose(fsqlfile);
        } else {
          perror("Can't open sql file");
          return 1;
        }
      } else {
        fprintf(stderr, "You have to specify one of: -E -T or -C option!\n");
        return 1;
      }
    }
  }


  if (logfilename[0]) {
    logfile=fopen(logfilename,"a");
#ifdef _IONBF
    if (logfile!=NULL) {
      setvbuf(logfile, NULL, _IONBF,0);
    }
#endif
  }
  if (benchm) {
    t1=time(NULL);
    if (logfilename[0]) {
      fprintf(logfile, "Transfer started at %s",ctime(&t1));
    }
    fprintf(stdout, "Transfer started at %s",ctime(&t1));
  }

  if (verbosity<0 || verbosity>4) {
    fprintf(stderr, "Wrong verbosity value\n");
    return 1;
  }

  unl_handle=unl_create();
  if (unl_handle==NULL) {
    return 1;
  }
  loader_handle=loader_create ();
  if (loader_handle==NULL) {
    return 1;
  }

  k=unl_setstmt(unl_handle, sqstr);
  if (k!=0) {
    return 1;
  }

  k=loader_setstmt(loader_handle, tqstr );
  if (k!=0) {
    return 1;
  }

  if (batchsize<0) {
    batchsize2=0;
  } else {
    batchsize2=batchsize;
  }
  unl_setbatchsize(unl_handle, batchsize2);
  loader_setbatchsize(loader_handle, batchsize2);

  loader_setusecursor(loader_handle, use_ins_curs);
  loader_setusecommit(loader_handle, use_commit);

  if (logfile==NULL) {
    unl_setloghandle(unl_handle, NULL);
  } else {
    unl_setloghandle(unl_handle, logfile);
  }
  if (logfile==NULL) {
    loader_setloghandle(loader_handle, NULL);
  } else {
	    loader_setloghandle(loader_handle, logfile);
  }

  switch (verbosity) { 
    case 0: 
      unl_setverbosity(unl_handle, 0);
      loader_setverbosity(loader_handle, 0);
      break;
    case 1: 
      unl_setverbosity(unl_handle, 20);
      loader_setverbosity(loader_handle, 20);
      break;
    case 2: 
      unl_setverbosity(unl_handle, 30);
      loader_setverbosity(loader_handle, 30);
      break;
    case 3: 
      unl_setverbosity(unl_handle, 40);
      loader_setverbosity(loader_handle, 40);
      break;
    case 4: 
      unl_setverbosity(unl_handle, 50);
      loader_setverbosity(loader_handle, 50);
      break;
    default: 
      unl_setverbosity(unl_handle, 30);
      loader_setverbosity(loader_handle, 30);
      break;
  }

  unl_settablename(unl_handle, tabalias);
  unl_setfetcharray(unl_handle, 1);


  loader_setoninit(loader_handle, &oninit_copy_load);
  unl_setoninit(unl_handle, &oninit_copy_unl);
  loader_setonbatch(loader_handle, &onbatch_copy);
  loader_setonfinish(loader_handle, &onfinish_copy_load);
  unl_setonfinish(unl_handle, &onfinish_copy_unl);

  k=unl_setdatahandlename(unl_handle, "");
  if (k!=0) {
    return 1;
  }

  k=loader_setdatahandlename(loader_handle, ""); /* always stdin */
  if (k!=0) {
    return 1;
  }

  unloaded_rows=0;
  loaded_rows=0;

  if (!same_db) {
    if (sdef_serv) {
      EXEC SQL SET CONNECTION DEFAULT;
    } else {
      EXEC SQL SET CONNECTION 'sconn';
    }
  }

  k=unl_prepare(unl_handle);
  if (k!=0) {
    return 1;
  }

  for (col = unl_handle->udesc->sqlvar, i = 0; i < unl_handle->udesc->sqld; col++, i++) {
    if (col->sqltype==CFIXBINTYPE) {
/* workaround for Informix bug: Fetch array don't work with smart blobs
   sometimes errror 1820, sometimes (different column order) fetch crashes */
      unl_free(unl_handle);
      unl_handle=unl_create();
      if (unl_handle==NULL) {
        return 1;
      }
      k=unl_setstmt(unl_handle, sqstr);
      if (k!=0) {
        return 1;
      }
      unl_setbatchsize(unl_handle, batchsize2);
      if (logfile==NULL) {
        unl_setloghandle(unl_handle, NULL);
      } else {
        unl_setloghandle(unl_handle, logfile);
      }
      switch (verbosity) { 
        case 0: 
          unl_setverbosity(unl_handle, 0);
          break;
        case 1: 
          unl_setverbosity(unl_handle, 20);
          break;
        case 2: 
          unl_setverbosity(unl_handle, 30);
          break;
        case 3: 
          unl_setverbosity(unl_handle, 40);
          break;
        case 4: 
          unl_setverbosity(unl_handle, 50);
          break;
        default: 
          unl_setverbosity(unl_handle, 30);
          break;
      }
      unl_settablename(unl_handle, tabalias);
      unl_setfetcharray(unl_handle, 0);
      unl_setoninit(unl_handle, &oninit_copy_unl);
      unl_setonfinish(unl_handle, &onfinish_copy_unl);
      k=unl_setdatahandlename(unl_handle, "");
      if (k!=0) {
        return 1;
      }
      k=unl_prepare(unl_handle);
      if (k!=0) {
        return 1;
      }
      break;
    }
  }

  if (lockmodewait) {
    EXEC SQL set lock mode to wait;
  }
  if (lockedtables[0]) {
    EXEC SQL begin work;
    p=lockedtables;
    q=p;
    while (p!=NULL) {
      p=strchr(q,',');
      if (p==NULL) {
        snprintf(cmd,sizeof(cmd),"lock table %s in share mode", q);
      } else {
        strcpy(cmd,"lock table ");
        l=strlen(cmd);
        i=p-q;
        while (q!=p) {
          cmd[i-(p-q)+l]=lockedtables[q-lockedtables];
          q++;
        }
        q=p+1;
        cmd[l+i]=0;
        strcpy(cmd+l+i," in share mode");
      }
      EXEC SQL execute immediate :cmd;
      if (sqlca.sqlcode!=0) {
        fprintf(stderr, "ERROR: can't lock table in share mode: "INT4_FORMAT","INT4_FORMAT",%s\n",
                sqlca.sqlcode,sqlca.sqlerrd[1],sqlca.sqlerrm);
        return 1;
      }
    }
  }
  if (isolation[0]==0 || isolation[0]=='0') {
    EXEC SQL set isolation to dirty read;
  } else {
    if (isolation[0]=='1') {
      EXEC SQL set isolation to committed read;
    } else {
      if (isolation[0]=='2') {
        EXEC SQL set isolation to cursor stability;
      } else {
        if (isolation[0]=='3') {
          EXEC SQL set isolation to repeatable read;
        } else {
          fprintf(stderr, "Wrong isolation level:%s\n", isolation);
          return 1;
  }}}}


  fieldsnum=unl_getfieldsnum(unl_handle);
  k=unl_open(unl_handle);
  if (k!=0) {
    return 1;
  }

  if (!same_db) {
    if (tdef_serv) {
      EXEC SQL SET CONNECTION DEFAULT;
    } else {
      EXEC SQL SET CONNECTION 'tconn';
    }
  }

  k=loader_prepare(loader_handle);
  if (k!=0) {
    return 1;
  }

  k=loader_getfieldsnum(loader_handle);
  if (k!=fieldsnum) {
    fprintf(stderr, "Source and target have different number of fields\n");
    return 1;
  }

  k=loader_open(loader_handle);
  if (k!=0) {
    return 1;
  }

  while (1) {
    if (!same_db) {
      if (sdef_serv) {
	EXEC SQL SET CONNECTION DEFAULT;
      } else {
	EXEC SQL SET CONNECTION 'sconn';
      }
    }

    /* get record from database */
    k=unl_getrecord(unl_handle, &all_records);
    if (k!=0) {
      return 1;
    }
    if (all_records) {
      break;
    }
    if (!same_db) {
      if (tdef_serv) {
	EXEC SQL SET CONNECTION DEFAULT;
      } else {
	EXEC SQL SET CONNECTION 'tconn';
      }
    }

    if (FetArrSize>1 && unl_handle->fetarrsize>1) {
      for (j=0;j<unl_handle->fetarrsize;j++) {
        for (k=0;k<fieldsnum;k++) {
          *((short *)((loader_handle->ldesc->sqlvar+k)->sqlind))=*(((short *)(unl_handle->udesc->sqlvar+k)->sqlind)+j);
          if (((unl_handle->udesc->sqlvar+k)->sqltype)==CLOCATORTYPE
           || ((unl_handle->udesc->sqlvar+k)->sqltype)==SQLBYTES
           || ((unl_handle->udesc->sqlvar+k)->sqltype)==SQLTEXT) {
            i=blob_copy((loc_t *)(((unl_handle->udesc->sqlvar+k)->sqldata)+j*unl_handle->typmsizes[k]),
                      (loc_t *)((loader_handle->ldesc->sqlvar+k)->sqldata),
                      loader_handle->rownum, loader_handle->rownum2,
                      logfile, loader_handle->onlogerror, loader_handle->verbosity);
            if (i!=0) {
              return 1;
            }
          } else {
            bycopy((((unl_handle->udesc->sqlvar+k)->sqldata)+j*unl_handle->typmsizes[k]),
                   ((loader_handle->ldesc->sqlvar+k)->sqldata),
                   unl_handle->typmsizes[k]);
          }
        }
        k=loader_putrecord(loader_handle);
        if (k!=0) {
          return 1;
        }
      }
    } else {
      for (k=0;k<fieldsnum;k++) {
        *((short *)((loader_handle->ldesc->sqlvar+k)->sqlind))=*(((short *)(unl_handle->udesc->sqlvar+k)->sqlind));
        if (((unl_handle->udesc->sqlvar+k)->sqltype)==CLOCATORTYPE
         || ((unl_handle->udesc->sqlvar+k)->sqltype)==SQLBYTES
         || ((unl_handle->udesc->sqlvar+k)->sqltype)==SQLTEXT) {
          i=blob_copy((loc_t *)(((unl_handle->udesc->sqlvar+k)->sqldata)),
                    (loc_t *)((loader_handle->ldesc->sqlvar+k)->sqldata),
                    loader_handle->rownum, loader_handle->rownum2,
                    logfile, loader_handle->onlogerror, loader_handle->verbosity);
          if (i!=0) {
            return 1;
          }
        } else {
          bycopy((((unl_handle->udesc->sqlvar+k)->sqldata)),
                 ((loader_handle->ldesc->sqlvar+k)->sqldata),
                 unl_handle->typmsizes[k]);
        }
      }
      k=loader_putrecord(loader_handle);
      if (k!=0) {
        return 1;
      }
    }
  }

  if (!same_db) {
    if (tdef_serv) {
      EXEC SQL SET CONNECTION DEFAULT;
    } else {
      EXEC SQL SET CONNECTION 'tconn';
    }
  }

  if (loader_close(loader_handle)) {
    return 1;
  }
  loaded_rows=loader_getloadedrows(loader_handle);
  if (loader_free(loader_handle)) { /* = loader_unprepare */
    return 1;
  }
  loader_handle=NULL;

  if (benchm) {
    t2=time(NULL);
    i=t2-t1;
    if (logfilename[0]) {
      fprintf(logfile, "Transfer finished at %s",ctime(&t2));
      fprintf(logfile, "Elapsed seconds: %ld\n",i);
    }
    fprintf(stdout, "Transfer finished at %s",ctime(&t2));
    fprintf(stdout, "Elapsed seconds: %ld\n",i);
  }
  if (logfilename[0]) {
    fclose(logfile);
  }

  if (!same_db) {
    if (sdef_serv) {
      EXEC SQL SET CONNECTION DEFAULT;
    } else {
      EXEC SQL SET CONNECTION 'sconn';
    }
  }
  if (lockedtables[0]) {
    EXEC SQL commit work;
  }

  if (unl_close(unl_handle)) {
    return 1;
  }
  unloaded_rows=unl_getunloadedrows(unl_handle);
  if (unl_free(unl_handle)) { /* = loader_unprepare */
    return 1;
  }
  unl_handle=NULL;


  return 0; 

}


int usage( char *prog )
{


  fprintf( stdout, "\n");
  fprintf( stdout, "%s - copy table data between Informix tables.\n\n", prog );

  fprintf( stdout, "Usage: \n" );
  fprintf( stdout, 
           "\t%s [-V] -d <db_name> [-s <server> -u <user> -p <password>]\n "
           "(-e <select stmt>|-t <table>|-c <sql file>)\n "
           "-D <db_name> [-S <server> -U <user> -P <password>]\n "
           "(-E <insert stmt>|-T <table>|-C <sql file>) [-n <batch size>]\n "
           "[-i <level>] [-z] [-l <logfile>] [-B] [-v <verbosity>] \n"
           "[-a <table alias>] [-r <tables>] [-w]\n\n",
           prog);
  fprintf( stdout, "\t\t-V - version information\n" ); 
  fprintf( stdout, "\t\t-d - source database name\n" );
  fprintf( stdout, "\t\t-s - source informix server name\n" );
  fprintf( stdout, "\t\t-u - user on source server\n" );
  fprintf( stdout, "\t\t-p - password on source server\n" );
  fprintf( stdout, "\t\t-e - SQL select statement in quotes\n" );
  fprintf( stdout, "\t\t-t - source table name\n" );
  fprintf( stdout, "\t\t-c - file containing SQL select statement\n" );
  fprintf( stdout, "\t\t-D - target database name\n" );
  fprintf( stdout, "\t\t-S - target informix server name\n" );
  fprintf( stdout, "\t\t-U - user on target server\n" );
  fprintf( stdout, "\t\t-P - password on target server\n" );
  fprintf( stdout, "\t\t-E - SQL insert statement in quotes\n" );
  fprintf( stdout, "\t\t-T - target table name\n" );
  fprintf( stdout, "\t\t-C - file containing SQL insert statement\n" );
  fprintf( stdout, "\t\t-n - batch size\n" );
  fprintf( stdout, "\t\t-z - use insert cursor\n" );
  fprintf( stdout, "\t\t-i - isolation level\n" );
  fprintf( stdout, "\t\t   0 - dirty read (default)\n" );
  fprintf( stdout, "\t\t   1 - commited read\n" );
  fprintf( stdout, "\t\t   2 - cursor stability\n" );
  fprintf( stdout, "\t\t   3 - repeatable read\n" );
  fprintf( stdout, "\t\t-l - log file name\n" );
  fprintf( stdout, "\t\t-B - benchmark\n" );
  fprintf( stdout, "\t\t-v - verbosity:\n" );
  fprintf( stdout, "\t\t     0 - completely quiet\n" );
  fprintf( stdout, "\t\t     1 - show errors only\n" );
  fprintf( stdout, "\t\t     2 - summary messages\n" );
  fprintf( stdout, "\t\t     3 - detailed messages (default)\n" );
  fprintf( stdout, "\t\t     4 - debug info\n" );
  fprintf( stdout, "\t\t-a - table alias for progress messages\n" );
  fprintf( stdout, "\t\t-r <tables> - lock tables (,-separated) in share mode\n" );
  fprintf( stdout, "\t\t-w - set lock mode to wait\n" );

  return 0;
} 

int oninit_copy_unl(char *fname, const char *tablename, int batchsize,
                    void **handle, char *escapech, const void *logfile,
                    int fieldsnum, char *slctstmt2,
                    onlogerror_upt onlogerror, int verbose,
                    const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];


  if (strcmp("<table>",tablename)) {
    sprintf(errstr,"Copying from table %s initiated", tablename);
  } else {
    sprintf(errstr,"Copying from table initiated");
  }
  (*handle) = NULL;    
  if (verbose>=10) {
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
               verbose, 30);
  }
  (*escapech)='\\';
  return 0;
}

int oninit_copy_load(const char *fname, const char *tablename, const int batchsize,
                     void **handle, const void *logfile, const int fieldsnum,
                     const char *insstmt2, onlogerror_lpt onlogerror, int verbose,
                     const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];

  if ( (*handle) == NULL ) {
      sprintf(errstr, "Copying to table %s initiated", tablename);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
                 verbose, 30);
    (*handle)=NULL;
  }
  return 0;
}

int onbatch_copy(const char *fname, const char *tablename, const void *handle,
                 const unsigned long int rownum, const unsigned long int loaded,
                 const unsigned long int inserted,
                 const unsigned long int updated,
                 const unsigned long int deleted,
                 const void *logfile, onlogerror_lpt onlogerror, int verbose)
{
  char errstr[ERR_BUF_SIZE];

  sprintf(errstr, "Transfered succesfully %ld records", loaded);
  onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
             verbose, 40);
  return 0;
}

int onfinish_copy_load(const char *fname, const char *tablename, const void *handle,
                  const unsigned long int rownum,
                  const unsigned long int rownum2,
                  const unsigned long int inserted,
                  const unsigned long int updated,
                  const unsigned long int deleted,
                  const void *logfile,
                  onlogerror_lpt onlogerror, const int verbose)
{
  char errstr[ERR_BUF_SIZE];

  snprintf(errstr, ERR_BUF_SIZE, "%ld records were succesfully transfered" ,
           rownum2);
  onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, 0, logfile,
             verbose, 30);
  return 0;
}

int onfinish_copy_unl(char *fname, char *tablename, void *handle,
                      unsigned long int rownum, const void *logfile,
                      onlogerror_upt onlogerror, int verbose)
{   
  return 0;
}

int blob_copy(loc_t *unl_blob, loc_t *load_blob, unsigned long int rownum,
          unsigned long int loaded, const void *logfile,
          onlogerror_upt onlogerror, int verbose)
{ char errstr[ERR_BUF_SIZE];

  if (unl_blob->loc_position==1) {
    load_blob->loc_loctype=LOCMEMORY;
    load_blob->loc_bufsize=unl_blob->loc_bufsize;
    load_blob->loc_size=unl_blob->loc_size;
    load_blob->loc_mflags=0;
    load_blob->loc_oflags=0;
/*    load_blob->loc_buffer=(char *)load2_malloc(load_blob->loc_bufsize);
    if (load_blob->loc_buffer==NULL) {
      sprintf(errstr,"Out of memory for blob");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
         0, logfile, verbose, 10);
      return 1;
    }
    bycopy(unl_blob->loc_buffer, load_blob->loc_buffer, load_blob->loc_bufsize);
*/
    load_blob->loc_buffer=unl_blob->loc_buffer;
    /* above is sufficient because unl_blob->loc_buffer is freed in
       onwritetextblob and it is never called (at least in version 1.70) */
  } else {
    if (unl_blob->loc_position!=2) {
      sprintf(errstr,"Incorrect BLOB storage: "INT4_FORMAT"", unl_blob->loc_position);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
         0, logfile, verbose, 10);
      return 1; 
    }
    load_blob->loc_loctype=LOCFNAME;
    load_blob->loc_size=-1;
    load_blob->loc_oflags=LOC_RONLY;
    load_blob->loc_fname=(char *)load2_malloc(unl_blob->loc_bufsize, __FILE__, __LINE__);
    if (load_blob->loc_fname==NULL) {
      sprintf(errstr,"Out of memory for blob");
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, loaded,
         0, logfile, verbose, 10);
      return 1;
    }
    strcpy(load_blob->loc_fname, unl_blob->loc_buffer);
    load2_free(unl_blob->loc_buffer, __FILE__, __LINE__);
    unl_blob->loc_buffer=NULL;
  }
  load_blob->loc_indicator=unl_blob->loc_indicator;
  load_blob->loc_type=unl_blob->loc_type;
  return 0;
}
