/******************************************************************************
*
* Procedure for creating temporary files
*
* Author: Robert Sosnowski, Robert.Sosnowski@bzwbk.pl
*
* Version 1.70, 17-10-2005
*
* Copyright (c) 2001 - 2007. All Rights Reserved.
*
* Permission granted to use & distribute for free. Changes in source code
* require Author's acceptance.
*
* This software can be used only at your own risk. Author is not responsible 
* for any damage caused by using this software.
* 
******************************************************************************/

#include "load2cfg.h"
#include "tempfile.h"

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_IO_H
#include <io.h>
/* _mktemp, creat prototypes in MSVC */
#endif
#include <stdio.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <stdlib.h>

static char *id="@(#) $Id: tempfile.ec 2521 2007-11-21 08:55:56Z sosnor01 $";
static const char hexdigit[] = "0123456789abcdef";


char *creatmpfile(void)
/* horrible differences between systems for creating temporary files! */
{
  static char *tmp_dir = NULL;
  char template1[256];
  static char filename[256];
  int i;
  char *p1;
/*  FILE *p2; */

  if (tmp_dir==NULL) {
#ifdef HAVE_GETENV
    tmp_dir=getenv("DBTEMP");
    if (tmp_dir == NULL) {
      tmp_dir=getenv("TMPDIR");
    }
    if (tmp_dir == NULL) {
      tmp_dir=getenv("TMP");
    }
    if (tmp_dir == NULL) {
      tmp_dir=getenv("TEMP");
    }
#endif

#ifdef P_tmpdir
    if (tmp_dir == NULL) {
      tmp_dir=P_tmpdir;
    }
#endif

    if (tmp_dir == NULL) {
      tmp_dir=".";
    }
  }

/*    mktemp mkstemp tmpnam tmpfile tempnam _mktemp _tempnam
    FILE *tmpfile(void);
*/
  strcpy(template1, tmp_dir);

#ifdef NT
  strcat(template1,"\\");
#else
  strcat(template1,"/");
#endif

#ifdef HAVE_MKSTEMP
  strcat(template1, "zaXXXXXX");
  i=mkstemp(template1);
  if (i<0) {
    return NULL;
  }
  close(i);
  strcpy(filename, template1);
  return filename;
#elif defined(HAVE_MKTEMP) || defined(HAVE__MKTEMP)
#ifndef HAVE_MKTEMP
#define mktemp _mktemp
#endif
  strcat(template1,"zaXXXXXX");
  mktemp(template1);
  if (template1==NULL || template1[0]==0) {
    return NULL;
  }
  i=creat(template1, 00600 /* S_IRUSR | S_IWUSR */);
  if (i<0) {
    return NULL;
  }
  close(i);
  strcpy(filename, template1);
  return filename;
#elif defined(HAVE_TEMPNAM) || defined(HAVE__TEMPNAM)
#ifndef HAVE_TEMPNAM
#define tempnam _tempnam
#endif
  p1=tempnam(tmp_dir,"za");
  if (p1==NULL || p1[0]==0) {
    return NULL;
  }
  i=creat(p1,  00600 /* S_IRUSR | S_IWUSR */);
  if (i<0) {
    return NULL;
  }
  close(i);
  strcpy(filename, p1);
  load2_free(p1, __FILE__, __LINE__);
  return filename;
/* #elif defined(HAVE_TMPFILE)
  p2=tmpfile();
  if (p2==NULL) {
    return -1
  }
  i=fileno(p2);
*/
#elif defined(HAVE_TMPNAM)
  p1=tmpnam(NULL);
  if (p1==NULL || p1[0]==0) {
    return NULL;
  }
  i=creat(p1,  00600 /* S_IRUSR | S_IWUSR */);
  if (i<0) {
    return NULL;
  }
  close(i);
  strcpy(filename, p1);
  return filename;
#else
  fprintf(stderr, "Your OS does not provide known possibility for temp file creation\n");
  return NULL;
#endif

}


char *creatmpfile2(char *prefix, char *extension)
{
  char template1[256];
  static char filename[256];
  int i;
  char *p1;

/*    mktemp mkstemp tmpnam tmpfile tempnam _mktemp _tempnam
    FILE *tmpfile(void);
*/
  strcpy(template1, ".");

#ifdef NT
  strcat(template1,"\\");
#else
  strcat(template1,"/");
#endif

  strcat(template1, prefix);
#ifdef HAVE_MKSTEMP
  strcat(template1, "XXXXXX");
  i=mkstemp(template1);
  if (i<0) {
    return NULL;
  }
  close(i);
  unlink(template1);
  strcat(template1, extension);
  strcpy(filename, template1);
  return filename;
#elif defined(HAVE_MKTEMP) || defined(HAVE__MKTEMP)
#ifndef HAVE_MKTEMP
#define mktemp _mktemp
#endif
  strcat(template1,"XXXXXX");
  mktemp(template1);
  if (template1==NULL || template1[0]==0) {
    return NULL;
  }
  strcat(template1, extension);
  i=creat(template1, 00600 /* S_IRUSR | S_IWUSR */);
  if (i<0) {
    return NULL;
  }
  close(i);
  strcpy(filename, template1);
  return filename;
#elif defined(HAVE_TEMPNAM) || defined(HAVE__TEMPNAM)
#ifndef HAVE_TEMPNAM
#define tempnam _tempnam
#endif
  p1=tempnam(".",prefix);
  if (p1==NULL || p1[0]==0) {
    return NULL;
  }
  strcpy(filename,p1);
  load2_free(p1, __FILE__, __LINE__);
  strcat(filename,extension);
  i=creat(p1,  00600 /* S_IRUSR | S_IWUSR */);
  if (i<0) {
    return NULL;
  }
  close(i);
  return filename;
#else
  fprintf(stderr, "Your OS does not provide known possibility for temp file creation\n");
  return NULL;
#endif

}


void *load2_calloc(size_t nmemb, size_t size)
{void *p;
 p=calloc(nmemb, size);
#ifdef DEBUG_ALLOC
 printf("calloc:nmemb=%ld,size=%ld,ptr=%p\n",nmemb,size,p);
#endif
 return p;
}


void *load2_malloc(size_t size1, char *file1, int line1)
{void * p;

  p=malloc(size1);
#ifdef DEBUG_ALLOC
 printf("malloc in %s(%ld):",file1, line1);
 printf("size=%ld,ptr=%p\n",size1,p);
#endif
 return p;
}

void load2_free(void *ptr, char *file1, int line1)
{
#ifdef DEBUG_ALLOC
 printf("free in %s(%ld):ptr=",file1, line1);
 printf("%p\n", ptr);
#endif
 free(ptr);
}

mint int8split(ifx_int8_t bignum, int4 *hi, int4 *med1, int4 *med2, int4 *lo)
{
  int4 i;
  ifx_int8_t max4, t1, t2, hi8, lo8;
  mint stat;

  i=256*256;
  stat=ifx_int8cvlong(i,&max4);
  if (stat==0) {
    ifx_int8copy(&max4, &t2);
    stat=ifx_int8mul(&max4, &max4, &max4);
  }
  if (stat==0) {
    stat=ifx_int8div(&bignum, &max4, &hi8);
  }
  if (stat==0) {
    stat=ifx_int8mul(&max4, &hi8, &t1);
  }
  if (stat==0) {
    stat=ifx_int8sub(&bignum, &t1, &lo8);
  }
  if (stat==0) {
    stat=ifx_int8div(&hi8, &t2, &t1);
  }
  if (stat==0) {
    stat=ifx_int8tolong(&t1, hi);
  }
  if (stat==0) {
    stat=ifx_int8mul(&t2, &t1, &t1);
  }
  if (stat==0) {
    stat=ifx_int8sub(&hi8, &t1, &t1);
  }
  if (stat==0) {
    stat=ifx_int8tolong(&t1, med1);
  }
  if (stat==0) {
    stat=ifx_int8div(&lo8, &t2, &t1);
  }
  if (stat==0) {
    stat=ifx_int8tolong(&t1, med2);
  }
  if (stat==0) {
    stat=ifx_int8mul(&t2, &t1, &t1);
  }
  if (stat==0) {
    stat=ifx_int8sub(&lo8, &t1, &t1);
  }
  if (stat==0) {
    stat=ifx_int8tolong(&t1, lo);
  }
  return stat;  
}

mint off_tsplit(off_t bignum, ifx_int8_t *hi, ifx_int8_t *med1, ifx_int8_t *med2, ifx_int8_t *lo)
{
  off_t t1, hi8, lo8;
  mint stat;

  hi8 = (bignum >> 16); /* always 0 if sizeof(off_t)==4 */
  hi8 = (hi8 >> 16 ); /* hi8 >> 32 works incorrectly */
  lo8 = (hi8 << 16);
  lo8 = bignum - (lo8 << 16);
  t1 = hi8 >> 16;
  stat=ifx_int8cvlong(t1, hi);
  if (stat!=0) {
    return stat;
  }
  t1 = hi8 - (t1 << 16);
  stat=ifx_int8cvlong(t1, med1);
  if (stat!=0) {
    return stat;
  }
  t1 = lo8 >> 16;
  stat=ifx_int8cvlong(t1, med2);
  if (stat!=0) {
    return stat;
  }
  t1 = lo8 - (t1 << 16);
  stat=ifx_int8cvlong(t1, lo);
  if (stat!=0) {
    return stat;
  }
  return 0;
}

mint int8off_t(ifx_int8_t offset, off_t *offset2)
{
  mint stat;

  off_t t1, t2, t3, t4;
  int4 w1, w2, w3, w4;
  stat=int8split(offset, &w1, &w2, &w3, &w4);
  if (stat!=0) {
    return stat;
  }
  t1=w1;
  t2=w2;
  t3=w3;
  t4=w4;
  t1=(t1<<16) + t2;
  t1=(t1<<16) + t3;
  t1=(t1<<16) + t4;
  *offset2 = t1;
  return 0;
}

mint off_t2int8(off_t offset, ifx_int8_t *offset2)
{
  mint stat;
  int i;
  ifx_int8_t max2;
  ifx_int8_t t1, t2, t3, t4;
  off_t j;
  int4 k;

  j = (offset >> 16);
  j = (j >> 16);
  if (j==0) {
    k=offset;
    stat=ifx_int8cvlong(k, offset2);
    return stat;
  }

  stat=off_tsplit(offset, (ifx_int8_t *)&t1, (ifx_int8_t *)&t2,
                  (ifx_int8_t *)&t3, (ifx_int8_t *)&t4);
  if (stat!=0) {
    return stat;
  }
  i=256*256;
  stat=ifx_int8cvlong(i,&max2);
  if (stat!=0) {
    return stat;
  }
  
  stat=ifx_int8mul(&max2, &t1, &t1);
  if (stat!=0) {
    return stat;
  }
  stat=ifx_int8add(&t1, &t2, &t1);
  if (stat!=0) {
    return stat;
  }
  stat=ifx_int8mul(&max2, &t1, &t1);
  if (stat!=0) {
    return stat;
  }
  stat=ifx_int8add(&t1, &t3, &t1);
  if (stat!=0) {
    return stat;
  }
  stat=ifx_int8mul(&max2, &t1, &t1);
  if (stat!=0) {
    return stat;
  }
  stat=ifx_int8add(&t1, &t4, &t1);
  if (stat!=0) {
    return stat;
  }

  ifx_int8copy(&t1, offset2);
  return stat;
}


int fseek8(FILE *f, off_t offset, int origin)
{
  int i;

#ifdef HAVE_FSEEKO
  i=fseeko(f, offset, origin);
#else
  i=fseek(f, offset, origin);
#endif
  return i; /* 0 - success, -1 - failed */
}

off_t ftell8(FILE *f)
{
#ifdef HAVE_FTELLO
  return ftello(f);
#else
  return (off_t)ftell(f);
#endif
}

char * off_t2hex(off_t offset)
{
  static char p[20];
  short digit, i, len;
  if (offset==0) {
    p[0]='0';
    p[1]=0;
    return p;
  }
  i=0;
  while (offset!=0) {
    digit=offset & 0xF;
    offset>>=4;
    p[i++]=hexdigit[digit];
  }
  len=i;
  for (i=0; i<len/2; i++) {
    p[len]=p[i];
    p[i]=p[len-i-1];
    p[len-i-1]=p[len];
  }
  p[len]=0;
  return p;
}
