/******************************************************************************
*
* Utility for unloading data in .unl format from Informix database
*
* Author: Robert Sosnowski, Robert.Sosnowski@bzwbk.pl
*
* Version 1.70, 17-10-2005
*
* Copyright (c) 2001 - 2007. All Rights Reserved.
*
* Permission granted to use & distribute for free.
*
* This software can be used only at your own risk. Author is not responsible 
* for any damage caused by using this software.
* 
******************************************************************************/

#include "load2cfg.h"
#ifndef HAVE_GETOPT
extern int      optind;
extern char    *optarg;
extern int	opterr;
int getopt(int argc, char *argv[], char *optionS);
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "unload2.h"
#include <string.h>
#include <time.h>

static char *id="@(#) $Id: unload.ec 2521 2007-11-21 08:55:56Z sosnor01 $";

void usage( char * );

#ifdef HAVE_ZLIB
void oncleanupgzu(char *fname, void *handle, const void *logfile,
                  onlogerror_upt onlogerror, int verbose,
                  const char *datahandleoptions);
int oncrlfgzu(void *fd, char *filename, unsigned long int rownum,
              const void *logfile, onlogerror_upt onlogerror, int verbose,
              const char *datahandleoptions);
int oninitgzu(char *fname, const char *tablename, int batchsize, void **handle,
              char *escapech, const void *logfile, int fieldsnum, char *slctstmt2,
              onlogerror_upt onlogerror, int verbose,
              const char *datahandleoptions);
int onwritefieldgz(void *fd, int sqltype, int4 sqlxid, char *filename,
                   const char *src, const unsigned int buflen,
                   char delim, char escapech, short int sqlind,
                   unsigned long int rownum,  int fieldnum,
                   char * fieldname, const void *logfile,
                   onlogerror_upt onlogerror, int verbose,
                   const char *datahandleoptions,
                   void *fieldstate);
#endif

int unloadg(const char * slctstmt2, const char *fname,
            const int batchsize, unsigned long int *unloaded_rows,
            const void *logfile, const int verbosity,
            const char *tablename, const int gzcompr,
            const short fetcharray);

int main( int argc, char **argv )
{

  EXEC SQL BEGIN DECLARE SECTION;
  static char dbname[TBLNAME_SIZE];
  static char server[TBLNAME_SIZE];
  static char server2[TBLNAME_SIZE+1];
  static char user[USERNAME_SIZE];
  static char password[USERNAME_SIZE];
  static char tabname[TBLNAME_SIZE], tabalias[TBLNAME_SIZE];
  static char cmd[300];
  EXEC SQL END DECLARE SECTION;

  static char  qstr[5000];
  static char  sqlfilename[128];
  static char  logfilename[128];
  static char  outfile[128], benchm;
  short verbosity;
  time_t t1, t2;
  int batchsize, argind;
  unsigned long int unloaded_rows;
  int ch;
  FILE * fsqlfile, *logfile;
  int i, l;
  char isolation[3];
  char keycols[500];
  int gzcompr;
  short fetcharray;
  char lockedtables[200], *p, *q;
  int lockmodewait;

  logfilename[0]=0;
  sqlfilename[0]=0;
  batchsize=10000;
  server[0]=0;
  user[0]=0;
  password[0]=0;
  logfile=NULL;
  benchm=0;
  isolation[0]=0;
  verbosity=3; /* show errors and progress messages */
  strcpy(tabalias, "<table>");
  keycols[0]=0;
  gzcompr=-2; /* no gzipping */
  fetcharray=0;
  lockedtables[0]=0;
  lockmodewait=0;

  setvbuf(stdout,NULL,_IONBF,0);
  if ( argc == 1 ) {
    usage( argv[0] );
    exit( -1 );
  }
    
#ifdef HAVE_ZLIB
  while ((ch = getopt( argc, argv, "Vd:s:u:p:e:t:c:n:k:i:v:l:Bg0123456789a:zr:w" )) != EOF) {
#else
  while ((ch = getopt( argc, argv, "Vd:s:u:p:e:t:c:n:k:i:v:l:Ba:zr:w" )) != EOF) {
#endif
    switch (ch) {
      case 'V':
        fprintf( stdout,
         "\nunload ver 1.70, Copyright 2001 - 2007 Robert Sosnowski\n\n" );
        exit( 0 );
        break;
      case 'd':
        /* database name */
        strcpy( dbname, optarg );
        break;
      case 's':
        /* informix server */
        strcpy( server, optarg );
        break;
      case 'u':
        /* user */
        strcpy( user, optarg );
        break;
      case 'p':
        /* password */
        strcpy( password, optarg );
        break;
      case 'e': 
        /* SQL statement */
        strcpy( qstr, optarg );
        break;
      case 't': 
        /* table name */
        strcpy( tabname, optarg );
        strcpy( tabalias, optarg);
        break;
      case 'c': 
        /* file with SQL statement */
        strcpy( sqlfilename, optarg );
        break;
      case 'n':   
        /* batchsize */
        batchsize=(int)atoi( optarg );
        break;
      case 'k':   
        /* key definition */
        strcpy(keycols, optarg);
        break;
      case 'i':
        /* isolation level */
        strncpy( isolation, optarg, 1 );
	isolation[1]=0;
        break;
      case 'l':
        /* log file */
        strcpy( logfilename, optarg );
        break;
      case 'v':
        /* verbosity */
        verbosity=(int)atoi( optarg );
        break;
      case 'B':
        /* Benchmark mode */
	benchm=1;
        break;
      case 'g':
        /* gzipped */
        gzcompr=-1;
        break;
      case '0':
        /* gzipped */
        gzcompr=0;
        break;
      case '1':
        /* gzipped */
        gzcompr=1;
        break;
      case '2':
        /* gzipped */
        gzcompr=2;
        break;
      case '3':
        /* gzipped */
        gzcompr=3;
        break;
      case '4':
        /* gzipped */
        gzcompr=4;
        break;
      case '5':
        /* gzipped */
        gzcompr=5;
        break;
      case '6':
        /* gzipped */
        gzcompr=6;
        break;
      case '7':
        /* gzipped */
        gzcompr=7;
        break;
      case '8':
        /* gzipped */
        gzcompr=8;
        break;
      case '9':
        /* gzipped */
        gzcompr=9;
        break;
      case 'a':
        /* Table alias for progress messages */
        strcpy( tabalias, optarg);
        break;
      case 'z':
        /* Use fetch array */
        fetcharray=1;
        break;
      case 'r':
        /* Lock tables in share mode */
        strcpy(lockedtables,optarg);
        break;
      case 'w':
        /* Set lock mode to wait */
        lockmodewait=1;
        break;
      default:
        usage( argv[0] );
        exit(2);
    }
  }

  if (strlen( dbname ) == 0) {
    fprintf( stdout, "Must supply database (-d)!\n" );
    usage( argv[0] );
    exit(2);
  }
  FetBufSize=30000;
    
  EXEC SQL WHENEVER ERROR CONTINUE;

  if (strlen(server)==0 && (p = strstr(dbname, "@")))
  {
    p[0] = '\0';
    strcpy(server, p+1);
  }

  if (strlen(server)!=0 || strlen(user)!=0 || strlen(password)!=0) {
    if (strlen(server)==0) {
      fprintf( stderr, "Informix server should be specified with -s option\n");
      return 1;
    }
    strcpy(server2,"@");
    strcat(server2, server);
    if ( strlen(user)==0 && strlen(password)==0) {
      EXEC SQL CONNECT TO :server2 AS "conn1" WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting server %s:"INT4_FORMAT"\n", server,
	         sqlca.sqlcode);
	return 1;
      }
    } else {
      if ( strlen(user)==0 || strlen(password)==0) {
    	fprintf( stderr, "Both options: -u and -p should be provided\n");
        return 1;
      }
      EXEC SQL CONNECT TO :server2 AS "conn1" USER :user USING :password
               WITH CONCURRENT TRANSACTION;
      if (sqlca.sqlcode<0) {
        fprintf( stderr, "Error while connecting server %s as user %s:"INT4_FORMAT"\n",
	         server, user, sqlca.sqlcode);
        return 1;
      }
    }
  } else {
    EXEC SQL CONNECT TO DEFAULT WITH CONCURRENT TRANSACTION;
    if (sqlca.sqlcode<0) {
      fprintf( stderr, "Error while connecting default server: "INT4_FORMAT"\n",
               sqlca.sqlcode);
      return 1;
    }
  }
  EXEC SQL DATABASE :dbname;
  if (sqlca.sqlcode < 0) {
    fprintf( stderr, "Cannot open database.  Code: "INT4_FORMAT", ISAM: "INT4_FORMAT".\n",
             sqlca.sqlcode, sqlca.sqlerrd[1] );
    return 1;
  }

  if (strlen( qstr ) != 0 ) {
    if (strlen(tabname)!=0) {
      fprintf(stderr, "You can't specify both -e and -t options!\n");
      return 1;
    }
    if (strlen(sqlfilename)!=0) {
      fprintf(stderr, "You can't specify both -e and -c options!\n");
      return 1;
    }
  } else {
    if (strlen(tabname) != 0) {
      if (strlen(sqlfilename)!=0) {
        fprintf(stderr, "You can't specify both -t and -c options!\n");
        return 1;
      }
      if (keycols[0]) {
        sprintf(qstr, "select * from %s order by %s", tabname, keycols);
      } else {
        sprintf(qstr, "select * from %s", tabname);
      }
    } else {
      if (strlen(sqlfilename)!=0) {
        fsqlfile=fopen(sqlfilename, "r");
        if (fsqlfile) {
          i=fread(qstr, 1, sizeof(qstr), fsqlfile);
          qstr[i]=0;
          fclose(fsqlfile);
        } else {
          perror("Can't open sql file");
          return 1;
        }
      } else {
        fprintf(stderr, "You have to specify one of: -e -t or -c option!\n");
        return 1;
      }
    }
  }

  if (verbosity<0 || verbosity>4) {
    fprintf(stderr, "Wrong verbosity value\n");
    return 1;
  }

  if (gzcompr<-2 || gzcompr>9) {
    fprintf(stderr, "Incorrect compression level\n");
    return 1;
  }

  if (logfilename[0]) {
    logfile=fopen(logfilename,"a");
#ifdef _IONBF
    if (logfile!=NULL) {
      setvbuf(logfile, NULL, _IONBF,0);
    }
#endif
  }

  strcpy(outfile,"-");
  argind=optind;

  i=0;
  do {
    i++;
    if (argind<argc && i==1) {
      strcpy(outfile, argv[argind]);
    }
  } while (++argind <argc);

  if (i>1) {
    fprintf(stderr, "You've specified %d files! Only one is allowed\n", i);
    return 1;
  }
  if (strcmp(outfile,"-")==0) {
    outfile[0]=0;
  }
  if (isolation[0]==0 || isolation[0]=='0') {
    EXEC SQL set isolation to dirty read;
  } else {
  if (isolation[0]=='1') {
    EXEC SQL set isolation to committed read;
  } else {
  if (isolation[0]=='2') {
    EXEC SQL set isolation to cursor stability;
  } else {
  if (isolation[0]=='3') {
    EXEC SQL set isolation to repeatable read;
  } else {
    fprintf(stderr, "Wrong isolation level:%s\n", isolation);
    return 1;
  }}}}
  if (benchm) {
    t1=time(NULL);
    if (logfilename[0]) {
      fprintf(logfile, "Unloading started at %s",ctime(&t1));
    }
    fprintf(stderr, "Unloading started at %s",ctime(&t1));
  }
  if (lockmodewait) {
    EXEC SQL set lock mode to wait;
  }
  if (lockedtables[0]) {
    EXEC SQL begin work;
    p=lockedtables;
    q=p;
    while (p!=NULL) {
      p=strchr(q,',');
      if (p==NULL) {
        sprintf(cmd,"lock table %s in share mode", q);
      } else {
        strcpy(cmd,"lock table ");
        l=strlen(cmd);
        i=p-q;
        while (q!=p) {
          cmd[i-(p-q)+l]=lockedtables[q-lockedtables];
          q++;
        }
        q=p+1;
        cmd[l+i]=0;
        strcpy(cmd+l+i," in share mode");
      }
      EXEC SQL execute immediate :cmd;
      if (sqlca.sqlcode!=0) {
        fprintf(stderr, "ERROR: can't lock table in share mode: "INT4_FORMAT","INT4_FORMAT",%s\n",
                sqlca.sqlcode,sqlca.sqlerrd[1],sqlca.sqlerrm);
      }
    }
  }
  if (unloadg(qstr, outfile, batchsize, &unloaded_rows, logfile, verbosity,
              tabalias, gzcompr, fetcharray)!=0) {
    return 1;
  }
  if (lockedtables[0]) {
    EXEC SQL commit work;
  }

  if (benchm) {
    t2=time(NULL);
    i=t2-t1;
    if (logfilename[0]) {
      fprintf(logfile, "Unloading finished at %s", ctime(&t2));
      fprintf(logfile, "Elapsed seconds (unloading): %d\n",i);
    }
    fprintf(stderr, "Unloading finished at %s", ctime(&t2));
    fprintf(stderr, "Elapsed seconds (unloading): %d\n",i);
  }
  if (logfilename[0]) {
    fclose(logfile);
  }


  return 0;
}


void usage( char *prog )
{


  fprintf( stdout, "\n");
  fprintf( stdout, "%s - unload data from Informix database.\n\n", prog );

  fprintf( stdout, "Usage: \n" );
#ifdef HAVE_ZLIB
  fprintf( stdout, 
           "\t%s [-V] -d <db_name> [-s <server> -u <user> -p <password>]\n "
           "(-e <select stmt>|-t <table>|-c <sql file>) [-n <batch size>]\n "
           "[-k <key columns>] [-i <level>] [-l <logfile>] [-B] [-g|1|..|9] [-v <verbosity>]\n "
           "[-a <table alias>] [-r <tables>] [-w] [-z] <unload file>\n\n", prog);
#else
  fprintf( stdout, 
           "\t%s [-V] -d <db_name> [-s <server> -u <user> -p <password>]\n "
           "(-e <select stmt>|-t <table>|-c <sql file>) [-n <batch size>]\n "
           "[-k <key columns>] [-i <level>] [-l <logfile>] [-B] [-v <verbosity>]\n "
           "[-a <table alias>] [-r <tables>] [-w] <unload file>\n\n", prog);
#endif
  fprintf( stdout, "\t\t-V - version information\n" ); 
  fprintf( stdout, "\t\t-d - database name\n" );
  fprintf( stdout, "\t\t-s - informix server name\n" );
  fprintf( stdout, "\t\t-u - user\n" );
  fprintf( stdout, "\t\t-p - password\n" );
  fprintf( stdout, "\t\t-e - SQL select statement in quotes\n" );
  fprintf( stdout, "\t\t-t - table name\n" );
  fprintf( stdout, "\t\t-c - file containing SQL select statement\n" );
  fprintf( stdout, "\t\t-n - batch size\n" );
  fprintf( stdout, "\t\t-k - sort by specified columns for -t option\n" );
  fprintf( stdout, "\t\t-i - isolation level\n" );
  fprintf( stdout, "\t\t   0 - dirty read (default)\n" );
  fprintf( stdout, "\t\t   1 - commited read\n" );
  fprintf( stdout, "\t\t   2 - cursor stability\n" );
  fprintf( stdout, "\t\t   3 - repeatable read\n" );
  fprintf( stdout, "\t\t-l - log file name\n" );
  fprintf( stdout, "\t\t-B - benchmark\n" );
#ifdef HAVE_ZLIB
  fprintf( stdout, "\t\t-g - gzipped unload\n" );
  fprintf( stdout, "\t\t-1 - fastest gzipping\n" );
  fprintf( stdout, "\t\t-9 - best gzipping\n" );
#endif
  fprintf( stdout, "\t\t-v - verbosity:\n" );
  fprintf( stdout, "\t\t     0 - completely quiet\n" );
  fprintf( stdout, "\t\t     1 - show errors only\n" );
  fprintf( stdout, "\t\t     2 - summary messages\n" );
  fprintf( stdout, "\t\t     3 - detailed messages (default)\n" );
  fprintf( stdout, "\t\t     4 - debug info\n" );
  fprintf( stdout, "\t\t-a - table alias for progress messages\n" );
  fprintf( stdout, "\t\t-r <tables> - lock tables (,-separated) in share mode\n" );
  fprintf( stdout, "\t\t-w - set lock mode to wait\n" );
  fprintf( stdout, "\t\t-z - use fetch array\n" );
}

int unloadg(const char * slctstmt2, const char *fname,
            const int batchsize, unsigned long int *unloaded_rows,
            const void *logfile, const int verbosity,
            const char *tablename, const int gzcompr,
            const short fetcharray)
{
  unl_handle_t *unl_handle;
  int i, k;
  char buffer[32767+1];
  int batchsize2;
  int fieldsnum;
  int all_records;
  unsigned long int bytesread;
  int sqlind;
  char delim;
  char s[5];

  delim='|';
  sprintf(s, "%d", gzcompr);

  unl_handle=unl_create();
  if (unl_handle==NULL) {
    return 1;
  }
  k=unl_setstmt(unl_handle, slctstmt2);
  if (k!=0) {
    return 1;
  }

  k=unl_setdatahandlename(unl_handle, fname);
  if (k!=0) {
    return 1;
  }
  if (gzcompr>=0) {
    k=unl_setdatahandleoptions(unl_handle, s);
    if (k!=0) {
      return 1;
    }
  }

  unl_setdelimiter(unl_handle, delim);

  if (batchsize<0) {
    batchsize2=0;
  } else {
    batchsize2=batchsize;
  }
  unl_setbatchsize(unl_handle, batchsize2);

  if (logfile==NULL) {
    unl_setloghandle(unl_handle, NULL);
  } else {
    unl_setloghandle(unl_handle, logfile);
  }
  switch (verbosity) { 
    case 0: 
            unl_setverbosity(unl_handle, 0);
            break;
    case 1: 
            unl_setverbosity(unl_handle, 20);
            break;
    case 2: 
            unl_setverbosity(unl_handle, 30);
            break;
    case 3: 
            unl_setverbosity(unl_handle, 40);
            break;
    case 4: 
            unl_setverbosity(unl_handle, 50);
            break;
    default: 
            unl_setverbosity(unl_handle, 30);
            break;
  }
  unl_settablename(unl_handle, tablename);
  unl_setfetcharray(unl_handle, fetcharray);

#ifdef HAVE_ZLIB
  if (gzcompr!=-2) {
    unl_setoninit(unl_handle, oninitgzu);
    unl_setonwritefield(unl_handle, onwritefieldgz);
    unl_setoncleanup(unl_handle, oncleanupgzu);
    unl_setoncrlf(unl_handle, oncrlfgzu);
  }
#endif
  *unloaded_rows=0;


  k=unl_prepare(unl_handle);
  if (k!=0) {
    return 1;
  }
  fieldsnum=unl_getfieldsnum(unl_handle);
  k=unl_open(unl_handle);
  if (k!=0) {
    return 1;
  }
  while (1) {
    k=unl_getrecord(unl_handle, &all_records);
    if (k!=0) {
      return 1;
    }

    if (all_records) {
      break;
    }
    if (unl_beforerow(unl_handle)) {
      return 1;
    }
    i=0; /* current field number */

    while (i<fieldsnum) {
      k=unl_getfield(unl_handle, i, buffer, sizeof(buffer), &bytesread, &sqlind);
      if (k!=0) {
        return 1;
      }
      k=unl_writefield(unl_handle, i, buffer, bytesread, sqlind,
                       unl_handle->fieldstates[i]);
      if (k!=0) {
        return 1;
      }
      i++;
    }

    if (i>=fieldsnum) {
      if (unl_afterrow(unl_handle)) {
        return 1;
      }
    }
  }
  if (unl_close(unl_handle)) {
    return 1;
  }
  *unloaded_rows=unl_getunloadedrows(unl_handle);
  if (unl_free(unl_handle)) { /* = loader_unprepare */
    return 1;
  }
  unl_handle=NULL;
  return 0; 
}

