/*****************************************************************************
*
* Standard event handlers for unloading functions dedicated for xml format
*
* Author: Robert Sosnowski, Robert.Sosnowski@bzwbk.pl
*
* Version 1.70, 17-10-2005
*
* Copyright (c) 2001 - 2007. All Rights Reserved.
*
* Permission granted to use & distribute for free. Changes in source code
* require Author's acceptance.
*
* This software can be used only at your own risk. Author is not responsible 
* for any damage caused by using this software.
* 
******************************************************************************/

#include "load2cfg.h"

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif


#include <stdio.h>
#include "unload2.h"
#include "tempfile.h"
#include <stdlib.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

#include <ctype.h>

#ifdef HAVE_ZLIB
#include <zlib.h>
#endif

#ifdef HAVE_ICONV_H
#include <iconv.h>
#endif


EXEC SQL include sqltypes;

static char *id="@(#) $Id: unload2gzx.ec 2521 2007-11-21 08:55:56Z sosnor01 $";

int onwritefield_xml1gz(void *fd, int sqltype, int4 sqlxid, char *filename,
                        char *src, const unsigned int buflen,
                        char delim, char escapech, short int sqlind,
                        unsigned long int rownum,  int fieldnum,
                        char *fieldname, const void *logfile,
                        onlogerror_upt onlogerror, int verbose,
                        const char *datahandleoptions,
                        void *fieldstate);
int oncrlf_xml1gz(void *fd, char *filename, unsigned long int rownum,
                  const void *logfile,
                  onlogerror_upt onlogerror, int verbose,
                  const char *datahandleoptions);

int onrecordbegin_xml1gz(void *fd, char *filename, unsigned long int rownum,
                         const void *logfile,
                         onlogerror_upt onlogerror, int verbose,
                         const char *datahandleoptions);
int oninit_xml1gz(char *fname, const char *tablename, int batchsize,
                  void **handle, char *escapech, const void *logfile,
                  int fieldsnum, char *slctstmt2,
                  onlogerror_upt onlogerror, int verbose,
                  const char *datahandleoptions);
int onfinish_xml1gz(char *fname, char *tablename, void *handle,
                    unsigned long int rownum,
                    const void *logfile,
                    onlogerror_upt onlogerror, int verbose);

int oninit_xml2gz(char *fname, const char *tablename, int batchsize,
                  void **handle, char *escapech, const void *logfile,
                  int fieldsnum, char *slctstmt2,
                  onlogerror_upt onlogerror, int verbose,
                  const char *datahandleoptions);

int oninit_html1gz(char *fname, const char *tablename, int batchsize,
                   void **handle, char *escapech, const void *logfile,
                   int fieldsnum, char *slctstmt2,
                   onlogerror_upt onlogerror, int verbose,
                   const char *datahandleoptions);

int onwritefield_html1gz(void *fd, int sqltype, int4 sqlxid, char *filename,
                         char *src, const unsigned int buflen,
                         char delim, char escapech, short int sqlind,
                         unsigned long int rownum,  int fieldnum,
                         char * fieldname, const void *logfile,
                         onlogerror_upt onlogerror, int verbose,
                         const char *datahandleoptions,
                         void *fieldstate);

int oncrlf_html1gz(void *fd, char *filename, unsigned long int rownum,
                   const void *logfile,
                   onlogerror_upt onlogerror, int verbose,
                   const char *datahandleoptions);

int onrecordbegin_html1gz(void *fd, char *filename, unsigned long int rownum,
                          const void *logfile,
                          onlogerror_upt onlogerror, int verbose,
                          const char *datahandleoptions);

int onfinish_html1gz(char *fname, char *tablename, void *handle,
                     unsigned long int rownum,
                     const void *logfile,
                     onlogerror_upt onlogerror, int verbose);

int oninit_xml3gz(char *fname, const char *tablename, int batchsize,
                  void **handle, char *escapech, const void *logfile,
                  int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                  int verbose, const char *datahandleoptions);

void encodeblock( unsigned char in[3], unsigned char out[4], int len );
void encodeblock3( unsigned char in[3], unsigned char out[4]);
void entityencode(char *srcbuf, char *dstbuf, int *p1, int *size1, int len1, int batchsize);


static char rowset_token[20], row_token[20];
static void set_rowset_token(char *s)
{
  strcpy(rowset_token, s);
}

static void set_row_token(char *s)
{
  strcpy(row_token, s);
}

int onwritebyteblobgz(gzFile fd1, char *filename, loc_t * src,
                      const unsigned int buflen,
                      char delim, unsigned long int rownum,
                      int fieldnum, char *fieldname, const void *logfile,
                      onlogerror_upt onlogerror, int verbose);

int onwritetextblobgzx(gzFile fd1, char *filename, loc_t * src,
                       const unsigned int buflen,
                       char delim, char escapech, unsigned long int rownum,
                       int fieldnum, char *fieldname, const void *logfile,
                       onlogerror_upt onlogerror, int verbose)
{ char * p;
  char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int j, k, len;
  FILE *fd2;
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  char ch;
  int sqlind;
  unsigned int maxblob;
  char *envp;

  maxblob=MAXMEMBLOB;
#if defined(HAVE_GETENV)
  envp=getenv("DBBLOBBUF");
  if (envp!=NULL && isdigit(envp[0])) {
    k=atol(envp);
    if (k*1024<MAXMEMBLOB) {
      maxblob=k*1024;
    }
  }
#endif
  p=src->loc_buffer;
  sqlind=src->loc_indicator;
  if (sqlind!=-1) {
    if (p[0]=='\0' && src->loc_position==1) { /* if we have an empty string */
      return 0;
    } else {
      if (src->loc_position==1) {
        len = src->loc_size;
        j = 0;
        while ( j < len) {
          k=0;
          while (k<maxblob-2-5 && j<len) {
            if (p[j] == '<' ) {
              buf[k]='&';
              k++;
              buf[k]='l';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (p[j] == '>' ) {
              buf[k]='&';
              k++;
              buf[k]='g';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (p[j] == '&' ) {
              buf[k]='&';
              k++;
              buf[k]='a';
              k++;
              buf[k]='m';
              k++;
              buf[k]='p';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (p[j] == '\"' ) {
              buf[k]='&';
              k++;
              buf[k]='q';
              k++;
              buf[k]='u';
              k++;
              buf[k]='o';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (p[j] == '\'' ) {
              buf[k]='&';
              k++;
              buf[k]='a';
              k++;
              buf[k]='p';
              k++;
              buf[k]='o';
              k++;
              buf[k]='s';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            buf[k]=p[j];
            j++;
            k++;
          }
          buf[k]='\0';
          stat=gzwrite(fd1, buf, k);
          if (stat==0 && k!=0) {
            sprintf(errstr, "gzOS error %d",errno);
            onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                       errno, logfile, verbose, 10);
            return 1;
          }
        }
      } else {
        fd2=fopen(p,"rb");
        if ( fd2 == NULL ) {
          sprintf(errstr,"Cannot open file %s:%d", p, errno);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
                     logfile, verbose, 10);
          return 1;
        }
        while (!feof(fd2)) {
          k=0;
          while (k<maxblob-2-5) {
            ch=fgetc(fd2);
            if ( ferror(fd2) ) {
              sprintf(errstr,"Cannot read temp file %s:%d", p, errno);
              onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
                         logfile, verbose, 10);
              return 1;
            }
            if (feof(fd2)) {
              break;
            }
            if (ch == '<' ) {
              buf[k]='&';
              k++;
              buf[k]='l';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (ch == '>' ) {
              buf[k]='&';
              k++;
              buf[k]='g';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (ch == '&' ) {
              buf[k]='&';
              k++;
              buf[k]='a';
              k++;
              buf[k]='m';
              k++;
              buf[k]='p';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (ch == '\"' ) {
              buf[k]='&';
              k++;
              buf[k]='q';
              k++;
              buf[k]='u';
              k++;
              buf[k]='o';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (ch == '\'' ) {
              buf[k]='&';
              k++;
              buf[k]='a';
              k++;
              buf[k]='p';
              k++;
              buf[k]='o';
              k++;
              buf[k]='s';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            buf[k]=ch;
            k++;
          }
          buf[k]='\0';
          stat=gzwrite(fd1, buf, k);
          if (stat==0 && k!=0) {
            sprintf(errstr, "gzOS error %d",errno);
            onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                       errno, logfile, verbose, 10);
            return 1;
          }
        }
        fclose(fd2);
      }
    }
  }
  if (src->loc_buffer!=NULL) {
    if (src->loc_position==2) {
      unlink(p);
    }
    free(src->loc_buffer);
    src->loc_buffer=NULL;
  }
  return 0;
}


int onwritebyteblobgz64(gzFile fd1, char *filename, loc_t * src,
                        char delim, unsigned long int rownum,
                        int fieldnum, char *fieldname, const void *logfile,
                        onlogerror_upt onlogerror, int verbose)
{ char * p;
  unsigned char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int i, j, k, len;
  FILE *fd2;
  char errstr[ERR_BUF_SIZE];
  int j2;
  int sqlind;
  unsigned int maxblob;
  char *envp;

  maxblob=MAXMEMBLOB;
#if defined(HAVE_GETENV)
  envp=getenv("DBBLOBBUF");
  if (envp!=NULL && isdigit(envp[0])) {
    k=atol(envp);
    if (k*1024<MAXMEMBLOB) {
      maxblob=k*1024;
    }
  }
#endif

  p=src->loc_buffer;
  sqlind=src->loc_indicator;
  if (sqlind!=-1) {
    if (src->loc_position==1) {
      len = src->loc_size;
      j=0;
      while (j<len) {
        i=0;
        while (i<=maxblob-4 && j<len) {
          if (len-j>2) {
            encodeblock3((unsigned char *)p+j, buf+i);
          } else {
            encodeblock((unsigned char *)p+j, buf+i, len-j);
          }
          j+=3;
          i+=4;
        }
        
        k=gzwrite(fd1, buf, i);
        if (k<i) {
          sprintf(errstr, "gzOS error while writing out file: %d",errno);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                     errno, logfile, verbose, 10);
          return 1;
        }
      }
    } else {
      fd2=fopen(p,"rb");
      if ( fd2 == NULL ) {
        sprintf(errstr,"Cannot open file %s:%d", p, errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                   verbose, 10);
        return 1;
      }
      while (1) {
        i=fread(buf, 1, 3*((maxblob-1)/4), fd2);
        if (ferror(fd2)) {
          sprintf(errstr, "OS error while reading temp file: %d:%d",errno, ferror(fd2));
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                     errno, logfile, verbose, 10);
          return 1;
        }
        j2=(i+2)/3; /* number of triples */
        if (i%3) {
          encodeblock(buf+(j2-1)*3, buf+(j2-1)*4, i%3);
          j2--;
        }
        j2--; /* number of remaining triples - 1 */
        while (j2>=0) {
          encodeblock3(buf+j2*3,buf+j2*4);
          j2--;
        }
        j=gzwrite(fd1, buf, 4*((i+2)/3));
        if (errno) {
          sprintf(errstr, "gzOS error while writing out file:%d",errno);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                     errno, logfile, verbose, 10);
          return 1;
        }
        if (feof(fd2)) {
          break;
        }
      }
      fclose(fd2);
    }
  }
  if (src->loc_buffer!=NULL && sqlind!=-1) {
/* additional check for sqlind!=-1 because it appeared
   that in many cases loc_buffer was not null altough memory pointed to
   was never allocated; therefore free caused memory leakage;
   fortunately all such cases occurs only for null BLOB */

    if (src->loc_position==2) {
      unlink(p);
    }
    free(src->loc_buffer);
    src->loc_buffer=NULL;
  }
  return 0;
}


#define BATCHSIZE 5000
int onwritesmartclobgzx(gzFile fd1, char *filename, int4 sqlxid, ifx_lo_t * src,
                        char delim, unsigned long int rownum, int fieldnum,
                        char *fieldname, const void *logfile,
                        onlogerror_upt onlogerror, int verbose)
{
  char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  char buf2[BATCHSIZE+1];
  char errstr[ERR_BUF_SIZE];
  mint lofd;
  mint error;
  ifx_lo_stat_t *stats;
  ifx_int8_t fsize64;
  mint n;
  int i, j;

  lofd=ifx_lo_open((ifx_lo_t *)src, LO_RDONLY, &error);
  if (lofd<0) {
    sprintf(errstr, "ifx_lo_open: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat(lofd, &stats);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat_size(stats,&fsize64);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat_size: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  i=0;
  while ((n=ifx_lo_read(lofd, buf, MAXMEMBLOB, &error))>0) {
    while (i<n) {
      entityencode(buf, buf2, &i, &j, n, BATCHSIZE);
      gzwrite(fd1, buf, j);
      if (errno) {
        sprintf(errstr, "gzOS error while writing out file:%d",errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                   errno, logfile, verbose, 10);
        return 1;
      }
    }
  }
  if (n<0) {
    sprintf(errstr, "ifx_lo_read: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  ifx_lo_close(lofd);
  ifx_lo_stat_free(stats);

  return 0;
}


int onwritesmartblobgzx(gzFile fd1, char *filename, int4 sqlxid, ifx_lo_t * src,
                      char delim, unsigned long int rownum, int fieldnum,
                      char *fieldname, const void *logfile,
                      onlogerror_upt onlogerror, int verbose)
{
  char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  char errstr[ERR_BUF_SIZE];
  mint lofd;
  mint error;
  ifx_lo_stat_t *stats;
  ifx_int8_t fsize64;
  mint n, j2;

  lofd=ifx_lo_open((ifx_lo_t *)src, LO_RDONLY, &error);
  if (lofd<0) {
    sprintf(errstr, "ifx_lo_open: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat(lofd, &stats);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat_size(stats,&fsize64);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat_size: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  while ((n=ifx_lo_read(lofd, buf, ((MAXMEMBLOB-1)/4)*3, &error))>0) {
    j2=(n+2)/3; /* number of triples */
    if (n%3) {
      encodeblock((unsigned char *)buf+(j2-1)*3, (unsigned char *)buf+(j2-1)*4, n%3);
      j2--;
    }
    j2--; /* number of remaining triples - 1 */
    while (j2>=0) {
      encodeblock3((unsigned char *)buf+j2*3,(unsigned char *)buf+j2*4);
      j2--;
    }
    gzwrite(fd1, buf, 4*((n+2)/3));
    if (errno) {
      sprintf(errstr, "gzOS error while writing out file:%d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                 errno, logfile, verbose, 10);
      return 1;
    }
  }
  if (n<0) {
    sprintf(errstr, "ifx_lo_read: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  ifx_lo_close(lofd);
  ifx_lo_stat_free(stats);

  return 0;
}

int onwritefield_xml1gz(void *fd, int sqltype, int4 sqlxid, char *filename,
                        char *src, const unsigned int buflen,
                        char delim, char escapech, short int sqlind,
                        unsigned long int rownum,  int fieldnum,
                        char * fieldname, const void *logfile,
                        onlogerror_upt onlogerror, int verbose,
                        const char *datahandleoptions,
                        void *fieldstate)
/* returns:  0 if success
 *           1 if error;
 * src : pointer to the buffer;
 */
{
  char buf[BUF1_SIZE+1];  /* size enough for max size SQL datatype (non-BLOB)
                             in Informix it is CHAR(32767) */
  unsigned int j, k, len;
  FILE *fd1;
  gzFile fd2;
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  char s[100];

  fd1=(FILE *)fd;
  fd2=(gzFile)fd;

  if (sqlind == -1) {
    return 0; /* NULL is not written at all */
  }
  if (sqltype==SQLTEXT || sqltype==SQLBYTES ) {
    sprintf(s, "<%s>",fieldname);
    stat=gzputs(fd2, s);
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    if (sqltype==SQLTEXT) {
      stat=onwritetextblobgzx(fd2, filename, (loc_t *) src, buflen, delim,
                              escapech, rownum, fieldnum, fieldname, logfile,
                              onlogerror, verbose);
    } else {
      stat=onwritebyteblobgz(fd2, filename, (loc_t *) src, buflen, delim,
                             rownum, fieldnum, fieldname, logfile,
                             onlogerror, verbose);
    }
    if (stat!=0) {
      return 1;
    }
    sprintf(s, "</%s>\n",fieldname);
    stat=gzputs(fd2, s);
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    return 0;
  }
 
  if (sqltype==CFIXBINTYPE && (sqlxid==XID_BLOB || sqlxid==XID_CLOB) ) {
    sprintf(s, "<%s>",fieldname);
    stat=gzputs(fd2, s);
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    if (sqlxid==XID_BLOB) {
      stat=onwritesmartblobgzx(fd2, filename, sqlxid, (ifx_lo_t *) src,
                             escapech, rownum, fieldnum, fieldname,
                             logfile, onlogerror, verbose);
    } else {
      stat=onwritesmartclobgzx(fd2, filename, sqlxid, (ifx_lo_t *) src,
                             escapech, rownum, fieldnum, fieldname,
                             logfile, onlogerror, verbose);
    }
    if (stat!=0) {
      return 1;
    }
    sprintf(s, "</%s>\n",fieldname);
    stat=gzputs(fd2,s);
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    return 0;
  }

  sprintf(s, "<%s>",fieldname);
  stat=gzputs(fd2, s);
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }

  len = stleng(src);
  j = 0;
  while ( j < len) {
    k=0;
    while (k<BUF1_SIZE-5 && j<len) {
      if (src[j] == '<' ) {
        buf[k]='&';
        k++;
        buf[k]='l';
        k++;
        buf[k]='t';
        k++;
        buf[k]=';';
        k++;
        j++;
        continue;
      }
      if (src[j] == '>' ) {
        buf[k]='&';
        k++;
        buf[k]='g';
        k++;
        buf[k]='t';
        k++;
        buf[k]=';';
        k++;
        j++;
        continue;
      }
      if (src[j] == '&' ) {
        buf[k]='&';
        k++;
        buf[k]='a';
        k++;
        buf[k]='m';
        k++;
        buf[k]='p';
        k++;
        buf[k]=';';
        k++;
        j++;
        continue;
      }
      if (src[j] == '\"' ) {
        buf[k]='&';
        k++;
        buf[k]='q';
        k++;
        buf[k]='u';
        k++;
        buf[k]='o';
        k++;
        buf[k]='t';
        k++;
        buf[k]=';';
        k++;
        j++;
        continue;
      }
      if (src[j] == '\'' ) {
        buf[k]='&';
        k++;
        buf[k]='a';
        k++;
        buf[k]='p';
        k++;
        buf[k]='o';
        k++;
        buf[k]='s';
        k++;
        buf[k]=';';
        k++;
        j++;
        continue;
      }
      buf[k]=src[j];
      j++;
      k++;
    }
    buf[k]='\0';
    stat=gzputs(fd2, buf);
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
  }
  sprintf(s, "</%s>\n",fieldname);
  stat=gzputs(fd2, s);
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }

  return 0;
}

int oncrlf_xml1gz(void *fd, char *filename, unsigned long int rownum,
                  const void *logfile,
                  onlogerror_upt onlogerror, int verbose,
                  const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  gzFile fd1;
  unsigned int stat;

  fd1=(gzFile)fd;
  stat=gzputs(fd1, "</");
  stat=gzputs(fd1, row_token);
  stat=gzputs(fd1, ">\n");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int onrecordbegin_xml1gz(void *fd, char *filename, unsigned long int rownum,
                         const void *logfile,
                         onlogerror_upt onlogerror, int verbose,
                         const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  gzFile fd1;
  unsigned int stat;
  char s[100];

  fd1=(gzFile)fd;
  sprintf(s,"<%s num=\"%ld\">\n", row_token, rownum);
  stat=gzputs(fd1, s);
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oninit_xml1gz(char *fname, const char *tablename, int batchsize,
                  void **handle, char *escapech,
                  const void *logfile, int fieldsnum, char *slctstmt2,
                  onlogerror_upt onlogerror, int verbose,
                  const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  char options[10];

  options[0]=0;
  strcpy(options,"wb");
  if (datahandleoptions && datahandleoptions[0]>='0'
      && datahandleoptions[0]<='9') {
    options[2]=datahandleoptions[0];
    options[3]=0;
  }

  set_rowset_token("ROWSET");
  set_row_token("ROW");

  if (fname[0]==0 || fname==NULL) {
    (*handle)=gzdopen(1,options);
    if (strcmp("<table>",tablename)) {
      sprintf(errstr,"file=<stdout>, tablename=%s", tablename);
    } else {
      sprintf(errstr,"file=<stdout>\n");
    }
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  } else {
    if (strcmp("<table>",tablename)) {
      sprintf(errstr, "file=%s, tablename=%s\n",fname, tablename);
    } else {
      sprintf(errstr, "file=%s\n",fname);
    }
    (*handle)=gzopen(fname,options);
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  }
  if ( (*handle) == NULL ) {
    sprintf(errstr,"Cannot open file %s:%d", fname, errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
               verbose, 10);
    return 1;
  }
  (*escapech)='\\';
  stat=gzputs((*handle),"<?xml version=\"1.0\" ?>\n");
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror((FILE *)(*handle)));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  stat=gzputs((*handle), "<");
  stat=gzputs((*handle), rowset_token);
  stat=gzputs((*handle), "\n");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}



int onfinish_xml1gz(char *fname, char *tablename, void *handle,
                    unsigned long int rownum,
                    const void *logfile,
                    onlogerror_upt onlogerror, int verbose)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  
  sprintf(errstr, "%ld records were unloaded", rownum);
  onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
             0, logfile, verbose, 30);
  stat=gzputs(handle, "</");
  stat=gzputs(handle, rowset_token);
  stat=gzputs(handle, ">\n");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oninit_xml2gz(char *fname, const char *tablename, int batchsize,
                  void **handle, char *escapech, const void *logfile,
                  int fieldsnum, char *slctstmt2,
                  onlogerror_upt onlogerror, int verbose,
                  const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  gzFile f;
  exec sql begin declare section;
  char *slctstmt;
  int fields_number;
  int nullable;
  int i;
  char sqlname[200];
  exec sql end declare section;
  int len;
  char *cp;
  char options[10];

  options[0]=0;
  strcpy(options,"wb");
  if (datahandleoptions && datahandleoptions[0]>='0'
      && datahandleoptions[0]<='9') {
    options[2]=datahandleoptions[0];
    options[3]=0;
  }
 
  set_rowset_token("ROWSET");
  set_row_token("ROW");

  slctstmt=slctstmt2;
  fields_number=fieldsnum;

  EXEC SQL allocate descriptor 'desc1' WITH MAX :fields_number;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr, "Cannot allocate descriptor "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    return 1;
  }
  EXEC SQL prepare usqlobj from :slctstmt;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr,"Cannot prepare: "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    EXEC SQL deallocate descriptor 'desc1';
    return 1;
  }
				    
  EXEC SQL describe usqlobj using sql descriptor 'desc1';
  if (fname[0]==0 || fname==NULL) {
    (*handle)=gzdopen(1,options);
    if (strcmp("<table>",tablename)) {
      sprintf(errstr,"file=<stdout>, tablename=%s", tablename);
    } else {
      sprintf(errstr,"file=<stdout>\n");
    }
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  } else {
    if (strcmp("<table>",tablename)) {
      sprintf(errstr, "file=%s, tablename=%s\n",fname, tablename);
    } else {
      sprintf(errstr, "file=%s\n",fname);
    }
    (*handle)=gzopen(fname,options);
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  }
  if ( (*handle) == NULL ) {
    sprintf(errstr,"Cannot open file %s:%d", fname, errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
               verbose, 10);
    return 1;
  }
  f=(gzFile)(*handle);
  (*escapech)='\\';
  stat=gzputs(f, "<?xml version=\"1.0\" ?>\n");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  gzprintf(f, "<!DOCTYPE %s [\n", rowset_token);
  gzprintf(f, "<!ELEMENT %s (%s)*>\n", rowset_token, row_token);
  gzprintf(f, "<!ELEMENT %s (", row_token);

  i=1;
  EXEC SQL get descriptor 'desc1' VALUE :i
           :nullable = NULLABLE, :sqlname=NAME ;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr, "Cannot get descriptor1 "INT4_FORMAT":%d",sqlca.sqlcode, i);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    EXEC SQL free usqlobj;
    EXEC SQL deallocate descriptor 'desc1';
    return 1;
  }
  len=stleng(sqlname);
  cp=sqlname+len-1;
  while(len>1 && *cp==' ') {
    *cp='\0';
    len--, cp--;
  }
  gzprintf(f, "%s", sqlname);
  if (nullable==1) { /* allow NULL values */
    gzputc(f, '?');
  }
  for (i=2; i<=fields_number; i++) {
    EXEC SQL get descriptor 'desc1' VALUE :i
             :nullable = NULLABLE, :sqlname=NAME ;
    if ( sqlca.sqlcode != 0 ) {
      sprintf(errstr, "Cannot get descriptor2 "INT4_FORMAT":%d",sqlca.sqlcode, i);
      onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
      EXEC SQL free usqlobj;
      EXEC SQL deallocate descriptor 'desc1';
      return 1;
    }
    len=stleng(sqlname);
    cp=sqlname+len-1;
    while(len>1 && *cp==' ') {
      *cp='\0';
      len--, cp--;
    }
    gzprintf(f, ", %s", sqlname);
    if (nullable==1) { /* allow NULL values */
      gzputc(f, '?');
    }
  }
  gzprintf(f, ")>\n");
  gzprintf(f, "<!ATTLIST %s num CDATA #REQUIRED>\n", row_token);

  for (i=1; i<=fields_number; i++) {
    EXEC SQL get descriptor 'desc1' VALUE :i
             :nullable = NULLABLE, :sqlname=NAME ;
    if ( sqlca.sqlcode != 0 ) {
      sprintf(errstr, "Cannot get descriptor2 "INT4_FORMAT":%d",sqlca.sqlcode, i);
      onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
      EXEC SQL free usqlobj;
      EXEC SQL deallocate descriptor 'desc1';
      return 1;
    }
    len=stleng(sqlname);
    cp=sqlname+len-1;
    while(len>1 && *cp==' ') {
      *cp='\0';
      len--, cp--;
    }
    gzprintf(f, "<!ELEMENT %s (#PCDATA)>\n", sqlname);
  }
  EXEC SQL free usqlobj;
  EXEC SQL deallocate descriptor 'desc1';

  gzprintf(f, "]>\n");
  stat=gzputs(f, "<");
  stat=gzputs(f, rowset_token);
  stat=gzputs(f, ">\n");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oninit_html1gz(char *fname, const char *tablename, int batchsize,
                   void **handle, char *escapech, const void *logfile,
                   int fieldsnum, char *slctstmt2,
                   onlogerror_upt onlogerror, int verbose,
                   const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  gzFile f;
  exec sql begin declare section;
  char *slctstmt;
  int fields_number;
  int nullable;
  int i;
  char sqlname[200];
  exec sql end declare section;
  int len;
  char *cp;
  char options[10];

  options[0]=0;
  strcpy(options,"wb");
  if (datahandleoptions && datahandleoptions[0]>='0'
      && datahandleoptions[0]<='9') {
    options[2]=datahandleoptions[0];
    options[3]=0;
  }

  set_rowset_token("ROWSET");
  set_row_token("ROW");

  slctstmt=slctstmt2;
  fields_number=fieldsnum;

  EXEC SQL allocate descriptor 'desc1' WITH MAX :fields_number;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr, "Cannot allocate descriptor "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    return 1;
  }
  EXEC SQL prepare usqlobj2 from :slctstmt;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr,"Cannot prepare: "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    EXEC SQL deallocate descriptor 'desc1';
    return 1;
  }
				    
  EXEC SQL describe usqlobj2 using sql descriptor 'desc1';
  if (fname[0]==0 || fname==NULL) {
    (*handle)=gzdopen(1,options);
    if (strcmp("<table>",tablename)) {
      sprintf(errstr,"file=<stdout>, tablename=%s", tablename);
    } else {
      sprintf(errstr,"file=<stdout>\n");
    }
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  } else {
    if (strcmp("<table>",tablename)) {
      sprintf(errstr, "file=%s, tablename=%s\n",fname, tablename);
    } else {
      sprintf(errstr, "file=%s\n",fname);
    }
    (*handle)=gzopen(fname,options);
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  }
  if ( (*handle) == NULL ) {
    sprintf(errstr,"Cannot open file %s:%d", fname, errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
               verbose, 10);
    return 1;
  }
  f=(gzFile)(*handle);
  (*escapech)='\\';
  stat=gzputs(f, "<table><tr>");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }

  /* table headings */
  for (i = 1; i <= fields_number; i++) {
    EXEC SQL get descriptor 'desc1' VALUE :i
             :nullable = NULLABLE, :sqlname=NAME ;
    if ( sqlca.sqlcode != 0 ) {
      sprintf(errstr, "Cannot get descriptor1 "INT4_FORMAT":%d",sqlca.sqlcode, i);
      onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
      EXEC SQL free usqlobj2;
      EXEC SQL deallocate descriptor 'desc1';
      return 1;
    }
    len=stleng(sqlname);
    cp=sqlname+len-1;
    while(len>1 && *cp==' ') {
      *cp='\0';
      len--, cp--;
    }
        
    gzprintf(f, "<th>%s</th>", sqlname);
  }    
  stat=gzputs(f, "</tr>\n");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }


  EXEC SQL free usqlobj2;
  EXEC SQL deallocate descriptor 'desc1';

  return 0;
}


int onwritebyteblobgzx2(gzFile fd1, char *filename, loc_t * src,
                      char delim, unsigned long int rownum,
                      int fieldnum, char *fieldname, const void *logfile,
                      onlogerror_upt onlogerror, int verbose)
{ char * p;
  unsigned char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int i, j, k, len;
  FILE *fd2;
  gzFile fd3;
  char errstr[ERR_BUF_SIZE];
  int sqlind;
  unsigned int maxblob;
  char *envp;
  char *p1;

  maxblob=MAXMEMBLOB;
#if defined(HAVE_GETENV)
  envp=getenv("DBBLOBBUF");
  if (envp!=NULL && isdigit(envp[0])) {
    k=atol(envp);
    if (k*1024<MAXMEMBLOB) {
      maxblob=k*1024;
    }
  }
#endif

  p=src->loc_buffer;
  sqlind=src->loc_indicator;
  if (sqlind!=-1) {
    p1=creatmpfile2("byte", ".jpg.gz");
    if (src->loc_position==1) {
      len = src->loc_size;
      fd3=gzopen(p1,"wb");
      if ( fd3 == NULL ) {
        sprintf(errstr,"Cannot open file %s:%d", p, errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                   verbose, 10);
        return 1;
      }
      k=gzwrite(fd3, p, len);
      if (errno) {
        sprintf(errstr, "gzOS error while writing out file: %d",errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                   errno, logfile, verbose, 10);
        return 1;
      }
      gzclose(fd3);
    } else {
      fd2=fopen(p,"rb");
      if ( fd2 == NULL ) {
        sprintf(errstr,"Cannot open file %s:%d", p, errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                   verbose, 10);
        return 1;
      }
      fd3=gzopen(p1,"wb");
      if ( fd3 == NULL ) {
        sprintf(errstr,"Cannot open file %s:%d", p, errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                   verbose, 10);
        return 1;
      }
      while (!feof(fd2)) {
        i=fread(buf, 1, maxblob-1, fd2);
        if (ferror(fd2)) {
          sprintf(errstr, "OS error while reading temp file: %d:%d",errno, ferror(fd2));
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                     errno, logfile, verbose, 10);
          return 1;
        }
        j=gzwrite(fd3, buf, i);
        if (errno) {
          sprintf(errstr, "gzOS error while writing out file:%d",errno);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                     errno, logfile, verbose, 10);
          return 1;
        }
      }
      fclose(fd2);
      gzclose(fd3);
    }
  }

  if (src->loc_buffer!=NULL && sqlind!=-1) {
/* additional check for sqlind!=-1 because it appeared
   that in many cases loc_buffer was not null altough memory pointed to
   was never allocated; therefore free caused memory leakage;
   fortunately all such cases occurs only for null BLOB */

    if (src->loc_position==2) {
      unlink(p);
    }
    free(src->loc_buffer);
    src->loc_buffer=NULL;
  }
  return 0;
}

int onwritesmartblobgzx2(gzFile fd1, char *filename, int4 sqlxid, ifx_lo_t * src,
                       char delim, unsigned long int rownum, int fieldnum,
                       char *fieldname, const void *logfile,
                       onlogerror_upt onlogerror, int verbose)
{
  char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  char errstr[ERR_BUF_SIZE];
  mint lofd;
  mint error;
  ifx_lo_stat_t *stats;
  ifx_int8_t fsize64;
  char result[256];
  gzFile f;
  mint n;
  unsigned int stat;

  lofd=ifx_lo_open((ifx_lo_t *)src, LO_RDONLY, &error);
  if (lofd<0) {
    sprintf(errstr, "ifx_lo_open: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat(lofd, &stats);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat: %d,"INT4_FORMAT",%s",error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat_size(stats,&fsize64);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat_size: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_filename((ifx_lo_t *)src, "blob????.???.jpg.gz", result, sizeof(result));
  if (error<0) {
    sprintf(errstr, "ifx_lo_filename: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  f=gzopen(result, "wb");
  if ( f == NULL ) {
    sprintf(errstr,"Cannot open file %s:%d", result, errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  while ((n=ifx_lo_read(lofd, buf, MAXMEMBLOB, &error))>0) {
    stat=gzwrite(f, buf, n);
    if (stat==0 && n!=0) {
      sprintf(errstr, "gzOS error %d", errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                 errno, logfile, verbose, 10);
      return 1;
    }
  }
  if (n<0) {
    sprintf(errstr, "ifx_lo_read: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  gzclose(f);
  ifx_lo_close(lofd);
  ifx_lo_stat_free(stats);
  gzprintf(fd1,"<object data=\"%s\" type=\"image/jpeg\" </object>",result);

  return 0;
}


int onwritefield_html1gz(void *fd, int sqltype, int4 sqlxid, char *filename,
                         char *src, const unsigned int buflen,
                         char delim, char escapech, short int sqlind,
                         unsigned long int rownum,  int fieldnum,
                         char * fieldname, const void *logfile,
                         onlogerror_upt onlogerror, int verbose,
                         const char *datahandleoptions,
                         void *fieldstate)
/* returns:  0 if success
 *           1 if error;
 * src : pointer to the buffer;
 */
{
  char buf[32767+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int j, k, len;
  gzFile fd1;
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;

  fd1=(gzFile)fd;

  if (sqlind == -1) {
    stat=gzputs(fd1, "<td>NULL</td>");
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
                 logfile, verbose, 10);
      return 1;
    }
    return 0;
  }
  if (sqltype==SQLTEXT || sqltype==SQLBYTES ) {
    stat=gzputs(fd1, "<td>");
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    if (sqltype==SQLTEXT) {
      stat=onwritetextblobgzx(fd1, filename, (loc_t *) src, buflen, delim,
                              escapech, rownum, fieldnum, fieldname, logfile,
                              onlogerror, verbose);
    } else {
      stat=onwritebyteblobgz(fd1, filename, (loc_t *) src, buflen, delim,
                             rownum, fieldnum, fieldname, logfile,
                             onlogerror, verbose);
    }
    if (stat!=0) {
      return 1;
    }
    stat=gzputs(fd1, "</td>");
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    return 0;
  }
  if (sqltype==CFIXBINTYPE && (sqlxid==XID_BLOB || sqlxid==XID_CLOB) ) {
    stat=gzputs(fd1, "<td>");
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    if (sqlxid==XID_BLOB) {
      stat=onwritesmartblobgzx2(fd1, filename, sqlxid, (ifx_lo_t *) src,
                              escapech, rownum, fieldnum, fieldname,
                              logfile, onlogerror, verbose);
    } else {
      stat=onwritesmartclobgzx(fd1, filename, sqlxid, (ifx_lo_t *) src,
                             escapech, rownum, fieldnum, fieldname,
                             logfile, onlogerror, verbose);
    }
    if (stat!=0) {
      return 1;
    }
    stat=gzputs(fd1, "</td>");
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    return 0;
  }

  stat=gzputs(fd1, "<td>");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }

  len = stleng(src);
  j = 0;
  while ( j < len) {
    k=0;
    while (k<BUF1_SIZE-5 && j<len) {
      buf[k]=src[j];
      j++;
      k++;
    }
    buf[k]='\0';
    stat=gzputs(fd1, buf);
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
  }
  stat=gzputs(fd1, "</td>");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oncrlf_html1gz(void *fd, char *filename, unsigned long int rownum,
                   const void *logfile,
                   onlogerror_upt onlogerror, int verbose,
                   const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  gzFile fd1;
  unsigned int stat;

  fd1=(gzFile)fd;
  stat=gzputs(fd1, "</tr>\n");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int onrecordbegin_html1gz(void *fd, char *filename, unsigned long int rownum,
                          const void *logfile,
                          onlogerror_upt onlogerror, int verbose,
                          const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  gzFile fd1;
  unsigned int stat;

  fd1=(gzFile)fd;
  stat=gzputs(fd1, "<tr>");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int onfinish_html1gz(char *fname, char *tablename, void *handle,
                     unsigned long int rownum,
                     const void *logfile,
                     onlogerror_upt onlogerror, int verbose)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  
  sprintf(errstr, "%ld records were unloaded", rownum);
  onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
             0, logfile, verbose, 30);

  stat=gzputs(handle, "</table>\n");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

void oncleanupxgz(char *fname, void *handle, const void *logfile,
                  onlogerror_upt onlogerror, int verbose,
                  const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  gzFile fd1;
  unsigned int stat;
  
  fd1=handle;
  if (fd1!=NULL) {
    stat=gzclose((gzFile)handle);
    if (stat==EOF) {
      sprintf(errstr, "gzOS error %d",errno);
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
                 logfile, verbose, 10);
      return;
    }
  }
}

int oninit_xml3gz(char *fname, const char *tablename, int batchsize,
                  void **handle, char *escapech, const void *logfile,
                  int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                  int verbose, const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  gzFile f;
  exec sql begin declare section;
  char *slctstmt;
  int fields_number;
  int nullable;
  int i;
  char sqlname[200];
  int ftype;
  int fexttype;
  exec sql end declare section;
  int len;
  char *cp;
  char xmltype[129];
  
  set_rowset_token("NewDataSet");
  if (strcmp("<table>",tablename)) {
    set_row_token((char *)tablename);
  } else {
    set_row_token("Table");
  }

  slctstmt=slctstmt2;
  fields_number=fieldsnum;

  EXEC SQL allocate descriptor 'desc1' WITH MAX :fields_number;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr, "Cannot allocate descriptor "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    return 1;
  }
  EXEC SQL prepare usqlobj3 from :slctstmt;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr,"Cannot prepare: "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    EXEC SQL deallocate descriptor 'desc1';
    return 1;
  }
				    
  EXEC SQL describe usqlobj3 using sql descriptor 'desc1';
  if (fname[0]==0) {
    (*handle)=stdout;
    if (strcmp("<table>",tablename)) {
      sprintf(errstr,"file=<stdout>, tablename=%s", tablename);
    } else {
      sprintf(errstr,"file=<stdout>\n");
    }
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  } else {
    if (strcmp("<table>",tablename)) {
      sprintf(errstr, "file=%s, tablename=%s\n",fname, tablename);
    } else {
      sprintf(errstr, "file=%s\n",fname);
    }
    (*handle) = gzopen(fname,"wb");
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  }
  if ( (*handle) == NULL ) {
    sprintf(errstr,"Cannot open file %s:%d", fname, errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
               verbose, 10);
    return 1;
  }
  f=(FILE *)(*handle);
  (*escapech)='\\';

  stat=gzputs(f, "<?xml version=\"1.0\" standalone=\"yes\"?>\n");
  stat=gzputs(f, "<xs:schema id=\"NewDataSet\" xmlns=\"\" xmlns:xs=\"http://www.w3.org/2001/XMLSchema\" xmlns:msdata=\"urn:schemas-microsoft-com:xml-msdata\">\n");
  stat=gzputs(f, "  <xs:element name=\"NewDataSet\" msdata:IsDataSet=\"true\" msdata:UseCurrentLocale=\"true\">\n");
  stat=gzputs(f, "    <xs:complexType>\n");
  stat=gzputs(f, "      <xs:choice minOccurs=\"0\" maxOccurs=\"unbounded\">\n");
  stat=gzprintf(f,"        <xs:element name=\"%s\">\n",row_token);
  stat=gzputs(f, "          <xs:complexType>\n");
  stat=gzputs(f, "            <xs:sequence>\n");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  for (i=1; i<=fields_number; i++) {
    EXEC SQL get descriptor 'desc1' VALUE :i
             :nullable = NULLABLE, :sqlname=NAME,
             :ftype = TYPE, :fexttype = EXTYPEID;
    if ( sqlca.sqlcode != 0 ) {
      sprintf(errstr, "Cannot get descriptor2 "INT4_FORMAT":%d",sqlca.sqlcode, i);
      onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
      EXEC SQL free usqlobj;
      EXEC SQL deallocate descriptor 'desc1';
      return 1;
    }
    len=stleng(sqlname);
    cp=sqlname+len-1;
    while(len>1 && *cp==' ') {
      *cp='\0';
      len--, cp--;
    }
    switch(ftype) {
      case SQLSMFLOAT:
        strcpy(xmltype,"float");
        break;
      case SQLFLOAT:
        strcpy(xmltype,"double");
        break;
      case SQLMONEY:
      case SQLDECIMAL:
        strcpy(xmltype,"decimal");
        break;
      case SQLINT:
      case SQLSERIAL:
        strcpy(xmltype,"int");
        break;
      case SQLSMINT:
        strcpy(xmltype,"short");
        break;
      case SQLINT8:
      case SQLSERIAL8:
        strcpy(xmltype,"long");
        break;
      case SQLDATE:
        strcpy(xmltype,"dateTime");
        break;
      case SQLDTIME:
        strcpy(xmltype,"dateTime");
        break;
      case SQLINTERVAL:
        strcpy(xmltype,"duration");
        break;
      case SQLBOOL:
        strcpy(xmltype,"boolean");
        break;
      case SQLBYTES:
        strcpy(xmltype, "base64Binary");
        break;
#ifdef SQLUDTFIXED
      case SQLUDTFIXED:
        if (fexttype==XID_BLOB) {
          strcpy(xmltype, "base64Binary");
        } else {
          strcpy(xmltype,"string");
        }
        break;
#endif
      default:
        strcpy(xmltype,"string");
    }
    gzprintf(f, "              <xs:element name=\"%s\" type=\"xs:%s\" minoccurs=\"%d\" />\n",
             sqlname, xmltype, nullable==1?0:1);

  }
  stat=gzputs(f, "            </xs:sequence>\n");
  stat=gzputs(f, "          </xs:complexType>\n");
  stat=gzputs(f, "        </xs:element>\n");
  stat=gzputs(f, "      </xs:choice>\n");
  stat=gzputs(f, "    </xs:complexType>\n");
  stat=gzputs(f, "  </xs:element>\n");
  stat=gzputs(f, "</xs:schema>\n");
  if (stat==EOF) {
    sprintf(errstr, "gzOS error %d",errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  stat=gzputs(f, "<");
  stat=gzputs(f, rowset_token);
  stat=gzputs(f, ">\n");
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror((FILE *)(*handle)));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

