/******************************************************************************
*
* Standard event handlers for unloading functions dedicated for xml format
*
* Author: Robert Sosnowski, Robert.Sosnowski@bzwbk.pl
*
* Version 1.70, 17-10-2005
*
* Copyright (c) 2001 - 2007. All Rights Reserved.
*
* Permission granted to use & distribute for free. Changes in source code
* require Author's acceptance.
*
* This software can be used only at your own risk. Author is not responsible 
* for any damage caused by using this software.
* 
******************************************************************************/

#include "load2cfg.h"
#include "tempfile.h"

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif


#include <stdio.h>
#ifdef HAVE_CTYPE_H
#include <ctype.h>
/* for isdigit on MSVC */
#endif

#ifdef HAVE_ICONV_H
#include <iconv.h>
#endif

#include "unload2.h"
#include <stdlib.h>

#ifdef HAVE_STRING_H
#include <string.h>
#endif

EXEC SQL include sqltypes;

static char *id="@(#) $Id: unload2x.ec 2521 2007-11-21 08:55:56Z sosnor01 $";
static const char cb64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

int onwritefield_xml1(void *fd, int sqltype, int4 sqlxid, char *filename,
                      char *src, const unsigned int buflen,
                      char delim, char escapech, short int sqlind,
                      unsigned long int rownum,  int fieldnum,
                      char *fieldname, const void *logfile,
                      onlogerror_upt onlogerror, int verbose,
                      const char *datahandleoptions,
                      void *fieldstate);
int oncrlf_xml1(void *fd, char *filename, unsigned long int rownum,
               const void *logfile,
               onlogerror_upt onlogerror, int verbose,
               const char *datahandleoptions);

int onrecordbegin_xml1(void *fd, char *filename, unsigned long int rownum,
                       const void *logfile,
                       onlogerror_upt onlogerror, int verbose,
                       const char *datahandleoptions);
int oninit_xml1(char *fname, const char *tablename, int batchsize,
                void **handle, char *escapech, const void *logfile,
                int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                int verbose, const char *datahandleoptions);
int onfinish_xml1(char *fname, char *tablename, void *handle, unsigned long int rownum,
                const void *logfile,
                onlogerror_upt onlogerror, int verbose);

int oninit_xml2(char *fname, const char *tablename, int batchsize,
                void **handle, char *escapech, const void *logfile,
                int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                int verbose, const char *datahandleoptions);

int oninit_html1(char *fname, const char *tablename, int batchsize,
                 void **handle, char *escapech, const void *logfile,
                 int fieldsnum, char *slctstmt2,
                 onlogerror_upt onlogerror, int verbose,
                 const char *datahandleoptions);

int onwritefield_html1(void *fd, int sqltype, int4 sqlxid, char *filename,
                       char *src, const unsigned int buflen,
                       char delim, char escapech, short int sqlind,
                       unsigned long int rownum,  int fieldnum,
                       char * fieldname, const void *logfile,
                       onlogerror_upt onlogerror, int verbose,
                       const char *datahandleoptions,
                       void *fieldstate);

int oncrlf_html1(void *fd, char *filename, unsigned long int rownum,
                 const void *logfile,
                 onlogerror_upt onlogerror, int verbose,
                 const char *datahandleoptions);

int onrecordbegin_html1(void *fd, char *filename, unsigned long int rownum,
                        const void *logfile,
                        onlogerror_upt onlogerror, int verbose,
                        const char *datahandleoptions);

int onfinish_html1(char *fname, char *tablename, void *handle,
                   unsigned long int rownum,
                   const void *logfile,
                   onlogerror_upt onlogerror, int verbose);

int oninit_xml3(char *fname, const char *tablename, int batchsize,
                void **handle, char *escapech, const void *logfile,
                int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                int verbose, const char *datahandleoptions);

int oninit_xsl1(char *fname, const char *tablename, int batchsize,
                void **handle, char *escapech, const void *logfile,
                int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                int verbose, const char *datahandleoptions);


static char rowset_token[20], row_token[20];
static void set_rowset_token(char *s)
{
  strcpy(rowset_token, s);
}

static void set_row_token(char *s)
{
  strcpy(row_token, s);
}

int onwritetextblobx(FILE *fd1, char *filename, loc_t * src,
                     char delim, char escapech, unsigned long int rownum,
                     int fieldnum, char *fieldname, const void *logfile,
                     onlogerror_upt onlogerror, int verbose)
{ char * p;
  char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int j, k, len;
  FILE *fd2;
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  char ch;
  int sqlind;
  unsigned int maxblob;
  char *envp;

  maxblob=MAXMEMBLOB;
#if defined(HAVE_GETENV)
  envp=getenv("DBBLOBBUF");
  if (envp!=NULL && isdigit(envp[0])) {
    k=atol(envp);
    if (k*1024<MAXMEMBLOB) {
      maxblob=k*1024;
    }
  }
#endif
  p=src->loc_buffer;
  sqlind=src->loc_indicator;
  if (sqlind!=-1) {
    if (p[0]=='\0' && src->loc_position==1) { /* if we have an empty string */
      return 0;
    } else {
      if (src->loc_position==1) {
        len = src->loc_size;
        j = 0;
        while ( j < len) {
          k=0;
          while (k<maxblob-2-5 && j<len) {
            if (p[j] == '<' ) {
              buf[k]='&';
              k++;
              buf[k]='l';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (p[j] == '>' ) {
              buf[k]='&';
              k++;
              buf[k]='g';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (p[j] == '&' ) {
              buf[k]='&';
              k++;
              buf[k]='a';
              k++;
              buf[k]='m';
              k++;
              buf[k]='p';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (p[j] == '\"' ) {
              buf[k]='&';
              k++;
              buf[k]='q';
              k++;
              buf[k]='u';
              k++;
              buf[k]='o';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (p[j] == '\'' ) {
              buf[k]='&';
              k++;
              buf[k]='a';
              k++;
              buf[k]='p';
              k++;
              buf[k]='o';
              k++;
              buf[k]='s';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            buf[k]=p[j];
            j++;
            k++;
          }
          buf[k]='\0';
          stat=fwrite(buf, 1, k, fd1);
          if (ferror(fd1)) {
            sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
            onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                       errno, logfile, verbose, 10);
            return 1;
          }
        }
      } else {
        fd2=fopen(p,"rb");
        if ( fd2 == NULL ) {
          sprintf(errstr,"Cannot open file %s:%d", p, errno);
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
                     logfile, verbose, 10);
          return 1;
        }
        while (!feof(fd2)) {
          k=0;
          while (k<maxblob-2-5 ) {
            ch=fgetc(fd2);
            if ( ferror(fd2) ) {
              sprintf(errstr,"Cannot read temp file %s:%d", p, errno);
              onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
                         logfile, verbose, 10);
              return 1;
            }
            if (feof(fd2)) {
              break;
            }
            if (ch == '<' ) {
              buf[k]='&';
              k++;
              buf[k]='l';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (ch == '>' ) {
              buf[k]='&';
              k++;
              buf[k]='g';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (ch == '&' ) {
              buf[k]='&';
              k++;
              buf[k]='a';
              k++;
              buf[k]='m';
              k++;
              buf[k]='p';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (ch == '\"' ) {
              buf[k]='&';
              k++;
              buf[k]='q';
              k++;
              buf[k]='u';
              k++;
              buf[k]='o';
              k++;
              buf[k]='t';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            if (ch == '\'' ) {
              buf[k]='&';
              k++;
              buf[k]='a';
              k++;
              buf[k]='p';
              k++;
              buf[k]='o';
              k++;
              buf[k]='s';
              k++;
              buf[k]=';';
              k++;
              j++;
              continue;
            }
            buf[k]=ch;
            k++;
          }
          buf[k]='\0';
          j=fwrite(buf, 1, k, fd1);
          if ( ferror(fd1)) {
            sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
            onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                       errno, logfile, verbose, 10);
            return 1;
          }
        }
        fclose(fd2);
      }
    }
  }
  if (src->loc_buffer!=NULL) {
    if (src->loc_position==2) {
      unlink(p);
    }
    free(src->loc_buffer);
    src->loc_buffer=NULL;
  }
  return 0;
}


void encodeblock( unsigned char in[3], unsigned char out[4], int len )
{
    out[3] = (unsigned char) (len > 2 ? cb64[ in[2] & 0x3f ] : '=');
    out[2] = (unsigned char) (len > 1 ? cb64[ ((in[1] & 0x0f) << 2) | ((in[2] & 0xc0) >> 6) ] : '=');
    out[1] = cb64[ ((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4) ];
    out[0] = cb64[ in[0] >> 2 ];
}

void encodeblock3( unsigned char in[3], unsigned char out[4])
{
    out[3] = cb64[ in[2] & 0x3f ] ;
    out[2] = cb64[ ((in[1] & 0x0f) << 2) | ((in[2] & 0xc0) >> 6) ] ;
    out[1] = cb64[ ((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4) ];
    out[0] = cb64[ in[0] >> 2 ];
}


int onwritebyteblob64(FILE *fd1, char *filename, loc_t * src,
                      char delim, unsigned long int rownum,
                      int fieldnum, char *fieldname, const void *logfile,
                      onlogerror_upt onlogerror, int verbose)
{ char * p;
  unsigned char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int i, j, k, len;
  FILE *fd2;
  char errstr[ERR_BUF_SIZE];
  int j2;
  int sqlind;
  unsigned int maxblob;
  char *envp;

  maxblob=MAXMEMBLOB;
#if defined(HAVE_GETENV)
  envp=getenv("DBBLOBBUF");
  if (envp!=NULL && isdigit(envp[0])) {
    k=atol(envp);
    if (k*1024<MAXMEMBLOB) {
      maxblob=k*1024;
    }
  }
#endif

  p=src->loc_buffer;
  sqlind=src->loc_indicator;
  if (sqlind!=-1) {
    if (src->loc_position==1) {
      len = src->loc_size;
      j=0;
      while (j<len) {
        i=0;
        while (i<=maxblob-4 && j<len) {
          if (len-j>2) {
            encodeblock3((unsigned char *)p+j, buf+i);
          } else {
            encodeblock((unsigned char *)p+j, buf+i, len-j);
          }
          j+=3;
          i+=4;
        }
        
        k=fwrite(buf, 1, i, fd1);
        if (ferror(fd1)) {
          sprintf(errstr, "OS error while writing out file: %d:%d",errno, ferror(fd1));
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                     errno, logfile, verbose, 10);
          return 1;
        }
      }
    } else {
      fd2=fopen(p,"rb");
      if ( fd2 == NULL ) {
        sprintf(errstr,"Cannot open file %s:%d", p, errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                   verbose, 10);
        return 1;
      }
      while (1) {
        i=fread(buf, 1, 3*((maxblob-1)/4), fd2);
        if (ferror(fd2)) {
          sprintf(errstr, "OS error while reading temp file: %d:%d",errno, ferror(fd2));
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                     errno, logfile, verbose, 10);
          return 1;
        }
        j2=(i+2)/3; /* number of triples */
        if (i%3) {
          encodeblock(buf+(j2-1)*3, buf+(j2-1)*4, i%3);
          j2--;
        }
        j2--; /* number of remaining triples - 1 */
        while (j2>=0) {
          encodeblock3(buf+j2*3,buf+j2*4);
          j2--;
        }
        j=fwrite(buf,1,4*((i+2)/3), fd1);
        if (ferror(fd1)) {
          sprintf(errstr, "OS error while writing out file:%d:%d",errno, ferror(fd1));
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                     errno, logfile, verbose, 10);
          return 1;
        }
        if (feof(fd2)) {
          break;
        }
      }
      fclose(fd2);
    }
  }
  if (src->loc_buffer!=NULL && sqlind!=-1) {
/* additional check for sqlind!=-1 because it appeared
   that in many cases loc_buffer was not null altough memory pointed to
   was never allocated; therefore free caused memory leakage;
   fortunately all such cases occurs only for null BLOB */

    if (src->loc_position==2) {
      unlink(p);
    }
    free(src->loc_buffer);
    src->loc_buffer=NULL;
  }
  return 0;
}


void entityencode(char *srcbuf, char *dstbuf, int *p1, int *size1, int len1, int batchsize)
{
  int i,j;
  char ch;

  i=*p1;
  j=0;
  while (i<len1 && j<batchsize-7) {
    ch=srcbuf[i];
    if (ch == '<' ) {
      dstbuf[j]='&';
      j++;
      dstbuf[j]='l';
      j++;
      dstbuf[j]='t';
      j++;
      dstbuf[j]=';';
      j++;
      i++;
      continue;
    }
    if (ch == '>' ) {
      dstbuf[j]='&';
      j++;
      dstbuf[j]='g';
      j++;
      dstbuf[j]='t';
      j++;
      dstbuf[j]=';';
      j++;
      i++;
      continue;
    }
    if (ch == '&' ) {
      dstbuf[j]='&';
      j++;
      dstbuf[j]='a';
      j++;
      dstbuf[j]='m';
      j++;
      dstbuf[j]='p';
      j++;
      dstbuf[j]=';';
      j++;
      i++;
      continue;
    }
    if (ch == '\"' ) {
      dstbuf[j]='&';
      j++;
      dstbuf[j]='q';
      j++;
      dstbuf[j]='u';
      j++;
      dstbuf[j]='o';
      j++;
      dstbuf[j]='t';
      j++;
      dstbuf[j]=';';
      j++;
      i++;
      continue;
    }
    if (ch == '\'' ) {
      dstbuf[j]='&';
      j++;
      dstbuf[j]='a';
      j++;
      dstbuf[j]='p';
      j++;
      dstbuf[j]='o';
      j++;
      dstbuf[j]='s';
      j++;
      dstbuf[j]=';';
      j++;
      i++;
      continue;
    }
    dstbuf[j]=ch;
    i++;
    j++;
  }
  *p1=i;
  *size1=j;
}
 
int onwritesmartclobx(FILE *fd1, char *filename, int4 sqlxid, ifx_lo_t * src,
                      char delim, unsigned long int rownum, int fieldnum,
                      char *fieldname, const void *logfile,
                      onlogerror_upt onlogerror, int verbose)
{
  char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
#define BATCHSIZE 5000
  char buf2[BATCHSIZE+1];
  char errstr[ERR_BUF_SIZE];
  mint lofd;
  mint error;
  ifx_lo_stat_t *stats;
  ifx_int8_t fsize64;
  mint n;
  int i, j;

  lofd=ifx_lo_open((ifx_lo_t *)src, LO_RDONLY, &error);
  if (lofd<0) {
    sprintf(errstr, "ifx_lo_open: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat(lofd, &stats);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat_size(stats,&fsize64);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat_size: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  i=0;
  while ((n=ifx_lo_read(lofd, buf, MAXMEMBLOB, &error))>0) {
    while (i<n) {
      entityencode(buf, buf2, &i, &j, n, BATCHSIZE);
      fwrite(buf, 1, j, fd1);
      if (ferror(fd1)) {
        sprintf(errstr, "OS error while writing out file:%d:%d",errno, ferror(fd1));
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                   errno, logfile, verbose, 10);
        return 1;
      }
    }
  }
  if (n<0) {
    sprintf(errstr, "ifx_lo_read: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  ifx_lo_close(lofd);
  ifx_lo_stat_free(stats);

  return 0;
}


int onwritesmartblobx(FILE *fd1, char *filename, int4 sqlxid, ifx_lo_t * src,
                      char delim, unsigned long int rownum, int fieldnum,
                      char *fieldname, const void *logfile,
                      onlogerror_upt onlogerror, int verbose)
{
  char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  char errstr[ERR_BUF_SIZE];
  mint lofd;
  mint error;
  ifx_lo_stat_t *stats;
  ifx_int8_t fsize64;
  mint n, j2;

  lofd=ifx_lo_open((ifx_lo_t *)src, LO_RDONLY, &error);
  if (lofd<0) {
    sprintf(errstr, "ifx_lo_open: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat(lofd, &stats);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat_size(stats,&fsize64);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat_size: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  while ((n=ifx_lo_read(lofd, buf, ((MAXMEMBLOB-1)/4)*3, &error))>0) {
    j2=(n+2)/3; /* number of triples */
    if (n%3) {
      encodeblock((unsigned char *)buf+(j2-1)*3, (unsigned char *)buf+(j2-1)*4, n%3);
      j2--;
    }
    j2--; /* number of remaining triples - 1 */
    while (j2>=0) {
      encodeblock3((unsigned char *)buf+j2*3,(unsigned char *)buf+j2*4);
      j2--;
    }
    fwrite(buf,1,4*((n+2)/3), fd1);
    if (ferror(fd1)) {
      sprintf(errstr, "OS error while writing out file:%d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                 errno, logfile, verbose, 10);
      return 1;
    }
  }
  if (n<0) {
    sprintf(errstr, "ifx_lo_read: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  ifx_lo_close(lofd);
  ifx_lo_stat_free(stats);

  return 0;
}


int onwritefield_xml1(void *fd, int sqltype, int4 sqlxid, char *filename,
                      char *src, const unsigned int buflen,
                      char delim, char escapech, short int sqlind,
                      unsigned long int rownum,  int fieldnum,
                      char * fieldname, const void *logfile,
                      onlogerror_upt onlogerror, int verbose,
                      const char *datahandleoptions,
                      void *fieldstate)
/* returns:  0 if success
 *           1 if error;
 * src : pointer to the buffer;
 */
{
  char buf[BUF1_SIZE+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int j, k, len;
  FILE *fd1;
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  char s[100];

  fd1=(FILE *)fd;

  if (sqlind == -1) {
    return 0; /* NULL is not written at all */
  }
  if (sqltype==SQLTEXT || sqltype==SQLBYTES ) {
    sprintf(s, "<%s>",fieldname);
    stat=fputs(s,fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    if (sqltype==SQLTEXT) {
      stat=onwritetextblobx(fd1, filename, (loc_t *) src, delim,
                            escapech, rownum, fieldnum, fieldname, logfile,
                            onlogerror, verbose);
    } else {
      stat=onwritebyteblob64(fd, filename, (loc_t *) src, delim, rownum,
                             fieldnum, fieldname, logfile, onlogerror, verbose);
    }
    if (stat!=0) {
      return 1;
    }
    sprintf(s, "</%s>\n",fieldname);
    stat=fputs(s,fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    return 0;
  }
  if (sqltype==CFIXBINTYPE && (sqlxid==XID_BLOB || sqlxid==XID_CLOB) ) {
    sprintf(s, "<%s>",fieldname);
    stat=fputs(s,fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    if (sqlxid==XID_BLOB) {
      stat=onwritesmartblobx(fd1, filename, sqlxid, (ifx_lo_t *) src,
                             escapech, rownum, fieldnum, fieldname,
                             logfile, onlogerror, verbose);
    } else {
      stat=onwritesmartclobx(fd1, filename, sqlxid, (ifx_lo_t *) src,
                             escapech, rownum, fieldnum, fieldname,
                             logfile, onlogerror, verbose);
    }
    if (stat!=0) {
      return 1;
    }
    sprintf(s, "</%s>\n",fieldname);
    stat=fputs(s,fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    return 0;
  }

  sprintf(s, "<%s>",fieldname);
  stat=fputs(s,fd1);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }

  len = stleng(src);
  j = 0;
  while ( j < len) {
    k=0;
    while (k<BUF1_SIZE-5 && j<len) {
      if (src[j] == '<' ) {
        buf[k]='&';
        k++;
        buf[k]='l';
        k++;
        buf[k]='t';
        k++;
        buf[k]=';';
        k++;
	j++;
	continue;
      }
      if (src[j] == '>' ) {
        buf[k]='&';
        k++;
        buf[k]='g';
        k++;
        buf[k]='t';
        k++;
        buf[k]=';';
        k++;
	j++;
	continue;
      }
      if (src[j] == '&' ) {
        buf[k]='&';
        k++;
        buf[k]='a';
        k++;
        buf[k]='m';
        k++;
        buf[k]='p';
        k++;
        buf[k]=';';
        k++;
	j++;
	continue;
      }
      if (src[j] == '\"' ) {
        buf[k]='&';
        k++;
        buf[k]='q';
        k++;
        buf[k]='u';
        k++;
        buf[k]='o';
        k++;
        buf[k]='t';
        k++;
        buf[k]=';';
        k++;
	j++;
	continue;
      }
      if (src[j] == '\'' ) {
        buf[k]='&';
        k++;
        buf[k]='a';
        k++;
        buf[k]='p';
        k++;
        buf[k]='o';
        k++;
        buf[k]='s';
        k++;
        buf[k]=';';
        k++;
	j++;
	continue;
      }
      buf[k]=src[j];
      j++;
      k++;
    }
    buf[k]='\0';
    stat=fputs(buf,fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
  }
  sprintf(s, "</%s>\n",fieldname);
  stat=fputs(s,fd1);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oncrlf_xml1(void *fd, char *filename, unsigned long int rownum,
                const void *logfile,
                onlogerror_upt onlogerror, int verbose,
                const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  FILE *fd1;
  unsigned int stat;

  fd1=(FILE *)fd;
  stat=fputs("</",fd1);
  stat=fputs(row_token,fd1);
  stat=fputs(">\n",fd1);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int onrecordbegin_xml1(void *fd, char *filename, unsigned long int rownum,
                       const void *logfile,
               onlogerror_upt onlogerror, int verbose,
               const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  FILE *fd1;
  unsigned int stat;
  char s[100];

  fd1=(FILE *)fd;
  sprintf(s,"<%s num=\"%ld\">\n", row_token, rownum);
  stat=fputs(s,fd1);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d\n",errno, ferror(fd1));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oninit_xml1(char *fname, const char *tablename, int batchsize,
                void **handle, char *escapech, const void *logfile,
                int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                int verbose, const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;

  set_rowset_token("ROWSET");
  set_row_token("ROW");

  if (fname[0]==0) {
    (*handle)=stdout;
    if (strcmp("<table>",tablename)) {
      sprintf(errstr,"file=<stdout>, tablename=%s", tablename);
    } else {
      sprintf(errstr,"file=<stdout>\n");
    }
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  } else {
    if (strcmp("<table>",tablename)) {
      sprintf(errstr, "file=%s, tablename=%s\n",fname, tablename);
    } else {
      sprintf(errstr, "file=%s\n",fname);
    }
#ifdef NOBINARY_FLAG
    (*handle) = fopen(fname,"w");
#else
    (*handle) = fopen(fname,"wb");
#endif
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  }
  if ( (*handle) == NULL ) {
    sprintf(errstr,"Cannot open file '%s:%d'", fname, errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
               verbose, 10);
    return 1;
  }
  (*escapech)='\\';
  stat=fputs("<?xml version=\"1.0\" ?>\n",(*handle));
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror((FILE *)(*handle)));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  stat=fputs("<", (*handle));
  stat=fputs(rowset_token,(*handle));
  stat=fputs(">\n",(*handle));
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror((FILE *)(*handle)));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}



int onfinish_xml1(char *fname, char *tablename, void *handle,
                  unsigned long int rownum,
                  const void *logfile,
                  onlogerror_upt onlogerror, int verbose)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  
  sprintf(errstr, "%ld records were unloaded", rownum);
  onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
             0, logfile, verbose, 30);

  stat=fputs("</",handle);
  stat=fputs(rowset_token,handle);
  stat=fputs(">\n",handle);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror((FILE *)handle));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oninit_xml2(char *fname, const char *tablename, int batchsize,
                void **handle, char *escapech, const void *logfile,
                int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                int verbose, const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  FILE * f;
  exec sql begin declare section;
  char *slctstmt;
  int fields_number;
  int nullable;
  int i;
  char sqlname[200];
  exec sql end declare section;
  int len;
  char *cp;
  
  set_rowset_token("ROWSET");
  set_row_token("ROW");

  slctstmt=slctstmt2;
  fields_number=fieldsnum;

  EXEC SQL allocate descriptor 'desc1' WITH MAX :fields_number;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr, "Cannot allocate descriptor "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    return 1;
  }
  EXEC SQL prepare usqlobj from :slctstmt;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr,"Cannot prepare: "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    EXEC SQL deallocate descriptor 'desc1';
    return 1;
  }
				    
  EXEC SQL describe usqlobj using sql descriptor 'desc1';
  if (fname[0]==0) {
    (*handle)=stdout;
    if (strcmp("<table>",tablename)) {
      sprintf(errstr,"file=<stdout>, tablename=%s", tablename);
    } else {
      sprintf(errstr,"file=<stdout>\n");
    }
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  } else {
    if (strcmp("<table>",tablename)) {
      sprintf(errstr, "file=%s, tablename=%s\n",fname, tablename);
    } else {
      sprintf(errstr, "file=%s\n",fname);
    }
#ifdef NOBINARY_FLAG
    (*handle) = fopen(fname,"w");
#else
    (*handle) = fopen(fname,"wb");
#endif
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  }
  if ( (*handle) == NULL ) {
    sprintf(errstr,"Cannot open file '%s:%d'\n", fname, errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
               verbose, 10);
    return 1;
  }
  f=(FILE *)(*handle);
  (*escapech)='\\';
  stat=fputs("<?xml version=\"1.0\" ?>\n",f);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(f));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  fprintf(f, "<!DOCTYPE %s [\n", rowset_token);
  fprintf(f, "<!ELEMENT %s (%s)*>\n", rowset_token, row_token);
  fprintf(f, "<!ELEMENT %s (", row_token);

  i=1;
  EXEC SQL get descriptor 'desc1' VALUE :i
           :nullable = NULLABLE, :sqlname=NAME ;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr, "Cannot get descriptor1 "INT4_FORMAT":%d",sqlca.sqlcode, i);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    EXEC SQL free usqlobj;
    EXEC SQL deallocate descriptor 'desc1';
    return 1;
  }
  len=stleng(sqlname);
  cp=sqlname+len-1;
  while(len>1 && *cp==' ') {
    *cp='\0';
    len--, cp--;
  }
  fprintf(f, "%s", sqlname);
  if (nullable==1) { /* allow NULL values */
    fputc('?',f);
  }
  for (i=2; i<=fields_number; i++) {
    EXEC SQL get descriptor 'desc1' VALUE :i
             :nullable = NULLABLE, :sqlname=NAME ;
    if ( sqlca.sqlcode != 0 ) {
      sprintf(errstr, "Cannot get descriptor2 "INT4_FORMAT":%d",sqlca.sqlcode, i);
      onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
      EXEC SQL free usqlobj;
      EXEC SQL deallocate descriptor 'desc1';
      return 1;
    }
    len=stleng(sqlname);
    cp=sqlname+len-1;
    while(len>1 && *cp==' ') {
      *cp='\0';
      len--, cp--;
    }
    fprintf(f, ", %s", sqlname);
    if (nullable==1) { /* allow NULL values */
      fputc('?',f);
    }
  }
  fprintf(f, ")>\n");
  fprintf(f, "<!ATTLIST %s num CDATA #REQUIRED>\n", row_token);

  for (i=1; i<=fields_number; i++) {
    EXEC SQL get descriptor 'desc1' VALUE :i
             :nullable = NULLABLE, :sqlname=NAME ;
    if ( sqlca.sqlcode != 0 ) {
      sprintf(errstr, "Cannot get descriptor2 "INT4_FORMAT":%d",sqlca.sqlcode, i);
      onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
      EXEC SQL free usqlobj;
      EXEC SQL deallocate descriptor 'desc1';
      return 1;
    }
    len=stleng(sqlname);
    cp=sqlname+len-1;
    while(len>1 && *cp==' ') {
      *cp='\0';
      len--, cp--;
    }
    fprintf(f, "<!ELEMENT %s (#PCDATA)>\n", sqlname);
  }
  EXEC SQL free usqlobj;
  EXEC SQL deallocate descriptor 'desc1';

  fprintf(f, "]>\n");
  stat=fputs("<",f);
  stat=fputs(rowset_token,f);
  stat=fputs(">\n",f);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(f));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oninit_html1(char *fname, const char *tablename, int batchsize,
                 void **handle, char *escapech, const void *logfile,
                 int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                 int verbose, const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  FILE * f;
  exec sql begin declare section;
  char *slctstmt;
  int fields_number;
  int nullable;
  int i;
  char sqlname[200];
  exec sql end declare section;
  int len;
  char *cp;
  
  set_rowset_token("ROWSET");
  set_row_token("ROW");

  slctstmt=slctstmt2;
  fields_number=fieldsnum;

  EXEC SQL allocate descriptor 'desc1' WITH MAX :fields_number;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr, "Cannot allocate descriptor "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    return 1;
  }
  EXEC SQL prepare usqlobj2 from :slctstmt;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr,"Cannot prepare: "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    EXEC SQL deallocate descriptor 'desc1';
    return 1;
  }
				    
  EXEC SQL describe usqlobj2 using sql descriptor 'desc1';
  if (fname[0]==0) {
    (*handle)=stdout;
    if (strcmp("<table>",tablename)) {
      sprintf(errstr,"file=<stdout>, tablename=%s", tablename);
    } else {
      sprintf(errstr,"file=<stdout>\n");
    }
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  } else {
    if (strcmp("<table>",tablename)) {
      sprintf(errstr, "file=%s, tablename=%s\n",fname, tablename);
    } else {
      sprintf(errstr, "file=%s\n",fname);
    }
#ifdef NOBINARY_FLAG
    (*handle) = fopen(fname,"w");
#else
    (*handle) = fopen(fname,"wb");
#endif
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  }
  if ( (*handle) == NULL ) {
    sprintf(errstr,"Cannot open file %s:%d", fname, errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
               verbose, 10);
    return 1;
  }
  f=(FILE *)(*handle);
  (*escapech)='\\';
  stat=fputs("<table>\n<tr>",f);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(f));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }

  /* table headings */
  for (i = 1; i <= fields_number; i++) {
    EXEC SQL get descriptor 'desc1' VALUE :i
             :nullable = NULLABLE, :sqlname=NAME ;
    if ( sqlca.sqlcode != 0 ) {
      sprintf(errstr, "Cannot get descriptor1 "INT4_FORMAT":%d",sqlca.sqlcode, i);
      onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
      EXEC SQL free usqlobj2;
      EXEC SQL deallocate descriptor 'desc1';
      return 1;
    }
    len=stleng(sqlname);
    cp=sqlname+len-1;
    while(len>1 && *cp==' ') {
      *cp='\0';
      len--, cp--;
    }
        
    fprintf(f, "<th>%s</th>", sqlname);
  }    
  stat=fputs("</tr>\n",f);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d", errno, ferror(f));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }


  EXEC SQL free usqlobj2;
  EXEC SQL deallocate descriptor 'desc1';

  return 0;
}

int onwritebyteblobx2(FILE *fd1, char *filename, loc_t * src,
                      char delim, unsigned long int rownum,
                      int fieldnum, char *fieldname, const void *logfile,
                      onlogerror_upt onlogerror, int verbose)
{ char * p;
  unsigned char buf[MAXMEMBLOB+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int i, j, k, len;
  FILE *fd2, *fd3;
  char errstr[ERR_BUF_SIZE];
  int sqlind;
  unsigned int maxblob;
  char *envp;
  char *p1;

  maxblob=MAXMEMBLOB;
#if defined(HAVE_GETENV)
  envp=getenv("DBBLOBBUF");
  if (envp!=NULL && isdigit(envp[0])) {
    k=atol(envp);
    if (k*1024<MAXMEMBLOB) {
      maxblob=k*1024;
    }
  }
#endif

  p=src->loc_buffer;
  sqlind=src->loc_indicator;
  if (sqlind!=-1) {
    p1=creatmpfile2("byte", ".jpg");
    if (src->loc_position==1) {
      len = src->loc_size;
      fd3=fopen(p1,"wb");
      if ( fd3 == NULL ) {
        sprintf(errstr,"Cannot open file %s:%d", p, errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                   verbose, 10);
        return 1;
      }
      k=fwrite(p, 1, len, fd3);
      if (ferror(fd3)) {
        sprintf(errstr, "OS error while writing out file: %d:%d",errno, ferror(fd3));
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                   errno, logfile, verbose, 10);
        return 1;
      }
      fclose(fd3);
    } else {
      fd2=fopen(p,"rb");
      if ( fd2 == NULL ) {
        sprintf(errstr,"Cannot open file %s:%d", p, errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                   verbose, 10);
        return 1;
      }
      fd3=fopen(p1,"wb");
      if ( fd3 == NULL ) {
        sprintf(errstr,"Cannot open file %s:%d", p, errno);
        onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                   verbose, 10);
        return 1;
      }
      while (!feof(fd2)) {
        i=fread(buf, 1, maxblob-1, fd2);
        if (ferror(fd2)) {
          sprintf(errstr, "OS error while reading temp file: %d:%d",errno, ferror(fd2));
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                     errno, logfile, verbose, 10);
          return 1;
        }
        j=fwrite(buf,1,i, fd3);
        if (ferror(fd3)) {
          sprintf(errstr, "OS error while writing out file:%d:%d",errno, ferror(fd3));
          onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
                     errno, logfile, verbose, 10);
          return 1;
        }
      }
      fclose(fd2);
      fclose(fd3);
    }
  }

  if (src->loc_buffer!=NULL && sqlind!=-1) {
/* additional check for sqlind!=-1 because it appeared
   that in many cases loc_buffer was not null altough memory pointed to
   was never allocated; therefore free caused memory leakage;
   fortunately all such cases occurs only for null BLOB */

    if (src->loc_position==2) {
      unlink(p);
    }
    free(src->loc_buffer);
    src->loc_buffer=NULL;
  }
  return 0;
}

int onwritesmartblobx2(FILE *fd1, char *filename, int4 sqlxid, ifx_lo_t * src,
                       char delim, unsigned long int rownum, int fieldnum,
                       char *fieldname, const void *logfile,
                       onlogerror_upt onlogerror, int verbose)
{
  char errstr[ERR_BUF_SIZE];
  mint lofd;
  mint error;
  ifx_lo_stat_t *stats;
  ifx_int8_t fsize64;
  char result[256];

  lofd=ifx_lo_open((ifx_lo_t *)src, LO_RDONLY, &error);
  if (lofd<0) {
    sprintf(errstr, "ifx_lo_open: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat(lofd, &stats);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_stat_size(stats,&fsize64);
  if (error<0) {
    sprintf(errstr, "ifx_lo_stat_size: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1],
               sqlca.sqlerrm, rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  error=ifx_lo_copy_to_file((ifx_lo_t *)src, "blob????.???.jpg", LO_CLIENT_FILE, result);
  if (error<0) {
    sprintf(errstr, "ifx_lo_copy_to_file: %d,"INT4_FORMAT",%s",
            error, sqlca.sqlerrd[1], sqlca.sqlerrm);
    onlogerror(__FILE__, __LINE__, errstr, error, sqlca.sqlerrd[1], sqlca.sqlerrm,
               rownum, rownum, 0, logfile, verbose, 10);
    return 1;
  }
  ifx_lo_close(lofd);
  ifx_lo_stat_free(stats);
  fprintf(fd1,"<object data=\"%s\" type=\"image/jpeg\" </object>",result);

  return 0;
}

int onwritefield_html1(void *fd, int sqltype, int4 sqlxid, char *filename,
                       char *src, const unsigned int buflen,
                       char delim, char escapech, short int sqlind,
                       unsigned long int rownum,  int fieldnum,
                       char * fieldname, const void *logfile,
                       onlogerror_upt onlogerror, int verbose,
                       const char *datahandleoptions,
                       void *fieldstate)
/* returns:  0 if success
 *           1 if error;
 * src : pointer to the buffer;
 */
{
  char buf[32767+1];  /* size enough for max size SQL datatype (non-BLOB)
                         in Informix it is CHAR(32767) */
  unsigned int j, k, len;
  FILE *fd1;
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;

  fd1=(FILE *)fd;

  if (sqlind == -1) {
    stat=fputs("<td>NULL</td>",fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
                 logfile, verbose, 10);
      return 1;
    }
    return 0;
  }
  if (sqltype==SQLTEXT || sqltype==SQLBYTES ) {
    stat=fputs("<td>",fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    if (sqltype==SQLTEXT) {
      stat=onwritetextblobx(fd1, filename, (loc_t *) src, delim,
                            escapech, rownum, fieldnum, fieldname, logfile,
                            onlogerror, verbose);
    } else {
      stat=onwritebyteblobx2(fd, filename, (loc_t *) src, delim, rownum,
                             fieldnum, fieldname, logfile, onlogerror, verbose);
    }
    if (stat!=0) {
      return 1;
    }
    stat=fputs("</td>",fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    return 0;
  }
  if (sqltype==CFIXBINTYPE && (sqlxid==XID_BLOB || sqlxid==XID_CLOB) ) {
    stat=fputs("<td>",fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    if (sqlxid==XID_BLOB) {
      stat=onwritesmartblobx2(fd1, filename, sqlxid, (ifx_lo_t *) src,
                              escapech, rownum, fieldnum, fieldname,
                              logfile, onlogerror, verbose);
    } else {
      stat=onwritesmartclobx(fd1, filename, sqlxid, (ifx_lo_t *) src,
                             escapech, rownum, fieldnum, fieldname,
                             logfile, onlogerror, verbose);
    }
    if (stat!=0) {
      return 1;
    }
    stat=fputs("</td>",fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
    return 0;
  }

  stat=fputs("<td>",fd1);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }

  len = stleng(src);
  j = 0;
  while ( j < len) {
    k=0;
    while (k<BUF1_SIZE-5 && j<len) {
      buf[k]=src[j];
      j++;
      k++;
    }
    buf[k]='\0';
    stat=fputs(buf,fd1);
    if (stat==EOF) {
      sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
                 logfile, verbose, 10);
      return 1;
    }
  }
  stat=fputs("</td>",fd1);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oncrlf_html1(void *fd, char *filename, unsigned long int rownum,
                 const void *logfile,
                 onlogerror_upt onlogerror, int verbose,
                 const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  FILE *fd1;
  unsigned int stat;

  fd1=(FILE *)fd;
  stat=fputs("</tr>\n",fd1);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int onrecordbegin_html1(void *fd, char *filename, unsigned long int rownum,
                        const void *logfile,
               onlogerror_upt onlogerror, int verbose,
               const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  FILE *fd1;
  unsigned int stat;

  fd1=(FILE *)fd;
  stat=fputs("<tr>",fd1);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(fd1));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int onfinish_html1(char *fname, char *tablename, void *handle,
                   unsigned long int rownum,
                   const void *logfile,
                   onlogerror_upt onlogerror, int verbose)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  
  sprintf(errstr, "%ld records were unloaded", rownum);
  onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, rownum, rownum,
             0, logfile, verbose, 30);

  stat=fputs("</table>\n",handle);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror((FILE *)handle));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oninit_xml3(char *fname, const char *tablename, int batchsize,
                void **handle, char *escapech, const void *logfile,
                int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                int verbose, const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  FILE * f;
  exec sql begin declare section;
  char *slctstmt;
  int fields_number;
  int nullable;
  int i;
  char sqlname[200];
  int ftype;
  int fexttype;
  exec sql end declare section;
  int len;
  char *cp;
  char xmltype[129];
  
  set_rowset_token("NewDataSet");
  if (strcmp("<table>",tablename)) {
    set_row_token((char *)tablename);
  } else {
    set_row_token("Table");
  }

  slctstmt=slctstmt2;
  fields_number=fieldsnum;

  EXEC SQL allocate descriptor 'desc1' WITH MAX :fields_number;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr, "Cannot allocate descriptor "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    return 1;
  }
  EXEC SQL prepare usqlobj3 from :slctstmt;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr,"Cannot prepare: "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    EXEC SQL deallocate descriptor 'desc1';
    return 1;
  }
				    
  EXEC SQL describe usqlobj3 using sql descriptor 'desc1';
  if (fname[0]==0) {
    (*handle)=stdout;
    if (strcmp("<table>",tablename)) {
      sprintf(errstr,"file=<stdout>, tablename=%s", tablename);
    } else {
      sprintf(errstr,"file=<stdout>\n");
    }
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  } else {
    if (strcmp("<table>",tablename)) {
      sprintf(errstr, "file=%s, tablename=%s\n",fname, tablename);
    } else {
      sprintf(errstr, "file=%s\n",fname);
    }
    (*handle) = fopen(fname,"wb");
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  }
  if ( (*handle) == NULL ) {
    sprintf(errstr,"Cannot open file %s:%d", fname, errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
               verbose, 10);
    return 1;
  }
  f=(FILE *)(*handle);
  (*escapech)='\\';

  stat=fputs("<?xml version=\"1.0\" standalone=\"yes\"?>\n",f);
  stat=fputs("<xs:schema id=\"NewDataSet\" xmlns=\"\" xmlns:xs=\"http://www.w3.org/2001/XMLSchema\" xmlns:msdata=\"urn:schemas-microsoft-com:xml-msdata\">\n",f);
  stat=fputs("  <xs:element name=\"NewDataSet\" msdata:IsDataSet=\"true\" msdata:UseCurrentLocale=\"true\">\n",f);
  stat=fputs("    <xs:complexType>\n",f);
  stat=fputs("      <xs:choice minOccurs=\"0\" maxOccurs=\"unbounded\">\n",f);
  stat=fprintf(f,"        <xs:element name=\"%s\">\n",row_token);
  stat=fputs("          <xs:complexType>\n",f);
  stat=fputs("            <xs:sequence>\n",f);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(f));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  for (i=1; i<=fields_number; i++) {
    EXEC SQL get descriptor 'desc1' VALUE :i
             :nullable = NULLABLE, :sqlname=NAME,
             :ftype = TYPE, :fexttype = EXTYPEID;
    if ( sqlca.sqlcode != 0 ) {
      sprintf(errstr, "Cannot get descriptor2 "INT4_FORMAT":%d",sqlca.sqlcode, i);
      onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
      EXEC SQL free usqlobj;
      EXEC SQL deallocate descriptor 'desc1';
      return 1;
    }
    len=stleng(sqlname);
    cp=sqlname+len-1;
    while(len>1 && *cp==' ') {
      *cp='\0';
      len--, cp--;
    }
    switch(ftype) {
      case SQLSMFLOAT:
        strcpy(xmltype,"float");
        break;
      case SQLFLOAT:
        strcpy(xmltype,"double");
        break;
      case SQLMONEY:
      case SQLDECIMAL:
        strcpy(xmltype,"decimal");
        break;
      case SQLINT:
      case SQLSERIAL:
        strcpy(xmltype,"int");
        break;
      case SQLSMINT:
        strcpy(xmltype,"short");
        break;
      case SQLINT8:
      case SQLSERIAL8:
        strcpy(xmltype,"long");
        break;
      case SQLDATE:
        strcpy(xmltype,"dateTime");
        break;
      case SQLDTIME:
        strcpy(xmltype,"dateTime");
        break;
      case SQLINTERVAL:
        strcpy(xmltype,"duration");
        break;
      case SQLBOOL:
        strcpy(xmltype,"boolean");
        break;
      case SQLBYTES:
        strcpy(xmltype, "base64Binary");
        break;
#ifdef SQLUDTFIXED
      case SQLUDTFIXED:
        if (fexttype==XID_BLOB) {
          strcpy(xmltype, "base64Binary");
        } else {
          strcpy(xmltype,"string");
        }
        break;
#endif
      default:
        strcpy(xmltype,"string");
    }
    fprintf(f, "              <xs:element name=\"%s\" type=\"xs:%s\" minoccurs=\"%d\" />\n",
             sqlname, xmltype, nullable==1?0:1);

  }
  stat=fputs("            </xs:sequence>\n",f);
  stat=fputs("          </xs:complexType>\n",f);
  stat=fputs("        </xs:element>\n",f);
  stat=fputs("      </xs:choice>\n",f);
  stat=fputs("    </xs:complexType>\n",f);
  stat=fputs("  </xs:element>\n",f);
  stat=fputs("</xs:schema>\n",f);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(f));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  stat=fputs("<", (*handle));
  stat=fputs(rowset_token,(*handle));
  stat=fputs(">\n",(*handle));
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror((FILE *)(*handle)));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

int oninit_xsl1(char *fname, const char *tablename, int batchsize,
                void **handle, char *escapech, const void *logfile,
                int fieldsnum, char *slctstmt2, onlogerror_upt onlogerror,
                int verbose, const char *datahandleoptions)
{
  char errstr[ERR_BUF_SIZE];
  unsigned int stat;
  FILE * f;
  exec sql begin declare section;
  char *slctstmt;
  int fields_number;
  int i;
  char sqlname[200];
  exec sql end declare section;
  int len;
  char *cp;
  
  set_rowset_token("ROWSET");
  set_row_token("ROW");

  slctstmt=slctstmt2;
  fields_number=fieldsnum;

  EXEC SQL allocate descriptor 'desc1' WITH MAX :fields_number;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr, "Cannot allocate descriptor "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    return 1;
  }
  EXEC SQL prepare usqlobj4 from :slctstmt;
  if ( sqlca.sqlcode != 0 ) {
    sprintf(errstr,"Cannot prepare: "INT4_FORMAT"",sqlca.sqlcode);
    onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
               sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
    EXEC SQL deallocate descriptor 'desc1';
    return 1;
  }
				    
  EXEC SQL describe usqlobj4 using sql descriptor 'desc1';
  if (fname[0]==0) {
    (*handle)=stdout;
    if (strcmp("<table>",tablename)) {
      sprintf(errstr,"file=<stdout>, tablename=%s", tablename);
    } else {
      sprintf(errstr,"file=<stdout>\n");
    }
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  } else {
    if (strcmp("<table>",tablename)) {
      sprintf(errstr, "file=%s, tablename=%s\n",fname, tablename);
    } else {
      sprintf(errstr, "file=%s\n",fname);
    }
    (*handle) = fopen(fname,"wb");
    if (verbose>=10) {
      onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
                 verbose, 30);
    }
  }
  if ( (*handle) == NULL ) {
    sprintf(errstr,"Cannot open file %s:%d", fname, errno);
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno, logfile,
               verbose, 10);
    return 1;
  }
  f=(FILE *)(*handle);
  (*escapech)='\\';

  stat=fputs("<?xml version=\"1.0\" ?>\n",f);
  stat=fputs("  <xsl:stylesheet version=\"1.0\" xmlns:xsl= \"http://www.w3.org/1999/XSL/Transform\">\n",f);
  stat=fputs("  <xsl:output method=\"text\" />\n",f);
  stat=fputs("  <xsl:strip-space elements=\"*\" />\n",f);
  stat=fprintf(f,"  <xsl:template match=\"/%s\">\n", rowset_token);
  stat=fprintf(f,"    <xsl:for-each select=\"%s\">\n",row_token);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(f));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  for (i=1; i<=fields_number; i++) {
    EXEC SQL get descriptor 'desc1' VALUE :i
             :sqlname=NAME;
    if ( sqlca.sqlcode != 0 ) {
      sprintf(errstr, "Cannot get descriptor2 "INT4_FORMAT":%d",sqlca.sqlcode, i);
      onlogerror(__FILE__, __LINE__, errstr, sqlca.sqlcode, sqlca.sqlerrd[1],
                 sqlca.sqlerrm, 0, 0, 0, logfile, verbose, 10);
      EXEC SQL free usqlobj;
      EXEC SQL deallocate descriptor 'desc1';
      return 1;
    }
    len=stleng(sqlname);
    cp=sqlname+len-1;
    while(len>1 && *cp==' ') {
      *cp='\0';
      len--, cp--;
    }
    fprintf(f, "      <xsl:value-of select=\"%s\" />|<xsl:text/>\n", sqlname);

  }
  stat=fputs("      <xsl:text>&#xa;</xsl:text>\n",f);
  stat=fputs("    </xsl:for-each>\n",f);
  stat=fputs("  </xsl:template>\n",f);
  stat=fputs("</xsl:stylesheet>\n",f);
  if (stat==EOF) {
    sprintf(errstr, "OS error %d:%d",errno, ferror(f));
    onlogerror(__FILE__, __LINE__, errstr, 0, 0, NULL, 0, 0, errno,
               logfile, verbose, 10);
    return 1;
  }
  return 0;
}

